﻿#region Header
// Copyright 2007-2011, Jerónimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using NetJsWire.Lite.Widgets.UI;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Designers
{
    public class BaseComponentDesigner : ControlDesigner
    {
        private DesignerVerbCollection m_Verbs;
        // DesignerVerbCollection is overridden from ComponentDesigner
        public override DesignerVerbCollection Verbs
        {
            get
            {
                if ( m_Verbs == null )
                {
                    // Create and initialize the collection of verbs
                    m_Verbs = new DesignerVerbCollection();

                    m_Verbs.Add( new DesignerVerb( "Get JavaScript", OnGetJavaScript ) );
                    m_Verbs.Add( new DesignerVerb( "Get class JavaScript", OnGetClassJavaScript ) );
                }
                return m_Verbs;
            }
        }

        public void OnGetJavaScript( object sender, EventArgs e )
        {
            Clipboard.SetText( ( ( BaseComponent ) Component ).Serialize( Formatting.Indented ) );
            MessageBox.Show( "The Javascript code is on the clipboard", "System" );
        }

        public void OnGetClassJavaScript( object sender, EventArgs e )
        {
            Clipboard.SetText( ( ( BaseComponent ) Component ).SerializeClass( Formatting.Indented ) );
            MessageBox.Show( "The JavaScript class code is on the clipboard", "System" );
        }

        protected override void PostFilterEvents( IDictionary events )
        {
            var toRemove = new List< string > { "Click", "BackColorChanged" };

            foreach ( var evName in toRemove )
            {
                events.Remove( evName );
            }
            base.PostFilterEvents( events );
        }
    }
}
