﻿#region Header
// Copyright 2007-2011, Jerónimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using NetJsWire.Lite.Widgets.UI.Form;

namespace NetJsWire.Lite.Designers
{
    public class HiddenDesigner : BoxComponentDesigner
    {
        protected Pen BorderPen
        {
            get
            {
                var color = Color.DarkGray;
                var pen = new Pen( color ) { DashStyle = DashStyle.Dash };
                return pen;
            }
        }

        protected virtual void DrawBorder( Graphics graphics )
        {
            var component = ( Hidden ) base.Component;
            if ( component == null ) return;

            var borderPen = BorderPen;
            var clientRectangle = Control.DisplayRectangle;
            clientRectangle.Width--;
            clientRectangle.Height--;
            graphics.DrawRectangle( borderPen, clientRectangle );
            borderPen.Dispose();
        }

        protected override void OnPaintAdornments( PaintEventArgs pe )
        {
            if ( Control is Hidden )
            {
                DrawBorder( pe.Graphics );
            }
            base.OnPaintAdornments( pe );
        }
    }
}
