#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Windows.Forms;
using NetJsWire.Lite.Editors.ICSharpCode.Src.Actions;
using NetJsWire.Lite.Editors.ICSharpCode.Src.Document;
using NetJsWire.Lite.Editors.ICSharpCode.Src.Document.FoldingStrategy;
using NetJsWire.Lite.Editors.ICSharpCode.Src.Document.HighlightingStrategy;
using NetJsWire.Lite.Editors.ICSharpCode.Src.Gui;
using NetJsWire.Lite.Editors.ICSharpCode.Src.Gui.CompletionWindow;

namespace NetJsWire.Lite.Editors.TextEditor
{
    /// <summary>
    ///   Main form for a multi-file text editor based on 
    ///   ICSharpCode.TextEditor.TextEditorControl.
    /// </summary>
    public partial class TextEditorForm : Form
    {
        internal static TextEditorControl _editorControl;
        private CodeCompletionWindow _codeCompletionWindow;

        char[] _collected = new char[ 4 ];
        private int _indexCollected;

        public TextEditorForm()
        {
            InitializeComponent();
            _editorControl = textEditorControl1;  //AddNewTextEditor( "New file" );
            _editorSettings = _editorControl.TextEditorProperties;
            textEditorControl1.Document.HighlightingStrategy = HighlightingStrategyFactory.CreateHighlightingStrategy( "JavaScript" );

            _editorControl.ActiveTextAreaControl.TextArea.KeyEventHandler += TextArea_KeyEventHandler;
            _editorControl.ActiveTextAreaControl.TextArea.KeyDown += delegate( object sender, KeyEventArgs e )
                                                                         {
                                                                             if ( e.Control == false )
                                                                                 return;
                                                                             if ( e.KeyCode != Keys.Space )
                                                                                 return;
                                                                             e.SuppressKeyPress = true;
                                                                             ShowIntellisense( ' ' );
                                                                         };
        }

        bool TextArea_KeyEventHandler( char ch )
        {
            //_collected[ _indexCollected++ ] = ch;
            //if ( _indexCollected == 4 )
            //{
            //    ShowIntellisense( ch );
            //    _collected = new char[ 4 ];
            //    _indexCollected = 0;
            //    return true;
            //}
            if ( ch == '.' )
            {
                ShowIntellisense( ch );
            }
            return false;
        }

        private void ShowIntellisense( char value )
        {
            ICompletionDataProvider completionDataProvider = new CodeCompletionProvider( imageList1 );

            _codeCompletionWindow = CodeCompletionWindow.ShowCompletionWindow(
                this,                // The parent window for the completion window
                _editorControl, 	     // The text editor to show the window for
                "<code>",	       	     // Filename - will be passed back to the provider
                completionDataProvider,// Provider to get the list of possible completions
                value		     // Key pressed - will be passed to the provider
                , true, false );
            if ( _codeCompletionWindow != null )
            {
                // ShowCompletionWindow can return null when the provider returns an empty list
                _codeCompletionWindow.Closed += CloseCodeCompletionWindow;
            }
        }

        void CloseCodeCompletionWindow( object sender, EventArgs e )
        {
            if ( _codeCompletionWindow != null )
                _codeCompletionWindow.Dispose();
        }

        private void TextEditorForm_KeyDown( object sender, KeyEventArgs e )
        {
            if ( !e.Control )
                return;
            if ( e.KeyCode != Keys.Enter )
                return;
            EditorText = textEditorControl1.Text;
            DialogResult = DialogResult.OK;
            Close();
        }

        private void btnAccept_Click( object sender, EventArgs e )
        {
            EditorText = textEditorControl1.Text;
            DialogResult = DialogResult.OK;
            Close();
        }

        private void btnCancel_Click( object sender, EventArgs e )
        {
            Close();
        }

        private void undoToolStripMenuItem_Click( object sender, EventArgs e )
        {
            DoEditAction( ActiveEditor, new Undo() );
        }

        private void redoToolStripMenuItem_Click( object sender, EventArgs e )
        {
            DoEditAction( ActiveEditor, new Redo() );
        }

        #region Code related to File menu

        /// <summary>
        ///   This variable holds the settings (whether to show line numbers, 
        ///   etc.) that all editor controls share.
        /// </summary>
        ITextEditorProperties _editorSettings;

        private void menuFileNew_Click( object sender, EventArgs e )
        {
            AddNewTextEditor( "New file" );
        }

        private TextEditorControl AddNewTextEditor( string title )
        {
            //var tab = new TabPage( title );
            var editor = new TextEditorControl();
            editor.Dock = DockStyle.Fill;
            editor.IsReadOnly = false;
            editor.Document.DocumentChanged +=
                ( sender, e ) => { SetModifiedFlag( editor, true ); };
            // When a tab page gets the focus, move the focus to the editor control
            // instead when it gets the Enter (focus) event. I use BeginInvoke 
            // because changing the focus directly in the Enter handler doesn't 
            // work.
            //tab.Enter +=
            //    new EventHandler( ( sender, e ) =>
            //    {
            //        var page = ( ( TabPage ) sender );
            //        page.BeginInvoke( new Action<TabPage>( p => p.Controls[ 0 ].Focus() ), page );
            //    } );
            //tab.Controls.Add( editor );
            //fileTabs.Controls.Add( tab );

            if ( _editorSettings == null )
            {
                _editorSettings = editor.TextEditorProperties;
                OnSettingsChanged();
            }
            else
                editor.TextEditorProperties = _editorSettings;
            return editor;
        }

        private void menuFileOpen_Click( object sender, EventArgs e )
        {
            if ( openFileDialog.ShowDialog() == DialogResult.OK )
                // Try to open chosen file
                OpenFiles( openFileDialog.FileNames );
        }

        private void OpenFiles( string[] fns )
        {
            // Close default untitled document if it is still empty
            //if ( fileTabs.TabPages.Count == 1
            //    && ActiveEditor.Document.TextLength == 0
            //    && string.IsNullOrEmpty( ActiveEditor.FileName ) )
            //    RemoveTextEditor( ActiveEditor );

            // Open file(s)
            foreach ( var fn in fns )
            {
                var editor = textEditorControl1;
                try
                {
                    editor.LoadFile( fn );
                    // Modified flag is set during loading because the document 
                    // "changes" (from nothing to something). So, clear it again.
                    SetModifiedFlag( editor, false );
                }
                catch ( Exception ex )
                {
                    MessageBox.Show( ex.Message, ex.GetType().Name );
                    RemoveTextEditor( editor );
                    return;
                }

                // ICSharpCode.TextEditor doesn't have any built-in code folding
                // strategies, so I've included a simple one. Apparently, the
                // foldings are not updated automatically, so in this demo the user
                // cannot add or remove folding regions after loading the file.
                editor.Document.FoldingManager.FoldingStrategy = new RegionFoldingStrategy();
                editor.Document.FoldingManager.UpdateFoldings( fn, null );
            }
        }

        private void menuFileClose_Click( object sender, EventArgs e )
        {
            EditorText = textEditorControl1.Text;
            Close();
        }

        private void RemoveTextEditor( TextEditorControl editor )
        {
            ( editor.Parent.Parent ).Controls.Remove( editor.Parent );
        }

        private void menuFileSave_Click( object sender, EventArgs e )
        {
            var editor = ActiveEditor;
            if ( editor != null )
                DoSave( editor );
        }

        private bool DoSave( TextEditorControl editor )
        {
            if ( string.IsNullOrEmpty( editor.FileName ) )
                return DoSaveAs( editor );
            else
            {
                try
                {
                    editor.SaveFile( editor.FileName );
                    SetModifiedFlag( editor, false );
                    return true;
                }
                catch ( Exception ex )
                {
                    MessageBox.Show( ex.Message, ex.GetType().Name );
                    return false;
                }
            }
        }

        private void menuFileSaveAs_Click( object sender, EventArgs e )
        {
            var editor = ActiveEditor;
            if ( editor != null )
                DoSaveAs( editor );
        }

        private bool DoSaveAs( TextEditorControl editor )
        {
            saveFileDialog.FileName = editor.FileName;
            if ( saveFileDialog.ShowDialog() == DialogResult.OK )
            {
                try
                {
                    editor.SaveFile( saveFileDialog.FileName );
                    editor.Parent.Text = Path.GetFileName( editor.FileName );
                    SetModifiedFlag( editor, false );

                    // The syntax highlighting strategy doesn't change
                    // automatically, so do it manually.
                    editor.Document.HighlightingStrategy =
                        HighlightingStrategyFactory.CreateHighlightingStrategyForFile( editor.FileName );
                    return true;
                }
                catch ( Exception ex )
                {
                    MessageBox.Show( ex.Message, ex.GetType().Name );
                }
            }
            return false;
        }

        private void menuFileExit_Click( object sender, EventArgs e )
        {
            Close();
        }

        #endregion

        #region Code related to Edit menu

        readonly FindAndReplaceForm _findForm = new FindAndReplaceForm();

        /// <summary>
        ///   Performs an action encapsulated in IEditAction.
        /// </summary>
        /// <remarks>
        ///   There is an implementation of IEditAction for every action that 
        ///   the user can invoke using a shortcut key (arrow keys, Ctrl+X, etc.)
        ///   The editor control doesn't provide a public funciton to perform one
        ///   of these actions directly, so I wrote DoEditAction() based on the
        ///   code in TextArea.ExecuteDialogKey(). You can call ExecuteDialogKey
        ///   directly, but it is more fragile because it takes a Keys value (e.g.
        ///   Keys.Left) instead of the action to perform.
        ///   <para />
        ///   Clipboard commands could also be done by calling methods in
        ///   editor.ActiveTextAreaControl.TextArea.ClipboardHandler.
        /// </remarks>
        private void DoEditAction( TextEditorControl editor, IEditAction action )
        {
            if ( editor != null && action != null )
            {
                var area = editor.ActiveTextAreaControl.TextArea;
                editor.BeginUpdate();
                try
                {
                    lock ( editor.Document )
                    {
                        action.Execute( area );
                        if ( area.SelectionManager.HasSomethingSelected && area.AutoClearSelection /*&& caretchanged*/)
                        {
                            if ( area.Document.TextEditorProperties.DocumentSelectionMode == DocumentSelectionMode.Normal )
                            {
                                area.SelectionManager.ClearSelection();
                            }
                        }
                    }
                }
                finally
                {
                    editor.EndUpdate();
                    area.Caret.UpdateCaretPosition();
                }
            }
        }

        private void menuEditCut_Click( object sender, EventArgs e )
        {
            if ( HaveSelection() )
                DoEditAction( ActiveEditor, new Cut() );
        }
        private void menuEditCopy_Click( object sender, EventArgs e )
        {
            if ( HaveSelection() )
                DoEditAction( ActiveEditor, new Copy() );
        }
        private void menuEditPaste_Click( object sender, EventArgs e )
        {
            DoEditAction( ActiveEditor, new Paste() );
        }
        private void menuEditDelete_Click( object sender, EventArgs e )
        {
            if ( HaveSelection() )
                DoEditAction( ActiveEditor, new Delete() );
        }

        private bool HaveSelection()
        {
            var editor = ActiveEditor;
            return editor != null &&
                   editor.ActiveTextAreaControl.TextArea.SelectionManager.HasSomethingSelected;
        }

        private void menuEditFind_Click( object sender, EventArgs e )
        {
            var editor = ActiveEditor;
            if ( editor == null ) return;
            _findForm.ShowFor( editor, false );
        }

        private void menuEditReplace_Click( object sender, EventArgs e )
        {
            var editor = ActiveEditor;
            if ( editor == null ) return;
            _findForm.ShowFor( editor, true );
        }

        private void menuFindAgain_Click( object sender, EventArgs e )
        {
            _findForm.FindNext( true, false,
                                string.Format( "Search text {0} not found.", _findForm.LookFor ) );
        }
        private void menuFindAgainReverse_Click( object sender, EventArgs e )
        {
            _findForm.FindNext( true, true,
                                string.Format( "Search text {0} not found.", _findForm.LookFor ) );
        }

        private void menuToggleBookmark_Click( object sender, EventArgs e )
        {
            var editor = ActiveEditor;
            if ( editor != null )
            {
                DoEditAction( ActiveEditor, new ToggleBookmark() );
                editor.IsIconBarVisible = editor.Document.BookmarkManager.Marks.Count > 0;
            }
        }

        private void menuGoToNextBookmark_Click( object sender, EventArgs e )
        {
            DoEditAction( ActiveEditor, new GotoNextBookmark
                                            ( bookmark => true ) );
        }

        private void menuGoToPrevBookmark_Click( object sender, EventArgs e )
        {
            DoEditAction( ActiveEditor, new GotoPrevBookmark
                                            ( bookmark => true ) );
        }

        #endregion

        #region Code related to Options menu

        /// <summary>
        ///   Toggles whether the editor control is split in two parts.
        /// </summary>
        /// <remarks>
        ///   Exercise for the reader: modify TextEditorControl and
        ///   TextAreaControl so it shows a little "splitter stub" like you see in
        ///   other apps, that allows the user to split the text editor by dragging
        ///   it.
        /// </remarks>
        private void menuSplitTextArea_Click( object sender, EventArgs e )
        {
            var editor = ActiveEditor;
            if ( editor == null ) return;
            editor.Split();
        }

        /// <summary>
        ///   Show current settings on the Options menu
        /// </summary>
        /// <remarks>
        ///   We don't have to sync settings between the editors because 
        ///   they all share the same DefaultTextEditorProperties object.
        /// </remarks>
        private void OnSettingsChanged()
        {
            menuShowSpacesTabs.Checked = _editorSettings.ShowSpaces;
            menuShowNewlines.Checked = _editorSettings.ShowEOLMarker;
            menuHighlightCurrentRow.Checked = _editorSettings.LineViewerStyle == LineViewerStyle.FullRow;
            menuBracketMatchingStyle.Checked = _editorSettings.BracketMatchingStyle == BracketMatchingStyle.After;
            menuEnableVirtualSpace.Checked = _editorSettings.AllowCaretBeyondEOL;
            menuShowLineNumbers.Checked = _editorSettings.ShowLineNumbers;
        }

        private void menuShowSpaces_Click( object sender, EventArgs e )
        {
            var editor = ActiveEditor;
            if ( editor == null ) return;
            editor.ShowSpaces = editor.ShowTabs = !editor.ShowSpaces;
            OnSettingsChanged();
        }
        private void menuShowNewlines_Click( object sender, EventArgs e )
        {
            var editor = ActiveEditor;
            if ( editor == null ) return;
            editor.ShowEOLMarkers = !editor.ShowEOLMarkers;
            OnSettingsChanged();
        }

        private void menuHighlightCurrentRow_Click( object sender, EventArgs e )
        {
            var editor = ActiveEditor;
            if ( editor == null ) return;
            editor.LineViewerStyle = editor.LineViewerStyle == LineViewerStyle.None
                                         ? LineViewerStyle.FullRow : LineViewerStyle.None;
            OnSettingsChanged();
        }

        private void menuBracketMatchingStyle_Click( object sender, EventArgs e )
        {
            var editor = ActiveEditor;
            if ( editor == null ) return;
            editor.BracketMatchingStyle = editor.BracketMatchingStyle == BracketMatchingStyle.After
                                              ? BracketMatchingStyle.Before : BracketMatchingStyle.After;
            OnSettingsChanged();
        }

        private void menuEnableVirtualSpace_Click( object sender, EventArgs e )
        {
            var editor = ActiveEditor;
            if ( editor == null ) return;
            editor.AllowCaretBeyondEOL = !editor.AllowCaretBeyondEOL;
            OnSettingsChanged();
        }

        private void menuShowLineNumbers_Click( object sender, EventArgs e )
        {
            var editor = ActiveEditor;
            if ( editor == null ) return;
            editor.ShowLineNumbers = !editor.ShowLineNumbers;
            OnSettingsChanged();
        }

        private void menuSetTabSize_Click( object sender, EventArgs e )
        {
            if ( ActiveEditor != null )
            {
                var result = InputBox.Show( "Specify the desired tab width.", "Tab size", _editorSettings.TabIndent.ToString() );
                int value;
                if ( result != null && int.TryParse( result, out value ) && value.IsInRange( 1, 32 ) )
                {
                    ActiveEditor.TabIndent = value;
                }
            }
        }

        private void menuSetFont_Click( object sender, EventArgs e )
        {
            var editor = ActiveEditor;
            if ( editor != null )
            {
                fontDialog.Font = editor.Font;
                if ( fontDialog.ShowDialog( this ) == DialogResult.OK )
                {
                    editor.Font = fontDialog.Font;
                    OnSettingsChanged();
                }
            }
        }

        #endregion

        #region Other stuff

        private string _editorText;

        /// <summary>
        ///   Returns a list of all editor controls
        /// </summary>
        private IEnumerable<TextEditorControl> AllEditors
        {
            get
            {
                //return from t in fileTabs.Controls.Cast<TabPage>()
                //       from c in t.Controls.OfType<TextEditorControl>()
                //       select c;
                return null;
            }
        }

        /// <summary>
        ///   Returns the currently displayed editor, or null if none are open
        /// </summary>
        private TextEditorControl ActiveEditor
        {
            get
            {
                //if ( fileTabs.TabPages.Count == 0 ) return null;
                //return fileTabs.SelectedTab.Controls.OfType<TextEditorControl>().FirstOrDefault();
                return textEditorControl1;
            }
        }

        public string EditorText
        {
            get
            {
                if ( _editorText == "" )
                    _editorText = null;

                return _editorText;
            }
            set
            {
                _editorText = value;
                textEditorControl1.Text = value;
            }
        }

        private void TextEditor_FormClosing( object sender, FormClosingEventArgs e )
        {
            // Ask user to save changes
            //foreach ( var editor in AllEditors )
            //{
            //    if ( IsModified( editor ) )
            //    {
            //        var r = MessageBox.Show( string.Format( "Save changes to {0}?", editor.FileName ?? "new file" ),
            //            "Save?", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question );
            //        if ( r == DialogResult.Cancel )
            //            e.Cancel = true;
            //        else if ( r == DialogResult.Yes )
            //            if ( !DoSave( editor ) )
            //                e.Cancel = true;
            //    }
            //}
        }

        /// <summary>
        ///   Gets whether the file in the specified editor is modified.
        /// </summary>
        /// <remarks>
        ///   TextEditorControl doesn't maintain its own internal modified 
        ///   flag, so we use the '*' shown after the file name to represent the 
        ///   modified state.
        /// </remarks>
        private bool IsModified( TextEditorControl editor )
        {
            // TextEditorControl doesn't seem to contain its own 'modified' flag, so 
            // instead we'll treat the "*" on the filename as the modified flag.
            return editor.Parent.Text.EndsWith( "*" );
        }
        private void SetModifiedFlag( TextEditorControl editor, bool flag )
        {
            if ( IsModified( editor ) != flag )
            {
                var p = editor.Parent;
                if ( IsModified( editor ) )
                    p.Text = p.Text.Substring( 0, p.Text.Length - 1 );
                else
                    p.Text += "*";
            }
        }

        /// <summary>
        ///   We handle DragEnter and DragDrop so users can drop files on the editor.
        /// </summary>
        private void TextEditorForm_DragEnter( object sender, DragEventArgs e )
        {
            if ( e.Data.GetDataPresent( DataFormats.FileDrop ) )
                e.Effect = DragDropEffects.Copy;
        }
        private void TextEditorForm_DragDrop( object sender, DragEventArgs e )
        {
            var list = e.Data.GetData( DataFormats.FileDrop ) as string[];
            if ( list != null )
                OpenFiles( list );
        }

        #endregion
    }

    /// <summary>
    ///   The class to generate the foldings, it implements ICSharpCode.TextEditor.Document.IFoldingStrategy
    /// </summary>
    public class RegionFoldingStrategy : IFoldingStrategy
    {
        #region IFoldingStrategy Members

        /// <summary>
        ///   Generates the foldings for our document.
        /// </summary>
        /// <param name = "document">The current document.</param>
        /// <param name = "fileName">The filename of the document.</param>
        /// <param name = "parseInformation">Extra parse information, not used in this sample.</param>
        /// <returns>A list of FoldMarkers.</returns>
        public List<FoldMarker> GenerateFoldMarkers( IDocument document, string fileName, object parseInformation )
        {
            var list = new List<FoldMarker>();

            var startLines = new Stack<int>();

            // Create foldmarkers for the whole document, enumerate through every line.
            for ( var i = 0; i < document.TotalNumberOfLines; i++ )
            {
                var seg = document.GetLineSegment( i );
                int offs, end = document.TextLength;
                char c;
                for ( offs = seg.Offset; offs < end && ( ( c = document.GetCharAt( offs ) ) == ' ' || c == '\t' ); offs++ )
                { }
                if ( offs == end )
                    break;
                var spaceCount = offs - seg.Offset;

                // now offs points to the first non-whitespace char on the line
                if ( document.GetCharAt( offs ) == '{' || document.GetCharAt( offs ) == '}' )
                {
                    var text = document.GetText( offs, seg.Length - spaceCount );
                    if ( text.StartsWith( "{" ) )
                        startLines.Push( i );
                    if ( text.StartsWith( "}" ) && startLines.Count > 0 )
                    {
                        // Add a new FoldMarker to the list.
                        var start = startLines.Pop();
                        list.Add( new FoldMarker( document, start,
                            document.GetLineSegment( start ).Length,
                            i, spaceCount + "}".Length ) );
                    }
                }
            }

            return list;
        }

        #endregion
    }

    public class CodeCompletionProvider : ICompletionDataProvider
    {
        private readonly ImageList imageList;

        public CodeCompletionProvider( ImageList imageList )
        {
            this.imageList = imageList;
        }

        #region ICompletionDataProvider Members

        public ImageList ImageList
        {
            get
            {
                return imageList;
            }
        }

        public string PreSelection
        {
            get
            {
                return null;
            }
        }

        public int DefaultIndex
        {
            get
            {
                return -1;
            }
        }

        public CompletionDataProviderKeyResult ProcessKey( char key )
        {
            if ( char.IsLetterOrDigit( key ) || key == '_' )
            {
                return CompletionDataProviderKeyResult.NormalKey;
            }
            return CompletionDataProviderKeyResult.InsertionKey;
        }

        /// <summary>
        ///   Called when entry should be inserted. Forward to the insertion action of the completion data.
        /// </summary>
        public bool InsertAction( ICompletionData data, TextArea textArea, int insertionOffset, char key )
        {
            //var word = GetCurrentWord();
            textArea.Caret.Position = textArea.Document.OffsetToPosition(
                Math.Min( insertionOffset /*- ( !string.IsNullOrEmpty( word ) ? ( word.Length + 1 ) : 1 )*/, textArea.Document.TextLength )
                );
            return data.InsertAction( textArea, key );
        }

        public ICompletionData[] GenerateCompletionData( string fileName, TextArea textArea, char charTyped )
        {
            var currWord = GetCurrentWord();
            var splliters = new[] { " ", ",", ";", ".", ":", "(", ")", "[", "]", "{", "}", "\"", "'", "!", "|", "||", "&", "&&", "=", "==", "===", "?", "", "!=", "!==", "|==", "", Environment.NewLine };

            var basic = new List<ICompletionData> {
                                new DefaultCompletionData("function", "Simple function", 0),
                                new DefaultCompletionData("Ext", "Ext", 0),
                                new DefaultCompletionData("var", "Variable declaration", 1)
                            };
            var splitted = textArea.TextView.Document.TextContent.Split( splliters, StringSplitOptions.RemoveEmptyEntries );
            foreach ( var s in splitted.Where( s => !s.Any( c => "=_0123456789/*-+".Contains( c ) ) && s.Length > 2 && s != "Ext" ).Distinct().OrderBy( s => s ) )
            {
                if ( !string.IsNullOrEmpty( s.Trim() ) && !basic.Any( b => b.Text.Equals( "Ext,function,var".Split( ',' ).Any() ) ) )
                    basic.Add( new DefaultCompletionData( s, s, 0 ) );
            }

            if ( currWord == "Ext" || currWord == "Ext." )
            {
                basic.Add( new DefaultCompletionData( "getCmp", "(id)\nGet component via ComponentMgr", 0 ) );
                basic.Add( new DefaultCompletionData( "StoreMgr.get", "Get a store via id", 0 ) );
                basic.Add( new DefaultCompletionData( "override", "Adds a list of functions to the prototype of an existing class, overwriting any existing methods with the same name.", 0 ) );
                basic.Add( new DefaultCompletionData( "extend", "Since Core version 4 this method is meant to be used internally only. Use Ext.define instead.", 0 ) );
                basic.Add( new DefaultCompletionData( "grid", "grid namespace.", 0 ) );
                basic.Add( new DefaultCompletionData( "list", "list namespace.", 0 ) );
                basic.Add( new DefaultCompletionData( "dd", "dd namespace.", 0 ) );
                basic.Add( new DefaultCompletionData( "tree", "tree namespace.", 0 ) );
                basic.Add( new DefaultCompletionData( "form", "form namespace.", 0 ) );
                basic.Add( new DefaultCompletionData( "menu", "menu namespace.", 0 ) );
                basic.Add( new DefaultCompletionData( "state", "state namespace.", 0 ) );
                basic.Add( new DefaultCompletionData( "layout", "layout namespace.", 0 ) );
                basic.Add( new DefaultCompletionData( "app", "app namespace.", 0 ) );
                basic.Add( new DefaultCompletionData( "ux", "ux namespace.", 0 ) );
                basic.Add( new DefaultCompletionData( "chart", "chart namespace.", 0 ) );
                basic.Add( new DefaultCompletionData( "direct", "direct namespace.", 0 ) );
            }

            return basic.ToArray();
        }

        #endregion

        public string GetCurrentWord()
        {
            var pos = TextEditorForm._editorControl.ActiveTextAreaControl.Caret.Offset;
            var word = GetWordAt( TextEditorForm._editorControl.Document, pos );
            if ( word.Length == 0 && ( TextEditorForm._editorControl.Text.Length > pos - 1 ) )
                word = GetWordAt( TextEditorForm._editorControl.Document, pos - 1 );

            return word.Trim();
        }

        public static string GetWordAt( IDocument document, int offset )
        {
            if ( offset < 0 || offset > document.TextLength || !IsWordPart( document.GetCharAt( offset - 1 ) ) )
            {
                return String.Empty;
            }
            var startOffset = offset;
            var endOffset = offset;
            while ( startOffset > 0 && IsWordPart( document.GetCharAt( startOffset - 1 ) ) )
            {
                --startOffset;
            }

            while ( endOffset < document.TextLength - 1 && IsWordPart( document.GetCharAt( endOffset + 1 ) ) )
            {
                ++endOffset;
            }

            //Debug.Assert(endOffset >= startOffset);
            return document.GetText( startOffset, endOffset - startOffset );
        }

        public static string GetPreviousWordAt( IDocument document, int offset )
        {
            var startOffset = offset;
            //int endOffset = offset;
            while ( startOffset > 0 && IsWordPart( document.GetCharAt( startOffset - 1 ) ) )
            {
                startOffset--;
            }
            startOffset -= 2;
            return GetWordAt( document, startOffset );

        }

        public static bool IsLetterDigitOrUnderscore( char c )
        {
            if ( !Char.IsLetterOrDigit( c ) )
            {
                return c == '_';
            }
            return true;
        }

        static bool IsWordPart( char ch )
        {
            return IsLetterDigitOrUnderscore( ch ); // || ch == '.';
        }
    }
}