﻿#region Header
// Copyright 2007-2011, Jerónimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.ComponentModel;
using NetJsWire.Lite.Widgets.UI.Util;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Data
{
    [JsonObject( MemberSerialization = MemberSerialization.OptIn )]
    public abstract class DataReader : BaseObservable
    {
        private FieldCollection _fields = new FieldCollection();

        ///<summary>
        ///  Initializes a new instance of DataReader.
        ///</summary>
        public DataReader()
            : this( null )
        {
        }

        ///<summary>
        ///  Initializes a new instance of DataReader and adds it to the Container.
        ///</summary>
        ///<param name = "container"></param>
        public DataReader( IContainer container )
        {
            if ( container != null )
                container.Add( this );
        }

        [JsonProperty( "fields" )]
        [DesignerSerializationVisibility( DesignerSerializationVisibility.Content )]
        public virtual FieldCollection Fields
        {
            get { return _fields; }
            set { _fields = value; }
        }

        [JsonProperty( "messageProperty" )]
        [DefaultValue( ( string ) null )]
        public string MessageProperty { get; set; }

        public bool ShouldSerializeFields()
        {
            return Fields != null && Fields.Count > 0;
        }

        protected void ResetFields()
        {
            Fields.Clear();
        }
    }
}
