#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Dd
{
    /// <summary>
    ///   /**
    ///   A DragDrop implementation that inserts an empty, bordered div into
    ///   the document that follows the cursor during drag operations.  At the time of
    ///   the click, the frame div is resized to the dimensions of the linked html
    ///   element, and moved to the exact location of the linked element.
    ///   References to the "frame" element refer to the single proxy element that
    ///   was created to be dragged in place of all DDProxy elements on the
    ///   page.
    /// </summary>
    public class DDProxy : DD
    {
        /// <summary>
        ///   The default drag frame div id
        /// </summary>
        [JsonProperty( "Ext" )]
        public static string Ext { get; set; }

        /// <summary>
        ///   By default we resize the drag frame to be the same size as the element
        ///   we want to drag (this is to get the frame effect).  We can turn it off
        ///   if we want a different behavior.
        /// </summary>
        [JsonProperty( "resizeFrame" )]
        public bool ResizeFrame { get; set; }

        /// <summary>
        ///   By default the frame is positioned exactly where the drag element is, so
        ///   we use the cursor offset provided by Ext.dd.DD.  Another option that works only if
        ///   you do not have constraints on the obj is to have the drag frame centered
        ///   around the cursor.  Set centerFrame to true for this effect.
        /// </summary>
        [JsonProperty( "centerFrame" )]
        public bool CenterFrame { get; set; }
    }
}
