#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Dd
{
    /// <summary>
    ///   /**
    ///   A simple class that provides the basic implementation needed to make any element draggable.
    /// </summary>
    public class DragSource : DDProxy
    {
        /// <summary>
        ///   A named drag drop group to which this object belongs.  If a group is specified, then this object will only
        ///   interact with other drag drop objects in the same group (defaults to undefined).
        /// </summary>
        [JsonProperty( "ddGroup" )]
        public virtual string DdGroup { get; set; }

        /// <summary>
        ///   The CSS class returned to the drag source when drop is allowed (defaults to "x-dd-drop-ok").
        /// </summary>
        [JsonProperty( "dropAllowed" )]
        public string DropAllowed { get; set; }

        /// <summary>
        ///   The CSS class returned to the drag source when drop is not allowed (defaults to "x-dd-drop-nodrop").
        /// </summary>
        [JsonProperty( "dropNotAllowed" )]
        public string DropNotAllowed { get; set; }

        /// <summary>
        ///   An empty function by default, but provided so that you can perform a custom action once the initial
        ///   drag event has begun.  The drag cannot be canceled from this function.
        ///   @param {Number} x The x position of the click on the dragged object
        ///   @param {Number} y The y position of the click on the dragged object
        /// </summary>
        [JsonProperty( "onStartDrag" )]
        public object OnStartDrag { get; set; }
    }
}
