#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using NetJsWire.Lite.Widgets.UI.Util;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Dd
{
    /// <summary>
    ///   /**
    ///   A DragTracker listens for drag events on an Element and fires events at the start and end of the drag,
    ///   as well as during the drag. This is useful for components such as {@link Ext.slider.MultiSlider}, where there is
    ///   an element that can be dragged around to change the Slider's value.
    ///   DragTracker provides a series of template methods that should be overridden to provide functionality
    ///   in response to detected drag operations. These are onBeforeStart, onStart, onDrag and onEnd.
    ///   See {@link Ext.slider.MultiSlider}'s initEvents function for an example implementation.
    ///   */
    ///   Ext.dd.DragTracker = Ext.extend(Ext.util.Observable,  {
    /// </summary>
    public class DragTracker : BaseObservable
    {
        /// <summary>
        ///   Defaults to <tt>false</tt>.
        /// </summary>
        [JsonProperty( "active" )]
        public bool Active { get; set; }

        /// <summary>
        ///   Number of pixels the drag target must be moved before dragging is considered to have started. Defaults to <tt>5</tt>.
        /// </summary>
        [JsonProperty( "tolerance" )]
        public double Tolerance { get; set; }

        /// <summary>
        ///   Defaults to <tt>false</tt>. Specify <tt>true</tt> to defer trigger start by 1000 ms.
        ///   Specify a Number for the number of milliseconds to defer trigger start.
        /// </summary>
        [JsonProperty( "autoStart" )]
        public object AutoStart { get; set; }
    }
}
