#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion
namespace NetJsWire.Lite.Widgets.UI.Dd
{
    /// <summary>
    ///   /**
    ///   <p>This class provides a container DD instance that allows dropping on multiple child target nodes.</p>
    ///   <p>By default, this class requires that child nodes accepting drop are registered with {@link Ext.dd.Registry}.
    ///     However a simpler way to allow a DropZone to manage any number of target elements is to configure the
    ///     DropZone with an implementation of {@link #getTargetFromEvent} which interrogates the passed
    ///     mouse event to see if it has taken place within an element, or class of elements. This is easily done
    ///     by using the event's {@link Ext.EventObject#getTarget getTarget} method to identify a node based on a
    ///     {@link Ext.DomQuery} selector.</p>
    ///   <p>Once the DropZone has detected through calling getTargetFromEvent, that the mouse is over
    ///     a drop target, that target is passed as the first parameter to {@link #onNodeEnter}, {@link #onNodeOver},
    ///     {@link #onNodeOut}, {@link #onNodeDrop}. You may configure the instance of DropZone with implementations
    ///     of these methods to provide application-specific behaviour for these events to update both
    ///     application state, and UI state.</p>
    ///   <p>For example to make a GridPanel a cooperating target with the example illustrated in
    ///     {@link Ext.dd.DragZone DragZone}, the following technique might be used:</p><pre><code>
    ///                                                                                        myGridPanel.on('render', function() {
    ///                                                                                        myGridPanel.dropZone = new Ext.dd.DropZone(myGridPanel.getView().scroller, {
    ///                                                                                        //      If the mouse is over a grid row, return that node. This is
    ///                                                                                        //      provided as the "target" parameter in all "onNodeXXXX" node event handling functions
    ///                                                                                        getTargetFromEvent: function(e) {
    ///                                                                                        return e.getTarget(myGridPanel.getView().rowSelector);
    ///                                                                                        },
    ///                                                                                        //      On entry into a target node, highlight that node.
    ///                                                                                        onNodeEnter : function(target, dd, e, data){
    ///                                                                                        Ext.fly(target).addClass('my-row-highlight-class');
    ///                                                                                        },
    ///                                                                                        //      On exit from a target node, unhighlight that node.
    ///                                                                                        onNodeOut : function(target, dd, e, data){
    ///                                                                                        Ext.fly(target).removeClass('my-row-highlight-class');
    ///                                                                                        },
    ///                                                                                        //      While over a target node, return the default drop allowed class which
    ///                                                                                        //      places a "tick" icon into the drag proxy.
    ///                                                                                        onNodeOver : function(target, dd, e, data){
    ///                                                                                        return Ext.dd.DropZone.prototype.dropAllowed;
    ///                                                                                        },
    ///                                                                                        //      On node drop we can interrogate the target to find the underlying
    ///                                                                                        //      application object that is the real target of the dragged data.
    ///                                                                                        //      In this case, it is a Record in the GridPanel's Store.
    ///                                                                                        //      We can use the data set up by the DragZone's getDragData method to read
    ///                                                                                        //      any data we decided to attach in the DragZone's getDragData method.
    ///                                                                                        onNodeDrop : function(target, dd, e, data){
    ///                                                                                        var rowIndex = myGridPanel.getView().findRowIndex(target);
    ///                                                                                        var r = myGridPanel.getStore().getAt(rowIndex);
    ///                                                                                        Ext.Msg.alert('Drop gesture', 'Dropped Record id ' + data.draggedRecord.id +
    ///                                                                                        ' on Record id ' + r.id);
    ///                                                                                        return true;
    ///                                                                                        }
    ///                                                                                        });
    ///                                                                                        }
    ///                                                                                      </code></pre>
    ///   See the {@link Ext.dd.DragZone DragZone} documentation for details about building a DragZone which
    ///   cooperates with this DropZone.
    /// </summary>
    public class DropZone : DropTarget
    {
    }
}
