#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Dd
{
    /// <summary>
    ///   /**
    ///   <p>Provides automatic scrolling of overflow regions in the page during drag operations.</p>
    ///   <p>The ScrollManager configs will be used as the defaults for any scroll container registered with it,
    ///     but you can also override most of the configs per scroll container by adding a
    ///     <tt>ddScrollConfig</tt> object to the target element that contains these properties: {@link #hthresh},
    ///     {@link #vthresh}, {@link #increment} and {@link #frequency}.  Example usage:
    ///     <pre><code>
    ///            var el = Ext.get('scroll-ct');
    ///            el.ddScrollConfig = {
    ///            vthresh: 50,
    ///            hthresh: -1,
    ///            frequency: 100,
    ///            increment: 200
    ///            };
    ///            Ext.dd.ScrollManager.register(el);
    ///          </code></pre>
    ///     <b>Note: This class uses "Point Mode" and is untested in "Intersect Mode".</b>
    ///     */
    ///     Ext.dd.ScrollManager = function(){
    /// </summary>
    public class ScrollManager
    {
        /// <summary>
        ///   The number of pixels from the top or bottom edge of a container the pointer needs to be to
        ///   trigger scrolling (defaults to 25)
        /// </summary>
        [JsonProperty( "vthresh" )]
        public static float Vthresh { get; set; }

        /// <summary>
        ///   The number of pixels from the right or left edge of a container the pointer needs to be to
        ///   trigger scrolling (defaults to 25)
        /// </summary>
        [JsonProperty( "hthresh" )]
        public static float Hthresh { get; set; }

        /// <summary>
        ///   The number of pixels to scroll in each scroll increment (defaults to 100)
        /// </summary>
        [JsonProperty( "increment" )]
        public static float Increment { get; set; }

        /// <summary>
        ///   The frequency of scrolls in milliseconds (defaults to 500)
        /// </summary>
        [JsonProperty( "frequency" )]
        public static float Frequency { get; set; }

        /// <summary>
        ///   True to animate the scroll (defaults to true)
        /// </summary>
        [JsonProperty( "animate" )]
        public static bool Animate { get; set; }

        /// <summary>
        ///   The animation duration in seconds -
        ///   MUST BE less than Ext.dd.ScrollManager.frequency! (defaults to .4)
        /// </summary>
        [JsonProperty( "animDuration" )]
        public static float AnimDuration { get; set; }

        /// <summary>
        ///   The named drag drop {@link Ext.dd.DragSource#ddGroup group} to which this container belongs (defaults to undefined).
        ///   If a ddGroup is specified, then container scrolling will only occur when a dragged object is in the same ddGroup.
        /// </summary>
        [JsonProperty( "ddGroup" )]
        public static string DdGroup { get; set; }
    }
}
