#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.Collections;
using System.ComponentModel;
using NetJsWire.Lite.Widgets.UI.Form;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI
{
    /// <summary>
    ///   /**
    ///   A base editor field that handles displaying/hiding on demand and has some built-in sizing and event handling logic.
    /// </summary>
    [ToolboxItem( false )]
    public class Editor : BaseComponent
    {
        /// <summary>
        ///   The Field object (or descendant) or config object for field
        /// </summary>
        [JsonProperty( "field" )]
        public FieldLW Field { get; set; }

        /// <summary>
        ///   True to {@link #completeEdit complete the editing process} if in edit mode when the
        ///   field is blurred. Defaults to <tt>true</tt>.
        /// </summary>
        [JsonProperty( "allowBlur" )]
        public bool AllowBlur { get; set; }

        /// <summary>
        ///   True for the editor to automatically adopt the size of the underlying field, "width" to adopt the width only,
        ///   or "height" to adopt the height only, "none" to always use the field dimensions. (defaults to false)
        /// </summary>
        [JsonProperty( "autoSize" )]
        public new string AutoSize { get; set; }

        /// <summary>
        ///   True to automatically revert the field value and cancel the edit when the user completes an edit and the field
        ///   validation fails (defaults to true)
        /// </summary>
        [JsonProperty( "revertInvalid" )]
        public bool RevertInvalid { get; set; }

        /// <summary>
        ///   True to skip the edit completion process (no save, no events fired) if the user completes an edit and
        ///   the value has not changed (defaults to false).  Applies only to string values - edits for other data types
        ///   will never be ignored.
        /// </summary>
        [JsonProperty( "ignoreNoChange" )]
        public bool IgnoreNoChange { get; set; }

        /// <summary>
        ///   False to keep the bound element visible while the editor is displayed (defaults to true)
        /// </summary>
        [JsonProperty( "hideEl" )]
        public virtual bool HideEl { get; set; }

        /// <summary>
        ///   The data value of the underlying field (defaults to "")
        /// </summary>
        [JsonProperty( "value" )]
        public object Value { get; set; }

        /// <summary>
        ///   The position to align to (see {@link Ext.Element#alignTo} for more details, defaults to "c-c?").
        /// </summary>
        [JsonProperty( "alignment" )]
        public virtual string Alignment { get; set; }

        /// <summary>
        ///   The offsets to use when aligning (see {@link Ext.Element#alignTo} for more details. Defaults to <tt>[0, 0]</tt>.
        /// </summary>
        [JsonProperty( "offsets" )]
        public IList Offsets { get; set; }

        /// <summary>
        ///   "sides" for sides/bottom only, "frame" for 4-way shadow, and "drop"for bottom-right shadow (defaults to "frame")
        /// </summary>
        [JsonProperty( "shadow" )]
        public string Shadow { get; set; }

        /// <summary>
        ///   True to constrain the editor to the viewport
        /// </summary>
        [JsonProperty( "constrain" )]
        public bool Constrain { get; set; }

        /// <summary>
        ///   Handle the keydown/keypress events so they don't propagate (defaults to true)
        /// </summary>
        [JsonProperty( "swallowKeys" )]
        public bool SwallowKeys { get; set; }

        /// <summary>
        ///   True to complete the edit when the enter key is pressed. Defaults to <tt>true</tt>.
        /// </summary>
        [JsonProperty( "completeOnEnter" )]
        public bool CompleteOnEnter { get; set; }

        /// <summary>
        ///   True to cancel the edit when the escape key is pressed. Defaults to <tt>true</tt>.
        /// </summary>
        [JsonProperty( "cancelOnEsc" )]
        public bool CancelOnEsc { get; set; }

        /// <summary>
        ///   True to update the innerHTML of the bound element when the update completes (defaults to false)
        /// </summary>
        [JsonProperty( "updateEl" )]
        public bool UpdateEl { get; set; }
    }
}
