#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.ComponentModel;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Grid
{
    /// <summary>
    ///   /**
    ///   <p>A Column definition class which renders a numeric data field according to a {@link #format} string.  See the
    ///     {@link Ext.grid.Column#xtype xtype} config option of {@link Ext.grid.Column} for more details.</p>
    ///   */
    ///   Ext.grid.NumberColumn = Ext.extend(Ext.grid.Column, {
    /// </summary>
    [Serializable]
    public class NumberColumn : Column
    {
        internal NumberColumn()
            : this( null )
        {
        }

        public NumberColumn( ColumnModel model )
            : base( model )
        {
            Xtype = "numbercolumn";
        }

        /// <summary>
        ///   A formatting string as used by {@link Ext.util.Format#number} to format a numeric value for this Column
        ///   (defaults to <tt>'0,000.00'</tt>).
        /// </summary>
        [JsonProperty( "format" )]
        [DefaultValue( ( string ) null )]
        public string Format { get; set; }
    }
}
