#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Drawing.Design;
using NetJsWire.Lite.Converters;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Grid
{
    /// <summary>
    ///   /**
    ///   <p>A Column definition class which renders a value by processing a {@link Ext.data.Record Record}'s
    ///     {@link Ext.data.Record#data data} using a {@link #tpl configured} {@link Ext.XTemplate XTemplate}.
    ///     See the {@link Ext.grid.Column#xtype xtype} config option of {@link Ext.grid.Column} for more
    ///     details.</p>
    ///   */
    ///   Ext.grid.TemplateColumn = Ext.extend(Ext.grid.Column, {
    /// </summary>
    [Serializable]
    public class TemplateColumn : Column
    {
        internal TemplateColumn()
            : this( null )
        {
        }

        public TemplateColumn( ColumnModel model )
            : base( model )
        {
            Xtype = "templatecolumn";
        }

        [JsonProperty( "tpl" )]
        [DefaultValue( ( string ) null )]
        [JsonConverter( typeof( NotQuotedJsonStringConverter ) )]
        [Description( "An XTemplate, or an XTemplate definition string to use to process a Record's data to produce a column's rendered value." )]
        [Editor( typeof( MultilineStringEditor ), typeof( UITypeEditor ) )]
        public string Tpl { get; set; }
    }
}
