﻿#region Header
// Copyright 2007-2011, Jerónimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.Drawing;
using System.Linq;
using System.Windows.Forms;
using System.Windows.Forms.Layout;

namespace NetJsWire.Lite.Widgets.UI.Layout
{
    public class AnchorLayoutEngine : LayoutEngine
    {
        public override bool Layout( object container, LayoutEventArgs layoutEventArgs )
        {
            var impContainer = container as Container;
            if ( impContainer != null )
            {
                if ( impContainer.Layout == LayoutType.Anchor || impContainer.Layout == LayoutType.Absolute )
                {
                    var leftPadding = 0;
                    var topPadding = 0;
                    var parentLeftPadding = 0;
                    var parentTopPadding = 0;
                    if ( impContainer is Panel )
                    {
                        parentLeftPadding = ( impContainer as Panel ).Padding.Left;
                        parentTopPadding = ( impContainer as Panel ).Padding.Top;
                        topPadding += parentTopPadding;
                    }

                    var controls = impContainer.Items != null ? impContainer.Items.Cast<BaseComponent>() : null;
                    if ( controls != null )
                    {
                        foreach ( var control in controls )
                        {
                            var anchorRight = -5555;
                            var anchorBottom = -5555;

                            if ( control is BoxComponent )
                            {
                                if ( !string.IsNullOrWhiteSpace( ( control as BoxComponent ).Anchor ) )
                                {
                                    var anchors = ( control as BoxComponent ).Anchor.Split( ' ' );
                                    if ( !anchors[ 0 ].Contains( "%" ) )
                                        anchorRight = Convert.ToInt32( anchors[ 0 ] );
                                    else
                                    {
                                        var anchorRightPercent = Convert.ToInt32( anchors[ 0 ].Replace( "%", "" ) );
                                        anchorRight = -( ( impContainer.Width - ( 5 + parentLeftPadding ) ) / 100 * anchorRightPercent );
                                    }

                                    if ( anchors.Length > 1 )
                                    {
                                        if ( !anchors[ 1 ].Contains( "%" ) )
                                            anchorBottom = Convert.ToInt32( anchors[ 1 ] );
                                        else
                                        {
                                            var anchorBottomPercent = Convert.ToInt32( anchors[ 1 ].Replace( "%", "" ) );
                                            anchorBottom = -( impContainer.Height / 100 * anchorBottomPercent );
                                        }
                                    }
                                }
                            }

                            if ( impContainer.Layout == LayoutType.Anchor )
                                control.Location = new Point( leftPadding, topPadding );

                            var width = control.Size.Width;
                            var height = control.Size.Height;

                            if ( anchorRight != -5555 )
                            {
                                width = impContainer.ClientSize.Width - leftPadding + anchorRight;
                            }
                            if ( anchorBottom != -5555 )
                            {
                                height = impContainer.ClientSize.Height - parentTopPadding + anchorBottom;
                            }

                            if ( impContainer.Layout == LayoutType.Absolute )
                            {
                                if ( anchorRight != -5555 )
                                    width -= control.Location.X;

                                if ( anchorBottom != -5555 )
                                    height -= control.Location.Y;
                            }

                            if ( control.Size.Width != width || control.Size.Height != height )
                                control.Size = new Size( width, height );
                            topPadding += control.Size.Height + 5;
                            //leftPadding = 0;
                        }
                    }
                }
            }
            return base.Layout( container, layoutEventArgs );
        }
    }
}
