﻿#region Header
// Copyright 2007-2011, Jerónimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.Drawing;
using System.Linq;
using System.Windows.Forms;
using System.Windows.Forms.Layout;

namespace NetJsWire.Lite.Widgets.UI.Layout
{
    public class BorderLayoutEngine : LayoutEngine
    {
        public override bool Layout( object container, LayoutEventArgs layoutEventArgs )
        {
            var impContainer = container as Container;
            if ( impContainer != null )
            {
                if ( impContainer.Layout == LayoutType.Border )
                {
                    var controls = impContainer.Items;
                    if ( controls != null )
                    {
                        var leftPadding = 0;
                        var topPadding = 0;
                        var eastLeft = 0;
                        var totalHeight = impContainer.ClientSize.Height;
                        var totalWidth = impContainer.ClientSize.Width;
                        var centerHeight = totalHeight;
                        var centerWidth = totalWidth;
                        var centerLeft = 0;
                        var southTop = 0;
                        var index = 0;
                        var northPanel = controls.OfType<BoxComponent>().FirstOrDefault( c => c.Region == "north" );
                        var westPanel = controls.OfType<BoxComponent>().FirstOrDefault( c => c.Region == "west" );
                        var centerPanel = controls.OfType<BoxComponent>().FirstOrDefault( c => c.Region == "center" );
                        var eastPanel = controls.OfType<BoxComponent>().FirstOrDefault( c => c.Region == "east" );
                        var southPanel = controls.OfType<BoxComponent>().FirstOrDefault( c => c.Region == "south" );

                        centerHeight -= northPanel != null ? northPanel.Height : 0;
                        centerHeight -= southPanel != null ? southPanel.Height : 0;

                        centerWidth -= westPanel != null ? westPanel.Width : 0;
                        centerWidth -= eastPanel != null ? eastPanel.Width : 0;
                        centerLeft += westPanel != null ? westPanel.Width : 0;

                        topPadding += northPanel != null ? northPanel.Height : 0;

                        if ( northPanel != null )
                        {
                            northPanel.Location = new Point( 0, 0 );
                            northPanel.Width = totalWidth;
                        }
                        if ( westPanel != null )
                        {
                            westPanel.Location = new Point( 0, topPadding );
                            westPanel.Height = centerHeight;
                        }
                        if ( centerPanel != null )
                        {
                            centerPanel.Location = new Point( centerLeft, topPadding );
                            centerPanel.Height = centerHeight;
                            centerPanel.Width = centerWidth;
                        }
                        if ( eastPanel != null )
                        {
                            eastLeft += westPanel != null ? westPanel.Width : 0;
                            eastLeft += centerPanel != null ? centerPanel.Width : 0;
                            eastPanel.Location = new Point( eastLeft, topPadding );
                            eastPanel.Height = centerHeight;
                        }
                        if ( southPanel != null )
                        {
                            southPanel.Location = new Point( 0, topPadding + centerHeight );
                            southPanel.Width = totalWidth;
                        }

                        return false;
                    }
                }
            }
            return base.Layout( container, layoutEventArgs );
        }
    }
}
