﻿#region Header
// Copyright 2007-2011, Jerónimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.Drawing;
using System.Linq;
using System.Windows.Forms;
using System.Windows.Forms.Layout;

namespace NetJsWire.Lite.Widgets.UI.Layout
{
    public class CardLayoutEngine : LayoutEngine
    {
        public override bool Layout( object container, LayoutEventArgs layoutEventArgs )
        {
            var impContainer = container as Container;
            if ( impContainer != null )
            {
                if ( impContainer.Layout == LayoutType.Card )
                {
                    var controls = impContainer.Items;
                    if ( controls != null )
                    {
                        var index = 0;
                        foreach ( var control in controls.OfType<Panel>() )
                        {
                            if ( control.Location.X != 0 || control.Location.Y != 0 )
                                control.Location = new Point( 0, 0 );
                            if ( index == impContainer.ActiveItem )
                            {
                                if ( !control.Size.Equals( impContainer.ClientSize ) )
                                    control.Size = impContainer.ClientSize;
                            }
                            else
                            {
                                if ( control.Size.Width != 0 || control.Size.Height != 0 )
                                    control.Size = new Size( 0, 0 );
                            }
                            if ( control.Border )
                                control.Border = false;
                            index++;
                        }

                        return false;
                    }
                }
            }
            return base.Layout( container, layoutEventArgs );
        }
    }
}
