﻿#region Header
// Copyright 2007-2011, Jerónimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.Drawing;
using System.Linq;
using System.Windows.Forms;
using System.Windows.Forms.Layout;

namespace NetJsWire.Lite.Widgets.UI.Layout
{
    public class ColumnLayoutEngine : LayoutEngine
    {
        public override bool Layout( object container, LayoutEventArgs layoutEventArgs )
        {
            var impContainer = container as Container;
            if ( impContainer != null )
            {
                if ( impContainer.Layout == LayoutType.Column )
                {
                    var controls = impContainer.Items;
                    if ( controls != null )
                    {
                        var height = impContainer.Height - ( impContainer.BorderWidth * 2 ) - ( ( impContainer as Panel ).DrawButtonsBar ? Panel.ButtonsBarHeight : 0 );
                        var containers = controls.OfType<Container>();
                        if ( containers == null )
                            return true;

                        const int topPadding = 0;
                        var leftPadding = 0;
                        var totalWidth = ( float ) impContainer.ClientSize.Width - containers.Where( c => c.ColumnWidth == 0 ).Sum( c => c.Width );
                        foreach ( var control in containers.Where( c => c.ColumnWidth == 0f ) )
                        {
                            control.Location = new Point( leftPadding, topPadding );
                            leftPadding += control.Width;
                        }

                        foreach ( var control in containers.Where( c => c.ColumnWidth != 0f ) )
                        {
                            control.Size = new Size( ( int ) ( totalWidth * control.ColumnWidth ), height );
                            //control.Width = ( int ) ( totalWidth * control.ColumnWidth );// -control.BorderWidth;
                            //control.Height = height;
                            control.Location = new Point( leftPadding, topPadding );
                            leftPadding += control.Width;// -( control.BorderWidth * 2 );
                        }

                        return false;
                    }
                }
            }
            return base.Layout( container, layoutEventArgs );
        }
    }
}
