﻿#region Header
// Copyright 2007-2011, Jerónimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.ComponentModel;
using System.ComponentModel.Design;
using System.Drawing.Design;
using NetJsWire.Lite.Converters;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Layout.Config
{
    [TypeConverter( typeof( ExpandableObjectConverter ) )]
    [JsonObject( MemberSerialization = MemberSerialization.OptIn )]
    public class LayoutConfig
    {
        [JsonConverter( typeof( NotQuotedJsonStringConverter ) )]
        [Editor( typeof( MultilineStringEditor ), typeof( UITypeEditor ) )]
        [Description( "An optional extra CSS class that will be added to the container. This can be useful for adding customized styles to the container or any of its children using standard CSS rules." )]
        [DefaultValue( ( string ) null )]
        public string ExtraCls { get; set; }


        [JsonProperty( "renderHidden", DefaultValueHandling = DefaultValueHandling.Ignore, NullValueHandling = NullValueHandling.Ignore )]
        [Description( "True to hide each contained item on render (defaults to false)." )]
        [DefaultValue( false )]
        public bool RenderHidden { get; set; }

        public virtual bool ShouldSerialize()
        {
            return !string.IsNullOrWhiteSpace( ExtraCls ) || RenderHidden;
        }

        public override string ToString()
        {
            return GetType().Name;
        }
    }
}