﻿#region Header
// Copyright 2007-2011, Jerónimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.Drawing;
using System.Linq;
using System.Windows.Forms;
using System.Windows.Forms.Layout;
using NetJsWire.Lite.Widgets.UI.Form;
using NetJsWire.Lite.Widgets.UI.Grid;

namespace NetJsWire.Lite.Widgets.UI.Layout
{
    internal class FormLayoutEngine : LayoutEngine
    {
        public override bool Layout( object container, LayoutEventArgs layoutEventArgs )
        {
            var impContainer = container as Container;
            if ( impContainer != null )
            {
                if ( impContainer.Layout == LayoutType.Form )
                {
                    var labelWidth = impContainer.LabelWidth;
                    var controls = impContainer.Items;
                    var formPanelContainer = Container.FindFormPanelInParents( impContainer );
                    if ( controls != null )
                    {
                        var labelAlign = impContainer.LabelAlign;
                        var labelPadding = 5f;

                        var leftPadding = 0;
                        var topPadding = 0;
                        var parentLeftPadding = 0;
                        var parentTopPadding = 0;
                        if ( impContainer is Panel )
                        {
                            parentLeftPadding = ( impContainer as Panel ).Padding.Left;
                            parentTopPadding = ( impContainer as Panel ).Padding.Top;
                            topPadding += parentTopPadding;
                        }
                        if ( formPanelContainer != null )
                        {
                            labelAlign = impContainer.LabelAlign != LabelAlign.Left || impContainer.LabelAlign != formPanelContainer.LabelAlign ? impContainer.LabelAlign : formPanelContainer.LabelAlign;
                            labelWidth = impContainer.LabelWidth != 100 || impContainer.LabelWidth != formPanelContainer.LabelWidth ? impContainer.LabelWidth : formPanelContainer.LabelWidth;
                        }
                        topPadding += ( labelAlign == LabelAlign.Top ? 20 : 0 );
                        var index = 0;
                        foreach ( var control in controls.Cast<BaseComponent>() )
                        {
                            var anchorRight = -5555;
                            var anchorBottom = -5555;

                            if ( control is BoxComponent )
                            {
                                if ( !string.IsNullOrWhiteSpace( ( control as BoxComponent ).Anchor ) )
                                {
                                    var anchors = ( control as BoxComponent ).Anchor.Split( ' ' );
                                    if ( !anchors[ 0 ].Contains( "%" ) )
                                        anchorRight = Convert.ToInt32( anchors[ 0 ] );
                                    else
                                    {
                                        var anchorRightPercent = Convert.ToInt32( anchors[ 0 ].Replace( "%", "" ) );
                                        anchorRight = -( ( impContainer.ClientSize.Width - ( ( !string.IsNullOrEmpty( control.FieldLabel ) ? labelWidth : 0 ) + 5 + parentLeftPadding ) ) / 100 * anchorRightPercent );
                                    }

                                    if ( anchors.Length > 1 )
                                    {
                                        if ( !anchors[ 1 ].Contains( "%" ) )
                                            anchorBottom = Convert.ToInt32( anchors[ 1 ] );
                                        else
                                        {
                                            var anchorBottomPercent = Convert.ToInt32( anchors[ 1 ].Replace( "%", "" ) );
                                            anchorBottom = -( impContainer.ClientSize.Height / 100 * anchorBottomPercent );
                                        }
                                    }
                                }
                            }

                            //leftPadding = labelAlign != LabelAlign.Top ? ( !string.IsNullOrEmpty( control.FieldLabel ) ? labelWidth : parentLeftPadding ) : ( labelAlign == LabelAlign.Left ? ( int ) labelPadding + parentLeftPadding : parentLeftPadding );
                            leftPadding = labelAlign != LabelAlign.Top ? ( ( !string.IsNullOrEmpty( control.FieldLabel ) || ( control is CompositeField && ( ( CompositeField ) control ).Items.Cast<BaseComponent>().Any( c => !string.IsNullOrWhiteSpace( c.FieldLabel ) ) ) ) ? ( int ) ( labelPadding + labelWidth + parentLeftPadding ) : ( int ) ( labelPadding + parentLeftPadding ) ) : ( labelAlign == LabelAlign.Left ? ( int ) labelPadding + parentLeftPadding : parentLeftPadding );
                            control.Location = new Point( leftPadding, topPadding );

                            var width = control.Size.Width;
                            var height = control.Size.Height;
                            if ( anchorRight != -5555 )
                            {
                                width = impContainer.ClientSize.Width - leftPadding + anchorRight;
                            }
                            if ( anchorBottom != -5555 )
                            {
                                if ( impContainer is Panel && ( ( Panel ) impContainer ).DrawButtonsBar )
                                {
                                    anchorBottom += ( Panel.ButtonsBarHeight - 8 );
                                }
                                if ( control is GridPanel && ( ( GridPanel ) control ).ShouldSerializeColumnModel() )
                                {
                                    anchorBottom += ( GridPanel.ColumnsBarHeight + 2 );
                                }

                                height = impContainer.ClientSize.Height - topPadding + anchorBottom;
                            }

                            if ( control.Size.Width != width || control.Size.Height != height )
                                control.Size = new Size( width, height );
                            topPadding += control.Height + 4 + ( labelAlign == LabelAlign.Top ? 22 : 0 );
                            //leftPadding = 0;
                        }

                        return false;
                    }
                }
            }
            return base.Layout( container, layoutEventArgs );
        }
    }
}
