﻿#region Header
// Copyright 2007-2011, Jerónimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.Drawing;
using System.Linq;
using System.Windows.Forms;
using NetJsWire.Lite.Widgets.UI.Form;
using NetJsWire.Lite.Widgets.UI.Layout.Config;

namespace NetJsWire.Lite.Widgets.UI.Layout
{
    public class HBoxLayoutEngine : BoxLayoutEngine
    {
        public override bool Layout( object container, LayoutEventArgs layoutEventArgs )
        {
            var impContainer = container as Container;
            if ( impContainer != null )
            {
                if ( impContainer.Layout == LayoutType.Hbox )
                {
                    var controls = impContainer.Items;
                    var config = ( HBoxLayoutConfig ) impContainer.LayoutConfig;
                    if ( controls != null )
                    {
                        var withFlex = controls.OfType<BaseComponent>().Where( c => c.Flex > 0 );
                        var notFlexed = controls.OfType<BaseComponent>().Where( c => c.Flex == 0 );
                        var notFlexedTotalWidth = notFlexed != null ? notFlexed.Sum( c => c.Size.Width ) : 0;
                        notFlexedTotalWidth += controls.OfType<BaseComponent>().Sum( c => c.Padding.Horizontal ) + config.DefaultMargins.Horizontal + config.Padding.Horizontal + impContainer.Padding.Horizontal;
                        var totalFlex = withFlex != null ? withFlex.Sum( c => c.Flex ) : 0;
                        var freeSpace = impContainer.ClientSize.Width - notFlexedTotalWidth;
                        var lefPadding = 0;
                        foreach ( var control in controls.Cast<BaseComponent>() )
                        {
                            lefPadding += control.Padding.Left;
                            if ( totalFlex > 0 )
                            {
                                if ( control.Flex > 0 )
                                {
                                    var mult = ( control.Flex / totalFlex );
                                    var width = freeSpace * mult;
                                    if ( width > 0 )
                                    {
                                        control.Size = new Size( ( int ) width, control.Size.Height );
                                        //freeSpace -= width;
                                    }
                                }
                            }
                            lefPadding += impContainer.Padding.Left + config.DefaultMargins.Left + config.Padding.Left;
                            control.Location = new Point( lefPadding, 0 );
                            lefPadding += control.Width + control.Padding.Right + impContainer.Padding.Right + config.DefaultMargins.Right + config.Padding.Right;
                        }

                        return false;
                    }
                }
            }
            else if ( container is CompositeField )
            {
                var impComposite = ( CompositeField ) container;
                var controls = impComposite.Items;
                //var config = ( HBoxLayoutConfig ) impComposite.LayoutConfig;
                if ( controls != null )
                {
                    var withFlex = controls.OfType<BaseComponent>().Where( c => c.Flex > 0 );
                    var notFlexed = controls.OfType<BaseComponent>().Where( c => c.Flex == 0 );
                    var notFlexedTotalWidth = notFlexed != null ? notFlexed.Sum( c => c.Size.Width ) : 0;
                    notFlexedTotalWidth += controls.OfType<BaseComponent>().Sum( c => c.Padding.Horizontal ) + impComposite.Padding.Horizontal;
                    var totalFlex = withFlex != null ? withFlex.Sum( c => c.Flex ) : 0;
                    var freeSpace = impComposite.ClientSize.Width - notFlexedTotalWidth;
                    var lefPadding = 0;
                    foreach ( var control in controls.Cast<BaseComponent>() )
                    {
                        lefPadding += control.Padding.Left;
                        if ( totalFlex > 0 )
                        {
                            if ( control.Flex > 0 )
                            {
                                var mult = ( control.Flex / totalFlex );
                                var width = freeSpace * mult;
                                if ( width > 0 )
                                {
                                    control.Size = new Size( ( int ) width, control.Size.Height );
                                    //freeSpace -= width;
                                }
                            }
                        }
                        lefPadding += impComposite.Padding.Left;
                        control.Location = new Point( lefPadding, 0 );
                        lefPadding += control.Width + control.Padding.Right + impComposite.Padding.Right + 5;
                    }

                    return false;
                }


            }
            return base.Layout( container, layoutEventArgs );
        }
    }
}
