﻿#region Header
// Copyright 2007-2011, Jerónimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.Drawing;
using System.Linq;
using System.Windows.Forms;
using System.Windows.Forms.Layout;
using NetJsWire.Lite.Widgets.UI.Toolbar;

namespace NetJsWire.Lite.Widgets.UI.Layout
{
    public class ToolbarLayoutEngine : LayoutEngine
    {
        public override bool Layout( object container, LayoutEventArgs layoutEventArgs )
        {
            var impContainer = container as Container;
            if ( impContainer != null && impContainer.Layout == LayoutType.Toolbar )
            {
                var controls = impContainer.Items != null ? impContainer.Items.Cast<BaseComponent>() : null;
                if ( controls != null )
                {
                    var leftPadding = 2;
                    foreach ( var control in controls )
                    {
                        control.Location = new Point( leftPadding, 2 );

                        if ( control is Fill )
                        {
                            var fillControl = control;
                            var width = impContainer.Size.Width - controls.Where( c => c != fillControl ).Sum( c => c.Size.Width );
                            width -= 6;
                            control.Size = new Size( width, control.Height );
                            leftPadding += width;
                        }
                        else
                        {
                            leftPadding += control.Size.Width;
                        }
                    }

                    return false;
                }
            }

            return base.Layout( container, layoutEventArgs );
        }
    }
}
