﻿#region Header
// Copyright 2007-2011, Jerónimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.Drawing;
using System.Linq;
using System.Windows.Forms;
using NetJsWire.Lite.Widgets.UI.Layout.Config;

namespace NetJsWire.Lite.Widgets.UI.Layout
{
    public class VBoxLayoutEngine : BoxLayoutEngine
    {
        public override bool Layout( object container, LayoutEventArgs layoutEventArgs )
        {
            var impContainer = container as Container;
            if ( impContainer != null )
            {
                if ( impContainer.Layout == LayoutType.Vbox )
                {
                    var controls = impContainer.Items;
                    var config = ( VBoxLayoutConfig ) impContainer.LayoutConfig;
                    if ( controls != null )
                    {
                        var withFlex = controls.OfType<BaseComponent>().Where( c => c.Flex > 0 );
                        var notFlexed = controls.OfType<BaseComponent>().Where( c => c.Flex == 0 );
                        var notFlexedTotalHeight = notFlexed != null ? notFlexed.Sum( c => c.Size.Height ) : 0;
                        notFlexedTotalHeight += controls.OfType<BaseComponent>().Sum( c => c.Padding.Vertical ) + config.DefaultMargins.Vertical + config.Padding.Vertical + impContainer.Padding.Vertical;
                        var totalFlex = withFlex != null ? withFlex.Sum( c => c.Flex ) : 0;
                        var freeSpace = impContainer.ClientSize.Height - notFlexedTotalHeight;
                        var topPadding = 0;
                        foreach ( var control in controls.Cast<BaseComponent>() )
                        {
                            if ( totalFlex > 0 )
                            {
                                if ( control.Flex > 0 )
                                {
                                    var mult = ( control.Flex / totalFlex );
                                    var height = freeSpace * mult;
                                    if ( height > 0 )
                                    {
                                        control.Size = new Size( control.Size.Width, ( int ) height );
                                    }
                                }
                            }
                            topPadding += control.Padding.Top + impContainer.Padding.Top + config.DefaultMargins.Top + config.Padding.Top;
                            control.Location = new Point( 0, topPadding );
                            topPadding += control.Height + control.Padding.Bottom + impContainer.Padding.Bottom + config.DefaultMargins.Bottom + config.Padding.Bottom;
                        }

                        return false;
                    }
                }
            }
            return base.Layout( container, layoutEventArgs );
        }
    }
}
