#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.ComponentModel;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Menu
{
    /// <summary>
    ///   /**
    ///   Adds a menu item that contains a checkbox by default, but can also be part of a radio group.
    /// </summary>
    public class CheckItem : Item
    {
        /// <summary>
        ///   All check items with the same group name will automatically be grouped into a single-select
        ///   radio button group (defaults to '')
        /// </summary>
        [JsonProperty( "group", DefaultValueHandling = DefaultValueHandling.Ignore, NullValueHandling = NullValueHandling.Ignore )]
        [DefaultValue( ( string ) null )]
        public string Group { get; set; }

        /// <summary>
        ///   The default CSS class to use for check items (defaults to "x-menu-item x-menu-check-item")
        /// </summary>
        [JsonProperty( "itemCls", DefaultValueHandling = DefaultValueHandling.Ignore, NullValueHandling = NullValueHandling.Ignore )]
        [DefaultValue( ( string ) null )]
        public override string ItemCls { get; set; }

        /// <summary>
        ///   The default CSS class to use for radio group check items (defaults to "x-menu-group-item")
        /// </summary>
        [JsonProperty( "groupClass", DefaultValueHandling = DefaultValueHandling.Ignore, NullValueHandling = NullValueHandling.Ignore )]
        [DefaultValue( ( string ) null )]
        public string GroupClass { get; set; }

        /// <summary>
        ///   True to initialize this checkbox as checked (defaults to false).  Note thatif this checkbox is part of a radio group (group = true) only the first item in the group that is
        ///   initialized with checked = true will be rendered as checked.
        /// </summary>
        [JsonProperty( "checked", DefaultValueHandling = DefaultValueHandling.Ignore )]
        [DefaultValue( false )]
        public bool Checked { get; set; }
    }
}
