#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.ComponentModel;
using System.Drawing.Design;
using NetJsWire.Lite.Converters;
using NetJsWire.Lite.Editors;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Menu
{
    /// <summary>
    ///   /**
    ///   <p>A menu containing a {@link Ext.ColorPalette} Component.</p>
    ///   <p>Notes:</p><div class = "mdetail-params"><ul>
    ///                                                <li>Although not listed here, the <b>constructor</b> for this class
    ///                                                  accepts all of the configuration options of <b>{@link Ext.ColorPalette}</b>.</li>
    ///                                                <li>If subclassing ColorMenu, any configuration options for the ColorPalette must be
    ///                                                  applied to the <tt><b>initialConfig</b></tt> property of the ColorMenu.
    ///                                                  Applying {@link Ext.ColorPalette ColorPalette} configuration settings to
    ///                                                  <b><tt>this</tt></b> will <b>not</b> affect the ColorPalette's configuration.</li>
    ///                                              </ul></div> *
    ///   @xtype colormenu
    ///   */
    ///   Ext.menu.ColorMenu = Ext.extend(Ext.menu.Menu, {
    /// </summary>
    [DesignTimeVisible( false )]
    public class ColorMenu : Menu
    {
        private bool _hideOnClick = true;

        /// <summary>
        ///   Optional. A function that will handle the select event of this menu.
        ///   The handler is passed the following parameters:<div class = "mdetail-params"><ul>
        ///                                                                                  <li><code>palette</code> : ColorPalette<div class = "sub-desc">The {@link #palette Ext.ColorPalette}.</div></li>
        ///                                                                                  <li><code>color</code> : String<div class = "sub-desc">The 6-digit color hex code (without the # symbol).</div></li>
        ///                                                                                </ul></div>
        /// </summary>
        [JsonProperty( "handler", DefaultValueHandling = DefaultValueHandling.Ignore, NullValueHandling = NullValueHandling.Ignore )]
        [JsonConverter( typeof( NotQuotedJsonStringConverter ) )]
        [Editor( typeof( ScriptEditor ), typeof( UITypeEditor ) )]
        [DefaultValue( ( string ) null )]
        public string Handler { get; set; }

        /// <summary>
        ///   The scope (<tt><b>this</b></tt> reference) in which the <code>{@link #handler}</code>
        ///   function will be called.  Defaults to this ColorMenu instance.
        /// </summary>
        [JsonProperty( "scope", DefaultValueHandling = DefaultValueHandling.Ignore, NullValueHandling = NullValueHandling.Ignore )]
        [JsonConverter( typeof( NotQuotedJsonStringConverter ) )]
        [DefaultValue( ( string ) null )]
        public string Scope { get; set; }

        /// <summary>
        ///   False to continue showing the menu after a color is selected, defaults to true.
        /// </summary>
        [JsonProperty( "hideOnClick", DefaultValueHandling = DefaultValueHandling.Ignore )]
        [DefaultValue( true )]
        public bool HideOnClick
        {
            get { return _hideOnClick; }
            set { _hideOnClick = value; }
        }

        /// <summary>
        ///   An id to assign to the underlying color palette. Defaults to <tt>null</tt>.
        /// </summary>
        [JsonProperty( "paletteId", DefaultValueHandling = DefaultValueHandling.Ignore, NullValueHandling = NullValueHandling.Ignore )]
        [DefaultValue( ( string ) null )]
        public string PaletteId { get; set; }

        /// <summary>
        ///   The {@link Ext.ColorPalette} instance for this ColorMenu
        /// </summary>
        [JsonProperty( "palette", DefaultValueHandling = DefaultValueHandling.Ignore, NullValueHandling = NullValueHandling.Ignore )]
        [DefaultValue( ( string ) null )]
        public ColorPalette Palette { get; set; }

        [Browsable( true )]
        [JsonProperty( "text", DefaultValueHandling = DefaultValueHandling.Ignore, NullValueHandling = NullValueHandling.Ignore )]
        [DefaultValue( ( string ) null )]
        public override string Text
        {
            get
            {
                return base.Text;
            }
            set
            {
                base.Text = value;
            }
        }

        [DefaultValue( "colormenu" )]
        [JsonProperty( "xtype", DefaultValueHandling = DefaultValueHandling.Include, NullValueHandling = NullValueHandling.Ignore )]
        public override string Xtype
        {
            get
            {
                return "colormenu";
            }
            set
            {
            }
        }
    }
}
