#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.ComponentModel;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Menu
{
    /// <summary>
    ///   /**
    ///   A base class for all menu items that require menu-related functionality (like sub-menus) and are not static
    ///   display items.  Item extends the base functionality of {@link Ext.menu.BaseItem} by adding menu-specific
    ///   activation and click handling.
    /// </summary>
    [DesignTimeVisible( false )]
    public class Item : BaseItem
    {
        private int _showDelay = 200;
        private string _xtype = "menuitem";

        /// <summary>
        ///   The submenu associated with this Item if one was configured.
        /// </summary>
        [JsonProperty( "menu" )]
        [DefaultValue( ( string ) null )]
        public Menu Menu { get; set; }

        /// <summary>
        ///   The path to an icon to display in this item (defaults to Ext.BLANK_IMAGE_URL).  Ificon is specified {@link #iconCls} should not be.
        /// </summary>
        [JsonProperty( "icon" )]
        [DefaultValue( ( string ) null )]
        public string Icon { get; set; }

        /// <summary>
        ///   A CSS class that specifies a background image that will be used as the icon forthis item (defaults to '').  If iconCls is specified {@link #icon} should not be.
        /// </summary>
        [JsonProperty( "iconCls" )]
        [DefaultValue( ( string ) null )]
        public string IconCls { get; set; }

        /// <summary>
        ///   The href attribute to use for the underlying anchor link (defaults to '#').
        /// </summary>
        [JsonProperty( "href" )]
        [DefaultValue( ( string ) null )]
        public string Href { get; set; }

        /// <summary>
        ///   The target attribute to use for the underlying anchor link (defaults to '').
        /// </summary>
        [JsonProperty( "hrefTarget" )]
        [DefaultValue( ( string ) null )]
        public string HrefTarget { get; set; }

        /// <summary>
        ///   The default CSS class to use for menu items (defaults to 'x-menu-item')
        /// </summary>
        [JsonProperty( "itemCls" )]
        [DefaultValue( ( string ) null )]
        public override string ItemCls { get; set; }

        /// <summary>
        ///   Length of time in milliseconds to wait before showing this item (defaults to 200)
        /// </summary>
        [JsonProperty( "showDelay" )]
        [DefaultValue( 200 )]
        public int ShowDelay
        {
            get { return _showDelay; }
            set { _showDelay = value; }
        }

        /// <summary>
        ///   The altText to use for the icon, if it exists. Defaults to <tt>''</tt>.
        /// </summary>
        [JsonProperty( "altText" )]
        [DefaultValue( ( string ) null )]
        public string AltText { get; set; }

        [DefaultValue( "menuitem" )]
        [JsonProperty( "xtype", DefaultValueHandling = DefaultValueHandling.Ignore,
            NullValueHandling = NullValueHandling.Ignore )]
        public override string Xtype
        {
            get { return _xtype; }
            set { _xtype = value; }
        }
    }
}
