#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.ComponentModel;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Menu
{
    /// <summary>
    ///   /**
    ///   Adds a separator bar to a menu, used to divide logical groups of menu items. Generally you will
    ///   add one of these by using "-" in you call to add() or in your items config rather than creating one directly.
    /// </summary>
    public class Separator : BaseItem
    {
        /// <summary>
        ///   The default CSS class to use for separators (defaults to "x-menu-sep")
        /// </summary>
        [JsonProperty( "itemCls", DefaultValueHandling = DefaultValueHandling.Ignore, NullValueHandling = NullValueHandling.Ignore )]
        [DefaultValue( ( string ) null )]
        public override string ItemCls { get; set; }

        /// <summary>
        ///   True to hide the containing menu after this item is clicked (defaults to false)
        /// </summary>
        [JsonProperty( "hideOnClick", DefaultValueHandling = DefaultValueHandling.Ignore )]
        [DefaultValue( false )]
        public override bool HideOnClick { get; set; }

        [DefaultValue( "menuseparator" )]
        [JsonProperty( "xtype", DefaultValueHandling = DefaultValueHandling.Include, NullValueHandling = NullValueHandling.Ignore )]
        public override string Xtype
        {
            get
            {
                return "menuseparator";
            }
        }
    }
}
