#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.ComponentModel;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Menu
{
    /// <summary>
    ///   /**
    ///   Adds a static text string to a menu, usually used as either a heading or group separator.
    /// </summary>
    public class TextItem : BaseItem
    {
        /// <summary>
        ///   The text to display for this item (defaults to '')
        /// </summary>
        [JsonProperty( "text", DefaultValueHandling = DefaultValueHandling.Ignore, NullValueHandling = NullValueHandling.Ignore )]
        [Browsable( true )]
        [DefaultValue( ( string ) null )]
        public override string Text { get; set; }

        /// <summary>
        ///   True to hide the containing menu after this item is clicked (defaults to false)
        /// </summary>
        [JsonProperty( "hideOnClick", DefaultValueHandling = DefaultValueHandling.Ignore )]
        public override bool HideOnClick { get; set; }

        /// <summary>
        ///   The default CSS class to use for text items (defaults to "x-menu-text")
        /// </summary>
        [JsonProperty( "itemCls", DefaultValueHandling = DefaultValueHandling.Ignore, NullValueHandling = NullValueHandling.Ignore )]
        [DefaultValue( ( string ) null )]
        public override string ItemCls { get; set; }

        [DefaultValue( "menutextitem" )]
        [JsonProperty( "xtype", DefaultValueHandling = DefaultValueHandling.Include, NullValueHandling = NullValueHandling.Ignore )]
        public override string Xtype
        {
            get
            {
                return "menutextitem";
            }
        }
    }
}
