#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion
namespace NetJsWire.Lite.Widgets.UI
{
    /// <summary>
    ///   /**
    ///   A split button that provides a built-in dropdown arrow that can fire an event separately from the default
    ///   click event of the button.  Typically this would be used to display a dropdown menu that provides additional
    ///   options to the primary button action, but any custom handler can provide the arrowclick implementation.  Example usage:
    ///   <pre><code>
    ///          // display a dropdown menu:
    ///          new Ext.SplitButton({
    ///          renderTo: 'button-ct', // the container id
    ///          text: 'Options',
    ///          handler: optionsHandler, // handle a click on the button itself
    ///          menu: new Ext.menu.Menu({
    ///          items: [
    ///          // these items will render as dropdown menu items when the arrow is clicked:
    ///          {text: 'Item 1', handler: item1Handler},
    ///          {text: 'Item 2', handler: item2Handler}
    ///          ]
    ///          })
    ///          });
    ///          // Instead of showing a menu, you provide any type of custom
    ///          // functionality you want when the dropdown arrow is clicked:
    ///          new Ext.SplitButton({
    ///          renderTo: 'button-ct',
    ///          text: 'Options',
    ///          handler: optionsHandler,
    ///          arrowHandler: myCustomHandler
    ///          });
    ///        </code></pre>
    ///   @cfg {Function} arrowHandler A function called when the arrow button is clicked (can be used instead of click event)
    ///   @cfg {String} arrowTooltip The title attribute of the arrow
    /// </summary>
    public class SplitButton : ButtonLW
    {
        /// <summary>
        ///   Create a new menu button
        ///   @xtype splitbutton
        /// </summary>
        /// <returns></returns>
        public SplitButton()
        {
            base.Xtype = "splitbutton";
        }
    }
}
