#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.ComponentModel;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Toolbar
{
    /// <summary>
    ///   /**
    ///   A non-rendering placeholder item which instructs the Toolbar's Layout to begin using
    ///   the right-justified button container.
    ///   <pre><code>
    ///          new Ext.Panel({
    ///          tbar : [
    ///          'Item 1',
    ///          {xtype: 'tbfill'}, // or '->'
    ///          'Item 2'
    ///          ]
    ///          });
    ///        </code></pre>
    /// </summary>
    [JsonObject( MemberSerialization = MemberSerialization.OptIn )]
    public class Fill : Spacer
    {
        /// <summary>
        ///   Creates a new Fill
        ///   @xtype tbfill
        /// </summary>
        /// <returns></returns>
        public Fill()
        {
            base.Xtype = "tbfill";
        }

        [JsonIgnore]
        [DefaultValue( 0 )]
        public override int Width
        {
            get
            {
                return 0;
            }
            set
            {
            }
        }

        [JsonIgnore]
        public override int Height
        {
            get
            {
                return base.Height;
            }
            set
            {
                base.Height = value;
            }
        }

        [JsonIgnore]
        public override int? X
        {
            get
            {
                return base.X;
            }
        }

        [JsonIgnore]
        public override int? Y
        {
            get
            {
                return base.Y;
            }
        }
    }
}
