#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.ComponentModel;
using System.Drawing;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Toolbar
{
    /// <summary>
    ///   /**
    ///   A simple element that adds extra horizontal space between items in a toolbar.
    ///   By default a 2px wide space is added via css specification:<pre><code>
    ///                                                                     .x-toolbar .xtb-spacer {
    ///                                                                     width:2px;
    ///                                                                     }
    ///                                                                   </code></pre>
    ///   <p>Example usage:</p>
    ///   <pre><code>
    ///          new Ext.Panel({
    ///          tbar : [
    ///          'Item 1',
    ///          {xtype: 'tbspacer'}, // or ' '
    ///          'Item 2',
    ///          // space width is also configurable via javascript
    ///          {xtype: 'tbspacer', width: 50}, // add a 50px space
    ///          'Item 3'
    ///          ]
    ///          });
    ///        </code></pre>
    /// </summary>
    [JsonObject( MemberSerialization = MemberSerialization.OptIn )]
    public class Spacer : Item
    {
        /// <summary>
        ///   Creates a new Spacer
        ///   @xtype tbspacer
        /// </summary>
        /// <returns></returns>
        public Spacer()
        {
            base.Xtype = "tbspacer";
            Size = new Size( 10, Size.Height );
        }

        [DefaultValue( 10 )]
        public override int Width
        {
            get
            {
                return base.Width;
            }
            set
            {
                base.Width = value;
            }
        }

        [JsonIgnore]
        public override int Height
        {
            get
            {
                return base.Height;
            }
            set
            {
                base.Height = value;
            }
        }

        [JsonIgnore]
        public override int? X
        {
            get
            {
                return base.X;
            }
        }

        [JsonIgnore]
        public override int? Y
        {
            get
            {
                return base.Y;
            }
        }
    }
}
