#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using NetJsWire.Lite.Widgets.UI.Dd;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Tree
{
    /// <summary>
    ///   /**
    /// </summary>
    public class TreeDropZone : DropZone
    {
        /// <summary>
        ///   Allow inserting a dragged node between an expanded parent node and its first child that will become a
        ///   sibling of the parent when dropped (defaults to false)
        /// </summary>
        [JsonProperty( "allowParentInsert" )]
        public bool AllowParentInsert { get; set; }

        /// <summary>
        ///   True if drops on the tree container (outside of a specific tree node) are allowed (defaults to false)
        /// </summary>
        [JsonProperty( "allowContainerDrop" )]
        public string AllowContainerDrop { get; set; }

        /// <summary>
        ///   True if the tree should only allow append drops (use for trees which are sorted, defaults to false)
        /// </summary>
        [JsonProperty( "appendOnly" )]
        public string AppendOnly { get; set; }

        /// <summary>
        ///   The TreePanel for this drop zone
        /// </summary>
        [JsonProperty( "tree" )]
        public TreePanel Tree { get; set; }

        /// <summary>
        ///   Arbitrary data that can be associated with this tree and will be included in the event object that gets
        ///   passed to any nodedragover event handler (defaults to {})
        /// </summary>
        [JsonProperty( "dragOverData" )]
        public TreePanel DragOverData { get; set; }

        /// <summary>
        ///   A named drag drop group to which this object belongs.  If a group is specified, then this object will only
        ///   interact with other drag drop objects in the same group (defaults to 'TreeDD').
        /// </summary>
        [JsonProperty( "ddGroup" )]
        public override string DdGroup { get; set; }

        /// <summary>
        ///   The delay in milliseconds to wait before expanding a target tree node while dragging a droppable node
        ///   over the target (defaults to 1000)
        /// </summary>
        [JsonProperty( "expandDelay" )]
        public string ExpandDelay { get; set; }
    }
}
