#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Tree
{
    /// <summary>
    ///   /**
    ///   Provides editor functionality for inline tree node editing.  Any valid {@link Ext.form.Field} subclass can be used
    ///   as the editor field.
    /// </summary>
    public class TreeEditor : Editor
    {
        /// <summary>
        ///   The position to align to (see {@link Ext.Element#alignTo} for more details, defaults to "l-l").
        /// </summary>
        [JsonProperty( "alignment" )]
        public override string Alignment { get; set; }

        /// <summary>
        ///   True to hide the bound element while the editor is displayed (defaults to false)
        /// </summary>
        [JsonProperty( "hideEl" )]
        public override bool HideEl { get; set; }

        /// <summary>
        ///   CSS class to apply to the editor (defaults to "x-small-editor x-tree-editor")
        /// </summary>
        [JsonProperty( "cls" )]
        public string Cls { get; set; }

        /// <summary>
        ///   True to shim the editor if selects/iframes could be displayed beneath it (defaults to false)
        /// </summary>
        [JsonProperty( "shim" )]
        public bool Shim { get; set; }

        /// <summary>
        ///   The maximum width in pixels of the editor field (defaults to 250).  Note that if the maxWidth would exceed
        ///   the containing tree element's size, it will be automatically limited for you to the container width, taking
        ///   scroll and client offsets into account prior to each edit.
        /// </summary>
        [JsonProperty( "maxWidth" )]
        public float MaxWidth { get; set; }

        /// <summary>
        ///   The number of milliseconds between clicks to register a float-click that will triggerediting on the current node (defaults to 350).  If two clicks occur on the same node within this time span,
        ///   the editor for the node will display, otherwise it will be processed as a regular click.
        /// </summary>
        [JsonProperty( "editDelay" )]
        public float EditDelay { get; set; }

        /// <summary>
        ///   The {@link Ext.tree.TreeNode TreeNode} this editor is bound to. Read-only.
        /// </summary>
        [JsonProperty( "editNode" )]
        public TreeNode EditNode { get; set; }
    }
}
