#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.Collections;
using System.Collections.Generic;
using NetJsWire.Lite.Widgets.UI.Data;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Tree
{
    /// <summary>
    ///   /**
    ///   @cfg {String} text The text for this node
    ///   @cfg {Boolean} expanded true to start the node expanded
    ///   @cfg {Boolean} allowDrag False to make this node undraggable if {@link #draggable} = true (defaults to true)
    ///   @cfg {Boolean} allowDrop False if this node cannot have child nodes dropped on it (defaults to true)
    ///   @cfg {Boolean} disabled true to start the node disabled
    ///   @cfg {String} icon The path to an icon for the node. The preferred way to do this
    ///   is to use the cls or iconCls attributes and add the icon via a CSS background image.
    ///   @cfg {String} cls A css class to be added to the node
    ///   @cfg {String} iconCls A css class to be added to the nodes icon element for applying css background images
    ///   @cfg {String} href URL of the link used for the node (defaults to #)
    ///   @cfg {String} hrefTarget target frame for the link
    ///   @cfg {Boolean} hidden True to render hidden. (Defaults to false).
    ///   @cfg {String} qtip An Ext QuickTip for the node
    ///   @cfg {Boolean} expandable If set to true, the node will always show a plus/minus icon, even when empty
    ///   @cfg {String} qtipCfg An Ext QuickTip config for the node (used instead of qtip)
    ///   @cfg {Boolean} singleClickExpand True for single click expand on this node
    ///   @cfg {Function} uiProvider A UI <b>class</b> to use for this node (defaults to Ext.tree.TreeNodeUI)
    ///   @cfg {Boolean} checked True to render a checked checkbox for this node, false to render an unchecked checkbox
    ///   (defaults to undefined with no checkbox rendered)
    ///   @cfg {Boolean} draggable True to make this node draggable (defaults to false)
    ///   @cfg {Boolean} isTarget False to not allow this node to act as a drop target (defaults to true)
    ///   @cfg {Boolean} allowChildren False to not allow this node to have child nodes (defaults to true)
    ///   @cfg {Boolean} editable False to not allow this node to be edited by an {@link Ext.tree.TreeEditor} (defaults to true)
    /// </summary>
    public class TreeNode : Node
    {
        private List<TreeNode> _childNodes = new List<TreeNode>();

        /// <summary>
        ///   Read-only. The text for this node. To change it use <code>{@link #setText}</code>.
        /// </summary>
        [JsonProperty( "text" )]
        public string Text { get; set; }

        /// <summary>
        ///   True if this node is disabled.
        /// </summary>
        [JsonProperty( "disabled" )]
        public bool Disabled { get; set; }

        /// <summary>
        ///   True if this node is hidden.
        /// </summary>
        [JsonProperty( "hidden" )]
        public bool Hidden { get; set; }

        [JsonProperty( "childNodes" )]
        public override IList ChildNodes
        {
            get { return _childNodes; }
            set { _childNodes = ( List<TreeNode> ) value; }
        }

        /// <summary>
        ///   Read-only. The UI for this node
        /// </summary>
        [JsonProperty( "ui" )]
        public TreeNodeUI Ui { get; set; }
    }
}
