#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.ComponentModel;
using NetJsWire.Lite.Designers;

namespace NetJsWire.Lite.Widgets.UI
{
    /// <summary>
    ///   /**
    ///   <p>A specialized container representing the viewable application area (the browser viewport).</p>
    ///   <p>The Viewport renders itself to the document body, and automatically sizes itself to the size of
    ///     the browser viewport and manages window resizing. There may only be one Viewport created
    ///     in a page. Inner layouts are available by virtue of the fact that all {@link Ext.Panel Panel}s
    ///     added to the Viewport, either through its {@link #items}, or through the items, or the {@link #add}
    ///     method of any of its child Panels may themselves have a layout.</p>
    ///   <p>The Viewport does not provide scrolling, so child Panels within the Viewport should provide
    ///     for scrolling if needed using the {@link #autoScroll} config.</p>
    ///   <p>An example showing a classic application border layout:</p><pre><code>
    ///                                                                        new Ext.Viewport({
    ///                                                                        layout: 'border',
    ///                                                                        items: [{
    ///                                                                        region: 'north',
    ///                                                                        html: '&lt;h1 class="x-panel-header">Page Title&lt;/h1>',
    ///                                                                        autoHeight: true,
    ///                                                                        border: false,
    ///                                                                        margins: '0 0 5 0'
    ///                                                                        }, {
    ///                                                                        region: 'west',
    ///                                                                        collapsible: true,
    ///                                                                        title: 'Navigation',
    ///                                                                        width: 200
    ///                                                                        // the west region might typically utilize a {@link Ext.tree.TreePanel TreePanel} or a Panel with {@link Ext.layout.AccordionLayout Accordion layout}
    ///                                                                        }, {
    ///                                                                        region: 'south',
    ///                                                                        title: 'Title for Panel',
    ///                                                                        collapsible: true,
    ///                                                                        html: 'Information goes here',
    ///                                                                        split: true,
    ///                                                                        height: 100,
    ///                                                                        minHeight: 100
    ///                                                                        }, {
    ///                                                                        region: 'east',
    ///                                                                        title: 'Title for the Grid Panel',
    ///                                                                        collapsible: true,
    ///                                                                        split: true,
    ///                                                                        width: 200,
    ///                                                                        xtype: 'grid',
    ///                                                                        // remaining grid configuration not shown ...
    ///                                                                        // notice that the GridPanel is added directly as the region
    ///                                                                        // it is not "overnested" inside another Panel
    ///                                                                        }, {
    ///                                                                        region: 'center',
    ///                                                                        xtype: 'tabpanel', // TabPanel itself has no title
    ///                                                                        items: {
    ///                                                                        title: 'Default Tab',
    ///                                                                        html: 'The first tab\'s content. Others may be added dynamically'
    ///                                                                        }
    ///                                                                        }]
    ///                                                                        });
    ///                                                                      </code></pre>
    /// </summary>
    [ToolboxItem( false )]
    [Designer( typeof( ContainerDesigner ) )]
    public class Viewport : Container
    {
        /// <summary>
        ///   Create a new Viewport
        ///   @xtype viewport
        /// </summary>
        /// <returns></returns>
        public Viewport()
        {
            base.Xtype = "viewport";
        }
    }
}
