#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.IO;
using System.IO.Compression;
using System.Text;
using System.Web;
using System.Web.SessionState;
using NetJsWire.JsonRpcHandler.RealAsync;

namespace NetJsWire.JsonRpcHandler.Handlers
{
    public class CSHandlerFactory : IHttpHandlerFactory, IRequiresSessionState
    {
        #region IHttpHandlerFactory Members

        public void ReleaseHandler( IHttpHandler handler )
        {
        }

        public IHttpHandler GetHandler( HttpContext context, string requestType, string url, string pathTranslated )
        {
            var encodings = context.Request.Headers.Get( "Accept-Encoding" );
            var filename = Path.GetFileNameWithoutExtension( context.Request.Path );

            //string token = context.Request.Headers[ "njsw-token" ];

            if ( encodings != null && filename.ToLower() != "images" && requestType == "POST" )
            {
                var baseStream = context.Response.Filter;
                encodings = encodings.ToLower();

                if ( encodings.Contains( "gzip" ) )
                {
                    context.Response.Filter = new GZipStream( baseStream, CompressionMode.Compress );
                    context.Response.AppendHeader( "Content-Encoding", "gzip" );
                }
                else if ( encodings.Contains( "deflate" ) )
                {
                    context.Response.Filter = new DeflateStream( baseStream, CompressionMode.Compress );
                    context.Response.AppendHeader( "Content-Encoding", "deflate" );
                }
            }

            if ( filename.ToLower() == "status" )
            {
                return new EmbeddedHandler( "status" );
            }
            if ( filename.ToLower() == "objdetail" )
            {
                return new EmbeddedHandler( "objdetail" );
            }
            //if ( filename.ToLower() == "extdirectapi" )
            //{
            //    return new DirectProxy();
            //}
            //if ( filename.ToLower() == "directrouter" )
            //{
            //}
            switch ( requestType )
            {
                case "POST":
                    context.Response.ContentEncoding = Encoding.UTF8;
                    context.Response.ContentType = "application/json";
                    switch ( filename.ToLower() )
                    {
                        case "index":
                            return new EmbeddedHandler( "index" );
                        case "custom":
                            return new EmbeddedHandler( "custom" );
                        //case "directrouter":
                        //    return new DirectRouter();
                        default:
                            if ( filename.Contains( "_sync_" ) )
                            {
                                return new MainHandler();
                            }
                            if ( filename.ToLower() == "unload" )
                            {
                                return new EmbeddedHandler( "unload" );
                            }
                            break;
                    }
                    return new AsyncHandler( false );
                case "GET":
                    if ( filename.ToLower() == "images" )
                    {
                        return new AsyncHandler( true );
                    }
                    if ( filename.ToLower() == "custom" )
                    {
                        return new EmbeddedHandler( "custom" );
                    }
                    break;
            }
            return null;

        }

        #endregion
    }
}