#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion
#region Using

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Web;
using NetJsWire.JsonRpcHandler.Util;
using NetJsWire.Widgets.Ui.Form;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

#endregion

namespace NetJsWire.JsonRpcHandler.Handlers
{
    public class CSJsonRequestProcessor : CSRequestProcessor
    {
        public CSJsonRequestProcessor( HttpContext context )
            : base( context )
        {
        }

        public override void Process()
        {
            var result = string.Empty;
            if ( Posts != null )
            {
                foreach ( var p in Posts )
                {
                    Post = p;
                    result += ProcessPost();
                }
            }
            else if ( Posts == null && Post != null && Post.JSONRpcPostParams != null &&
                      Post.JSONRpcPostParams.Id != null )
            {
                result = ProcessPost();
            }

            if ( Exceptions.Count > 0 )
            {
                result = Exceptions.Aggregate( result, ( current, exc ) => current + ( "ShowException(" + exc + ");" ) );
                Exceptions.Clear();
            }
            if ( Posts == null && Post != null && Post.id == "-11" && Exceptions.Count == 0 )
            {
                Context.Response.Write( result );
            }
            else if ( Posts == null && Post != null && Post.id == "-12" && Exceptions.Count == 0 )
            {
                //Context.Response.ContentType = "text/html";
                result = JsonConvert.SerializeObject( result );
                Context.Response.Write( string.Format( "{{success:{0}, json:{1}}}",
                                                       WebApplication.Instance.FormPostSuccess.ToString().ToLower(),
                                                       result ) );
            }
            else if ( Posts == null && ( Post == null || string.IsNullOrEmpty( Post.id ) ) )
            {
                Context.Response.End();
            }
            else
            {
                result = JsonConvert.SerializeObject( new JSONRpcResult { ID = "1", Result = string.Format( "function(){{{0}}}", result ) } );  //  JsonConvert.ExportToString( new JSONRpcResult( "1", result, null ) );
                Context.Response.Write( result );
            }

            //Context.Response.AppendHeader( "Content-Length", result.Length.ToString() );

            //Context.Response.End();
            //Context.Response.Close();
        }

        private string ProcessPost()
        {
            try
            {
                WebApplication.SetInstanceContext( Context );

                if ( !WebApplication.Instance.Objects.ContainsKey( Post.JSONRpcPostParams.Id ) )
                {
                    if ( null == Context.Session )
                    {
                        throw new SessionEndedException( WebApplication.SessionEndedErrorText );
                    }

                    throw new Exception( WebApplication.ObjectNotFoundErrorText );
                }

                WebApplication.Instance.LastAccess = DateTime.Now;
                Caller = WebApplication.Instance.Objects[ Post.JSONRpcPostParams.Id ];

                if ( Method != null )
                {
                    var parameters = Method.GetParameters();
                    var sendParameters = new List<object> { Post.JSONRpcPostParams.Data };
                    if ( parameters != null && parameters.Length > 0 )
                    {
                        if ( !parameters[ 0 ].ParameterType.IsAssignableFrom( Post.JSONRpcPostParams.Data.GetType() ) )
                        {
                            if ( Post.JSONRpcPostParams.Data != null )
                            {
                                sendParameters = new List<object>();
                                dynamic data = Post.JSONRpcPostParams.Data;
                                foreach ( var o in data )
                                {
                                    sendParameters.Add( o );
                                }
                            }
                            Method.Invoke( Caller,
                                           parameters.Length == 1
                                               ? new[] { sendParameters.ToArray() }
                                               : sendParameters.ToArray() );
                        }
                        else
                        {
                            Method.Invoke( Caller, new[] { Post.JSONRpcPostParams.Data } );
                        }
                    }
                    else
                    {
                        Method.Invoke( Caller, null );
                    }
                    Debug.WriteLine( "Invoked: " + Method.Name + " on: " + Post.JSONRpcPostParams.Id );
                }
                else if ( Field != null )
                {
                    var eventDelegates = ( MulticastDelegate ) Field.GetValue( Caller );
                    if ( eventDelegates != null )
                    {
                        foreach ( var del in eventDelegates.GetInvocationList() )
                        {
                            if ( Post.JSONRpcPostParams.Data != null )
                            {
                                if ( Post.JSONRpcPostParams.Data is JObject && ( ( JObject ) Post.JSONRpcPostParams.Data )[ "Type" ] != null )
                                {
                                    var type = ( ( JObject ) Post.JSONRpcPostParams.Data ).Value<string>( "Type" );
                                    var t = TypeLoader.GetType( type );
                                    var arg = Activator.CreateInstance( t, Post.JSONRpcPostParams.Data );
                                    del.Method.Invoke( Caller, new[] { Caller, arg } );
                                    Debug.WriteLine( "Invoked: " + del.Method.Name + " on: " + Post.JSONRpcPostParams.Id );
                                }
                                else
                                {
                                    //Special case to handle posted forms
                                    if ( del.Method.Name == "FormPanelFormPost" )
                                    {
                                        Context.Response.ContentType = "text/html";
                                        del.Method.Invoke( del.Target,
                                                           new[]
                                                               {
                                                                   Post.JSONRpcPostParams.Data,
                                                                   new FormPostedEventArgs(
                                                                       (
                                                                       Dictionary< string, string > )
                                                                       Post.JSONRpcPostParams.Data,
                                                                       Post.Files )
                                                               } );

                                        if ( Caller.FileUpload )
                                        {
                                            if ( Caller.AutoSaveFiles && Post.Files != null &&
                                                 Post.Files.Count > 0 )
                                            {
                                                string path =
                                                    Context.Server.MapPath(
                                                                           ( !string.IsNullOrEmpty( Caller.SaveFilesPath )
                                                                                 ? Caller.SaveFilesPath
                                                                                 : "" ) );

                                                for ( var i = 0; i < Post.Files.Count; i++ )
                                                {
                                                    Post.Files[ i ].SaveAs( path.Replace( "\\webapp", "" ) +
                                                                            Post.Files[ i ].FileName );
                                                }
                                            }
                                        }
                                    }
                                    else
                                    {
                                        del.Method.Invoke( del.Target,
                                                           new[] { Post.JSONRpcPostParams.Data, EventArgs.Empty } );
                                    }
                                    Debug.WriteLine( "Invoked: " + del.Method.Name + " on: " + Post.JSONRpcPostParams.Id );
                                }
                            }
                            else
                            {
                                del.Method.Invoke( del.Target, new object[] { null, EventArgs.Empty } );
                                Debug.WriteLine( "Invoked: " + del.Method.Name + " on: " + Post.JSONRpcPostParams.Id );
                            }
                        }
                    }
                }
            }
            catch ( SessionEndedException )
            {
                Exceptions.Add( "\"" + WebApplication.SessionEndedErrorTitle + "\",\"" +
                                WebApplication.SessionEndedErrorText + "\"" );
            }
            catch ( Exception ex )
            {
                var exeText = ex.InnerException != null
                                     ? ex.InnerException.Message.Replace( Environment.NewLine, "<br />" )
                                     : ex.Message.Replace( Environment.NewLine, "<br />" );
                var stackTrace = ex.InnerException != null
                                        ? ex.InnerException.StackTrace
                                        : ex.StackTrace;
                exeText = JsonConvert.ToString( exeText );

                stackTrace = JsonConvert.ToString( stackTrace );
                Exceptions.Add( exeText + "," + stackTrace );
            }
            var s = WebApplication.Instance.RenderChanges();
            return s;
        }
    }
}