#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Web;
using System.Web.SessionState;
using NetJsWire.JsonRpcHandler.Util;
using Newtonsoft.Json;

namespace NetJsWire.JsonRpcHandler.Handlers
{
    public abstract class CSRequestProcessor : IRequiresSessionState
    {
        private readonly List<string> _exceptions;
        private readonly JSONRpcPost[] _posts;
        //private MethodInfo _method;
        private readonly StringBuilder _sbResponse = new StringBuilder();
        private dynamic _oBase;
        private JSONRpcPost _post;
        //private bool _shouldEndResponse = false;

        //we avoid empty parameters constructor
        private CSRequestProcessor()
        {
        }

        ///<summary>
        ///  Process the request
        ///</summary>
        ///<param name = "context">The calling context.</param>
        protected CSRequestProcessor( HttpContext context )
        {
            var form = context.Request.Form;
            _exceptions = new List<string>();
            Context = context;
            var data = ( new StreamReader( context.Request.InputStream ) ).ReadToEnd();

            if ( !string.IsNullOrEmpty( data ) && !data.ToLowerInvariant().Contains( "griddataservice" ) &&
                 !data.ToLowerInvariant().Contains( "form_service" ) )
            {
                _posts = ( JSONRpcPost[] ) JsonConvert.DeserializeObject( data, typeof( JSONRpcPost[] ) );
            }

            if ( _posts == null && _post == null )
            {
                if ( !string.IsNullOrEmpty( data ) )
                {
                    try
                    {
                        var srv = form[ "service" ];
                        if ( srv.ToLower() == "griddataservice" )
                        {
                            _post = new JSONRpcPost
                                        {
                                            JSONRpcPostParams = new JSONRpcPostParams(),
                                            id = "-11",
                                            method = form[ "method" ],
                                            service = srv
                                        };
                            _post.JSONRpcPostParams.Id = form[ "id" ];
                            _post.JSONRpcPostParams.Data = form;
                        }
                    }
                    catch
                    {
                    }

                    if ( form[ "service" ] == "form_service" )
                    {
                        _post = new JSONRpcPost
                                    {
                                        JSONRpcPostParams = new JSONRpcPostParams(),
                                        id = "-12",
                                        method = "form_post",
                                        service = form[ "service" ]
                                    };
                        _post.JSONRpcPostParams.Id = form[ "id" ];
                        _post.JSONRpcPostParams.Data = form.AllKeys.Where( key => !string.IsNullOrWhiteSpace( key ) && key != "id" && key != "method" && key != "service" ).ToDictionary( key => key, key => HttpUtility.UrlDecode( form[ key ], Encoding.Default ) );

                        if ( context.Request.Files != null && context.Request.Files.Count > 0 )
                        {
                            _post.Files = context.Request.Files;
                        }
                    }
                }
                else
                {
                    _post = new JSONRpcPost
                                {
                                    JSONRpcPostParams = new JSONRpcPostParams(),
                                    id = "1",
                                    method = "",
                                    service = ""
                                };
                    _post.JSONRpcPostParams.Id = null;
                    _post.JSONRpcPostParams.Data = null;
                }
            }
            try
            {
                WebApplication.SetInstanceContext( context );
                //if(context.Session != null)
                //    WebApplication.Instance = (WebApplication)context.Session["WebApplication"];
                //WebApplication.Instance = (WebApplication)WebApps.Getobject(context.Session["WebApplication"].ToString());
            }
            catch
            {
                //With this we prevent the server to crash on an unhandled exception
                //TODO: Better handling for this, we have to tell the client if the session has ended...
                WebApplication.Instance = null;
            }
        }

        public JSONRpcPost Post
        {
            get { return _post; }
            internal set { _post = value; }
        }

        public JSONRpcPost[] Posts
        {
            get { return _posts; }
        }

        public List<string> Exceptions
        {
            get { return _exceptions; }
        }

        public dynamic Caller
        {
            get { return _oBase; }
            internal set { _oBase = value; }
        }

        public HttpContext Context { get; set; }

        public MethodInfo Method
        {
            get
            {
                return _oBase.GetType().GetMethod( _post.method );
            }
        }

        public FieldInfo Field
        {
            get { return ClientUtils.FindEventInType( _oBase.GetType(), _post.method ); }
        }

        public void SerializeObject( object o )
        {
            _sbResponse.Append( JsonConvert.SerializeObject( o ) );
        }

        public abstract void Process();
    }
}