#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.ComponentModel;
using NetJsWire.Widgets.Config;

namespace NetJsWire.JsonRpcHandler
{
    public interface IWidget
    {
        ///<summary>
        ///  The renderer is the responsible for representing the object in the client side.
        ///</summary>
        [ Description( "The renderer" ), Browsable( true ) ]
        IRenderer Renderer { get; }

        /// <summary>
        ///   If the framework should generate a global Javascript variable that points to the Widget, the name of the variable will be the same as ClientID property of the Renderer.
        /// </summary>
        /// <remarks>
        ///   Only do this if its really necesary because every global property will stay there until you manually delete it, the framework wont delete globals, is up to the developer to do that.
        /// </remarks>
        [DefaultValue( false )]
        [ Description( "If the framework should generate a global Javascript variable that points to the Widget" ) ]
        bool GenerateClientMember { get; set; }

        /// <summary>
        ///   The ConfigOptions that will be passed to Ext when constructing the widget.
        /// </summary>
        IConfigOptions ConfigOptions { get; }

        /// <summary>
        ///   Add a client event to the Widget in the form of: Windget.on('name', function);
        /// </summary>
        /// <param name = "name">The event name.</param>
        /// <param name = "function">A function name or inline code to execute.</param>
        void On( string name, string function );

        /// <summary>
        ///   Adds a CSS class to the component's underlying element.
        /// </summary>
        /// <param name = "cls">The CSS class name to add.</param>
        void AddClass( string cls );

        /// <summary>
        ///   Removes a CSS class from the component's underlying element.
        /// </summary>
        /// <param name = "cls">The CSS class name to remove.</param>
        void RemoveClass( string cls );
    }
}