﻿#region Header
// Copyright 2007-2011, Jerónimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.Threading;
using System.Web;

namespace NetJsWire.JsonRpcHandler.RealAsync
{
    internal class AsyncRequestState : IAsyncResult
    {
        internal AsyncCallback Cb;
        internal HttpContext Ctx;
        internal object ExtraData;
        private ManualResetEvent _callCompleteEvent;
        private bool _isCompleted;

        public AsyncRequestState( HttpContext ctx, AsyncCallback cb,
                                  object extraData )
        {
            Ctx = ctx;
            Cb = cb;
            ExtraData = extraData;
        }

        // IAsyncResult
        //

        #region IAsyncResult Members

        public object AsyncState
        {
            get { return ( ExtraData ); }
        }

        public bool CompletedSynchronously
        {
            get { return ( false ); }
        }

        public bool IsCompleted
        {
            get { return ( _isCompleted ); }
        }

        public WaitHandle AsyncWaitHandle
        {
            get
            {
                lock ( this )
                {
                    return _callCompleteEvent ?? ( _callCompleteEvent = new ManualResetEvent( false ) );
                }
            }
        }

        #endregion

        internal void CompleteRequest()
        {
            _isCompleted = true;
            lock ( this )
            {
                if ( _callCompleteEvent != null )
                    _callCompleteEvent.Set();
            }
            // if a callback was registered, invoke it now
            if ( Cb != null )
                Cb( this );
        }
    }
}