#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.Collections;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using System.Web;

namespace NetJsWire.JsonRpcHandler.Util
{
    public class Util
    {
        public static string RootPath
        {
            get { return HttpContext.Current.Server.MapPath( HttpContext.Current.Request.ApplicationPath + "\\" ); }
        }

        public static string Join( IList list, string sep )
        {
            return Join( list, sep, false );
        }

        public static string Join( IList list, string sep, bool formatForClient )
        {
            return Join( list, sep, formatForClient, 0 );
        }

        public static string Join( IList list, string sep, bool formatForClient, int startIndex )
        {
            return Join( list, sep, formatForClient, startIndex, list.Count - 1 );
        }

        public static string Join( IList list, string sep, bool formatForClient, int startIndex, int finishIndex )
        {
            if ( list == null || list.Count == 0 )
                return null;
            var o = "";
            for ( var i = startIndex; i < finishIndex; i++ )
            {
                string val;
                val = prepareValue( list, formatForClient, i );
                o += val + sep;
            }
            o += prepareValue( list, formatForClient, finishIndex );
            return o;
        }

        private static string prepareValue( IList list, bool formatForClient, int i )
        {
            string val;
            if ( formatForClient )
                val = FormatForClient( list[ i ] as string );
            else
            {
                if ( list[ i ] != null )
                    val = list[ i ].ToString();
                else
                    val = string.Empty;
            }
            return val;
        }


        public static string Join<T>( T[] cols )
        {
            return Join( cols, "," );
        }

        public static string Join<T>( List<T> cols )
        {
            return Join( cols, "," );
        }

        public static string Quotize( string s )
        {
            if ( string.IsNullOrWhiteSpace( s ) )
            {
                return "''";
            }
            s = s.Replace( "</script>", "</scr' + 'ipt>" );
            s = s.Replace( "</SCRIPT>", "</scr' + 'ipt>" );
            return Surround( s, "'" );
        }

        public static string Surround( string inner, string surrounder )
        {
            return surrounder + inner + surrounder;
        }

        public static string SurroundTag( string inner, string tag )
        {
            return string.Format( "<{0}>{1}</{0}>", tag, inner );
        }

        public static string FormatForClient( string input )
        {
            if ( input == null )
                return string.Empty;
            var thistext = input.Replace( "\\", "\\\\" );
            thistext = thistext.Replace( "\r\n", "\\r\\n" );
            thistext = thistext.Replace( "\n", "\\n" );
            thistext = thistext.Replace( "\r", "\\r" );
            thistext = thistext.Replace( "'", "\\'" );
            thistext = thistext.Replace( ":", @"\:" );
            thistext = thistext.Replace( "\"", "" );
            return thistext;
        }

        public static string ToJavaScriptString( string input )
        {
            return Quotize( FormatForClient( input ) );
        }

        public static string Coalesce( params string[] args )
        {
            for ( var i = 0; i < args.Length; i++ )
            {
                if ( args[ i ] != null )
                    return args[ i ];
            }
            return null;
        }

        public static string Textalize( string source )
        {
            var r = new Regex( @"(?<tag>====|===|==|'''|\''|\||##|\*\*)(?<inner>.*?)\k<tag>", RegexOptions.Singleline );
            MatchEvaluator me = matchTag;
            source = r.Replace( source, me );
            source = source.Replace( "\r\n\r\n", "<br><br>" );
            source = source.Replace( "\n\n", "<br><br>" );
            source = source.Replace( "\\n", "<br>" );
            return source;
        }

        private static string matchTag( Match m )
        {
            var tag = string.Empty;
            var atts = string.Empty;
            var inner = m.Groups[ "inner" ].Value;
            switch ( m.Groups[ "tag" ].Captures[ 0 ].Value )
            {
                case "'''":
                    tag = "B";
                    break;
                case "''":
                    tag = "I";
                    break;
                case "|":
                    tag = "A";
                    atts = " HREF='" + m.Groups[ "inner" ].Value + "'";
                    break;
                case "##":
                    tag = "OL";
                    inner = inner.Replace( "#.", "<LI>" );
                    break;
                case "**":
                    tag = "UL";
                    inner = inner.Replace( "*.", "<LI>" );
                    break;
                case "====":
                    tag = "H3";
                    break;
                case "===":
                    tag = "H2";
                    break;
                case "==":
                    tag = "H1";
                    break;
            }
            return string.Format( "<{0}{2}>{1}</{0}>", tag, Textalize( inner ), atts );
        }

        public object Coalesce( params object[] values )
        {
            for ( var i = 0; i < values.Length; i++ )
                if ( values[ i ] != null )
                    return values[ i ];
            return null;
        }

        public static string HashToMime( Dictionary<string, string> input )
        {
            var output = string.Empty;
            var pairs = new List<string>();
            foreach ( var key in input.Keys )
            {
                pairs.Add( key + "=" + input[ key ] );
            }
            return Join( pairs, "&" );
        }
    }
}