// Copyright 2007-2011, Jernimo Milea
//
// This file is part of NetJsWire.
//
//    NetJsWire is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    NetJsWire is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;

namespace JSONRpcHandler
{
    public static class WebApps
    {
        private static readonly Hashtable ht = Hashtable.Synchronized(new Hashtable());
        //private static Timer t;
        //private static AutoResetEvent autoEvent = new AutoResetEvent(false);
        //private static TimerCallback timerDelegate = new TimerCallback(Timer_CallBack);
        private const int mins = 30;

        public static object Getobject(string name)
        {
            try
            {
                lock (ht)
                {
                    if (ht.ContainsKey(name))
                    {
                        WebApplication app = (WebApplication)ht[name];

                        return app;
                    }
                }
            }
            finally
            {
                CheckExpired();
            }

            return null;
        }

        public static void CheckExpired()
        {
            lock (ht)
            {
                List<string> toRemove = new List<string>();
                foreach (string key in ht.Keys)
                {
                    WebApplication app = (WebApplication)ht[key];

                    if (app.LastAccess < DateTime.Now.AddMinutes(-mins))
                    {
                        toRemove.Add(key);
                    }
                }

                foreach (string s in toRemove)
                {
                    RemoveObject(s);
                }
            }
        }

        public static void RemoveObject(string name)
        {
            lock (ht)
            {
                if (ht.ContainsKey(name))
                    ht.Remove(name);
            }
        }

        public static void AddObject(string name, object value)
        {
            lock (ht)
            {
                if (!ht.ContainsKey(name))
                {
                    //ht.Add(name, new WebApp((WebApplication)value, DateTime.Now));
                    ht.Add(name, value);
                }
                else
                {
                    //                    ht[name] = new WebApp((WebApplication)value, DateTime.Now);
                    ht[name] = value;
                }
            }

            CheckExpired();
        }

        public static object GetObjectAt(int index)
        {
            lock (ht)
            {
                if (ht.Count > index)
                {
                    int i = 0;
                    foreach (WebApplication item in ht.Values)
                    {
                        if (i == index)
                            return item;
                        i++;
                    }
                }
            }

            return null;
        }

        public static bool HasObjects
        {
            get { return ht.Count > 0; }
        }

        public static int ObjectCount
        {
            get { return ht.Count; }
        }

        public static Hashtable Objects
        {
            get { return ht; }
        }

        public static IEnumerable GetAllValues()
        {
            foreach (WebApplication item in ht.Values)
            {
                yield return item;
            }
        }
    }
}