﻿#region Header
// Copyright 2007-2011, Jerónimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.Collections.Generic;
using NetJsWire.Widgets.Core;
using Newtonsoft.Json;

namespace NetJsWire.Util
{
    public class CommandConverter : JsonConverter
    {
        public override void WriteJson( JsonWriter writer, object value, JsonSerializer serializer )
        {
            var _cmd = ( Command ) value;
            var _hasPreviousObjects = false;
            if ( _cmd.Value != null && ( _cmd.Value as object[] ).Length > 1 )
            {
                var _commands = ( _cmd.Value as object[] );
                var _index = 0;
                foreach ( var command in _commands )
                {
                    if ( command is IDictionary<string, object> )
                    {
                        writer.WriteStartObject();
                        foreach ( var val in command as IDictionary<string, object> )
                        {
                            writer.WritePropertyName( val.Key );
                            writer.WriteValue( val.Value );
                        }
                        writer.WriteEndObject();
                        _hasPreviousObjects = true;
                    }
                    else
                    {
                        if ( command.ToString().StartsWith( "Ext." ) )
                        {
                            writer.WriteRawValue( ( ( _index > 0 || _hasPreviousObjects ) && _index < _commands.Length - 1 ? "," : "" ) + command );
                        }
                        else
                        {
                            if ( ( _index > 0 || _hasPreviousObjects ) )
                            {
                                writer.WriteRaw( "," );
                            }
                            writer.WriteValue( command );
                        }

                        _hasPreviousObjects = true;
                    }
                }
            }
            else
            {
                serializer.Serialize( writer, _cmd.Value );
            }
        }

        public override bool CanConvert( Type objectType )
        {
            return typeof( Command ).IsAssignableFrom( objectType );
        }

        public override object ReadJson( JsonReader reader, Type objectType, object existingValue, JsonSerializer serializer )
        {
            throw new NotImplementedException();
        }
    }
}
