﻿#region Header
// Copyright 2007-2011, Jerónimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.ComponentModel;
using NetJsWire.JsonRpcHandler;
using Newtonsoft.Json;

namespace NetJsWire.Widgets.Config
{
    [JsonObject( MemberSerialization = MemberSerialization.OptIn )]
    public class ArrayStoreConfigOptions : StoreConfigOptions
    {
        private int _idIndex;

        public ArrayStoreConfigOptions( IWidget ownerWidget )
            : base( ownerWidget )
        {
            _xtype = "arraystore";
        }

        [DefaultValue( "arraystore" )]
        [JsonProperty( "xtype", DefaultValueHandling = DefaultValueHandling.Include )]
        public override string Xtype
        {
            get { return base.Xtype; }
            set { }
        }

        [Browsable( true )]
        [DefaultValue( 0 )]
        [Description( "The subscript within row Array that provides an ID for the Record." )]
        [JsonProperty( "idIndex", DefaultValueHandling = DefaultValueHandling.Ignore )]
        public override int IdIndex
        {
            get { return _idIndex; }
            set
            {
                SetValue( "idIndex", value, 0, _idIndex );
                _idIndex = value;
            }
        }
    }
}
