#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Runtime.InteropServices;
using NetJsWire.JsonRpcHandler;
using NetJsWire.Widgets.Config;
using NetJsWire.Widgets.Renderers.Core;
using NetJsWire.Widgets.Ui.Layout;
using Panel = System.Windows.Forms.Panel;

namespace NetJsWire.Widgets.Core
{
    ///<summary>
    ///  The base container for the application.
    ///</summary>
    [ ToolboxItem( false ),
      Designer(
          "System.Windows.Forms.Design.ControlDesigner, System.Design, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a"
          ), ComVisible( false ), ClassInterface( ClassInterfaceType.AutoDispatch ),
      Designer(
          "System.Windows.Forms.Design.UserControlDocumentDesigner, System.Design, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a"
          , typeof (IRootDesigner) ), DefaultEvent( "Load" ), DesignerCategory( "UserControl" ) ]
    public class ClientDocument : Panel, IClientDocument, ITopLevelContainer, IWidgetContainer
    {
        private IConfigOptions _configOptions;
        private LayoutType _layout = LayoutType.Absolute;
        protected IRenderer _renderer;
        protected int innerHeight;
        protected int innerWidth;
        protected string pageTitle;

        ///<summary>
        ///  Initializes a new instance of ClientDocument.
        ///</summary>
        public ClientDocument()
        {
            _renderer = new Widget( this );
            WebApplication.Instance.HasChanges = true;
        }

        ///<summary>
        ///  The InnerWidth of the Viewport.
        ///</summary>
        public int InnerWidth
        {
            get { return innerWidth; }
            set { innerWidth = value; }
        }

        ///<summary>
        ///  The InnerHeight of the Viewport.
        ///</summary>
        public int InnerHeight
        {
            get { return innerHeight; }
            set { innerHeight = value; }
        }

        #region IClientDocument Members

        public void Delete()
        {
            WebApplication.Instance.AddSingleChange( "delete " + Renderer.ClientID + ";" );
            Dispose( true );
        }

        public string PageTitle
        {
            get { return pageTitle; }
            set
            {
                pageTitle = value;
                Renderer.AddCommand( "setTitle", value );
                WebApplication.Instance.HasChanges = true;
            }
        }

        public bool IsIE { get; set; }

        public IRenderer Renderer
        {
            get { return _renderer; }
        }

        public void On( string name, string function )
        {
            Renderer.AddClientEvent( name, function );
        }

        public void AddClass( string cls )
        {
            Renderer.AddCommand( "addClass", cls );
        }

        public void RemoveClass( string cls )
        {
            Renderer.AddCommand( "removeClass", cls );
        }

        [ DefaultValue( false ) ]
        [ Description(
            "If the framework should generate a global Javascript variable that points to the Widget, the name of the variable will be the same as ClientID property of the Renderer."
            ) ]
        public bool GenerateClientMember { get; set; }

        [ RefreshProperties( RefreshProperties.All ),
          DesignerSerializationVisibility( DesignerSerializationVisibility.Content ), Category( "Config" ) ]
        public IConfigOptions ConfigOptions
        {
            get
            {
                if ( _configOptions == null )
                {
                    _configOptions = new ComponentConfigOptions( this );
                }
                return _configOptions;
            }
        }

        #endregion

        #region IWidgetContainer Members

        ///<summary>
        ///  The layout type to use, as default we use absolute layout so we can have absolutely positioned elements.
        ///</summary>
        [ DefaultValue( ( int ) LayoutType.Absolute ) ]
        public new LayoutType Layout
        {
            get { return _layout; }
            set { _layout = value; }
        }

        [Category( "LayoutConfig" )]
        public string LayoutConfig { get; set; }

        #endregion

        public event EventHandler Poll;

        ~ClientDocument()
        {
            Dispose( true );
        }

        protected override void Dispose( bool disposing )
        {
            WebApplication.Instance.RemoveObject( ( ( Widget ) Renderer ).InternalID );
            base.Dispose( disposing );
        }

        public void StartPoll()
        {
            Renderer.AddCommand( "StartPoll", ( ( Widget ) Renderer ).InternalID );
        }
    }
}