﻿#region Header
// Copyright 2007-2011, Jerónimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.Collections.Generic;
using System.ComponentModel;
using NetJsWire.Widgets.Config;
using NetJsWire.Widgets.Core;
using Newtonsoft.Json;

namespace NetJsWire.Widgets.Data
{
    [JsonObject( "dataProxy" )]
    public abstract class DataProxy : Component
    {
        private Dictionary<string, string> _api = new Dictionary<string, string> {};

        private List<Listener> _listeners = new List<Listener> { Capacity = 1 };

        [JsonProperty( "api", DefaultValueHandling = DefaultValueHandling.Ignore ), JsonConverter( typeof( NotQuotedJsonStringConverter ) )]
        [DefaultValue( typeof( Dictionary<string, string> ), "{}" )]
        [TypeConverter( typeof( CollectionConverter ) )]
        public Dictionary<string, string> Api
        {
            get { return _api; }
            set { _api = value; }
        }

        [JsonProperty( "listeners", DefaultValueHandling = DefaultValueHandling.Ignore )]
        //[DefaultValue( typeof( List<Listener> ), "{Capacity = 1}" )]
        [TypeConverter( typeof( CollectionConverter ) )]
        public List<Listener> Listeners
        {
            get { return _listeners; }
            set { _listeners = value; }
        }

        [JsonProperty( "restFul", DefaultValueHandling = DefaultValueHandling.Ignore )]
        [DefaultValue( false )]
        public bool RestFul { get; set; }

        public bool ShouldSerializeListeners()
        {
            return Listeners != null && Listeners.Count > 0;
        }

        public bool ShouldSerializeApi()
        {
            return Api != null && Api.Count > 0;
        }
    }
}
