﻿#region Header
// Copyright 2007-2011, Jerónimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.Drawing;
using System.Windows.Forms;
using NetJsWire.JsonRpcHandler;
using NetJsWire.JsonRpcHandler.Attributes;
using NetJsWire.Widgets.Core;
using NetJsWire.Widgets.Renderers.Core;

namespace NetJsWire.Widgets.Renderers.Basic
{
    [ ClassName( "Ext.form.Label" ) ]
    internal class Label : Widget, IRenderer
    {
        public Label( object baseObject )
            : base( baseObject )
        {
        }

        #region IRenderer Members

        public override void Render()
        {
            if ( !Rendered )
            {
                if ( BaseParent is IWidgetContainer )
                {
                    SetAttribute( ControlNode, "isInContainer", true );
                    SetConstructionProperty( "isInContainer", true );
                }
                else
                {
                    SetStyleProperty( "position", "absolute" );
                    SetConstructionProperty( "renderTo", ClientParentToRender );
                }

                EnsureParentNodePosition();

                SetConstructionProperty( "html", ( ( Control ) Base ).Text );
                SetConstructionProperty( "width", ( ( Control ) Base ).Width );

                if ( null != ( Base as Ui.Basic.Label ).LabelFor )
                {
                    SetConstructionProperty( "forId",
                                             ( ( Base as Ui.Basic.Label ).LabelFor as IWidget ).Renderer.InternalID );
                }

                var cAlign = ( Base as Ui.Basic.Label ).TextAlign;
                if ( cAlign == ContentAlignment.BottomCenter || cAlign == ContentAlignment.MiddleCenter ||
                     cAlign == ContentAlignment.TopCenter )
                    SetConstructionProperty( "cls", "x-text-center" );
                else if ( cAlign == ContentAlignment.BottomRight || cAlign == ContentAlignment.MiddleRight ||
                          cAlign == ContentAlignment.TopRight )
                    SetConstructionProperty( "cls", "x-text-right" );
            }
            base.Render();
        }

        #endregion
    }
}