#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.Collections.Generic;
using System.ComponentModel;
using System.Web.UI.WebControls;
using System.Windows.Forms;
using NetJsWire.JsonRpcHandler;
using NetJsWire.JsonRpcHandler.Attributes;
using NetJsWire.JsonRpcHandler.Util;
using NetJsWire.Widgets.Config;
using NetJsWire.Widgets.Core;
using NetJsWire.Widgets.Ui.Layout;

namespace NetJsWire.Widgets.Ui.Form
{
    [Renderer( typeof( Renderers.Form.FieldSet ) )]
    public partial class FieldSet : GroupBox, IWidget, IWidgetContainer
    {
        private readonly IRenderer _renderer;
        private IConfigOptions _configOptions;
        private List<CssStyle> styles = new List<CssStyle>();

        public FieldSet()
        {
            InitializeComponent();
            _renderer = Helper.GetRenderer( this );
        }

        public FieldSet( IContainer container )
        {
            container.Add( this );

            InitializeComponent();
            _renderer = Helper.GetRenderer( this );
        }

        public Unit ClientWidth { get; set; }

        [DefaultValue( false )]
        public bool CheckBoxToggle { get; set; }

        [DefaultValue( false )]
        public bool Collapsible { get; set; }

        [DesignerSerializationVisibility( DesignerSerializationVisibility.Content )]
        public List<CssStyle> Styles
        {
            get { return styles; }
            set { styles = value; }
        }

        #region IWidget Members

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Advanced )]
        public IRenderer Renderer
        {
            get { return _renderer; }
        }

        public void On( string name, string function )
        {
            Renderer.AddClientEvent( name, function );
        }

        public void AddClass( string cls )
        {
            Renderer.AddCommand( "addClass", cls );
        }

        public void RemoveClass( string cls )
        {
            Renderer.AddCommand( "removeClass", cls );
        }

        [DefaultValue( false )]
        [Description(
           "If the framework should generate a global Javascript variable that points to the Widget, the name of the variable will be the same as ClientID property of the Renderer."
           )]
        public bool GenerateClientMember { get; set; }

        [RefreshProperties( RefreshProperties.All ),
         DesignerSerializationVisibility( DesignerSerializationVisibility.Content ), Category( "Config" )]
        public IConfigOptions ConfigOptions
        {
            get
            {
                if ( _configOptions == null )
                {
                    _configOptions = new PanelConfigOptions( this ) { Xtype = "fieldset", Layout = LayoutType.Form };
                }
                return _configOptions;
            }
        }

        #endregion

        #region IWidgetContainer Members

        [DefaultValue( ( int ) LayoutType.Form )]
        [Category( "Layout" )]
        public new LayoutType Layout
        {
            get { return ConfigOptions.Layout; }
            set { ConfigOptions.Layout = value; }
        }

        [DefaultValue( ( string ) null )]
        [Category( "Layout" )]
        public string LayoutConfig
        {
            get { return ConfigOptions.LayoutConfig; }
            set { ConfigOptions.LayoutConfig = value; }
        }

        #endregion
    }
}