#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.ComponentModel;
using System.Windows.Forms;
using NetJsWire.JsonRpcHandler;
using NetJsWire.JsonRpcHandler.Attributes;
using NetJsWire.JsonRpcHandler.Util;
using NetJsWire.Widgets.Config;

namespace NetJsWire.Widgets.Ui.Form
{
    [Renderer( typeof( Renderers.Form.FileUploadField ) )]
    public partial class FileUploadField : UserControl, IWidget, IFormField
    {
        private readonly IRenderer _renderer;
        private bool _allowBlank = true;
        private IConfigOptions _configOptions;

        public FileUploadField()
        {
            InitializeComponent();

            _renderer = Helper.GetRenderer( this );
        }

        public string ButtonText
        {
            get { return btnSearch.Text; }
            set { btnSearch.Text = value; }
        }

        [DefaultValue( true )]
        public bool AllowBlank
        {
            get
            {
                return _allowBlank;
            }
            set
            {
                if ( !Renderer.Rendered )
                {
                    if ( !value )
                        Renderer.SetConstructionProperty( "allowBlank", value );
                    else
                        Renderer.RemoveConstructionProperty( "allowBlank" );
                }
                else
                {
                    if ( _allowBlank != value )
                        Renderer.SetEqualProperty( "allowBlank", value );
                }

                _allowBlank = value;
            }
        }

        #region IFormField Members

        public string FieldLabel { get; set; }

        #endregion

        #region IWidget Members

        public IRenderer Renderer
        {
            get { return _renderer; }
        }

        public void On( string name, string function )
        {
            throw new NotImplementedException();
        }

        public void AddClass( string cls )
        {
            throw new NotImplementedException();
        }

        public void RemoveClass( string cls )
        {
            throw new NotImplementedException();
        }

        [DefaultValue( false )]
        [Description(
           "If the framework should generate a global Javascript variable that points to the Widget, the name of the variable will be the same as ClientID property of the Renderer."
           )]
        public bool GenerateClientMember { get; set; }

        [RefreshProperties( RefreshProperties.All ),
         DesignerSerializationVisibility( DesignerSerializationVisibility.Content ), Category( "Config" )]
        public IConfigOptions ConfigOptions
        {
            get
            {
                if ( _configOptions == null )
                {
                    _configOptions = new FieldConfigOptions( this );
                }
                return _configOptions;
            }
        }

        #endregion
    }
}