#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.ComponentModel;
using System.Drawing;
using System.Web.UI.WebControls;
using System.Windows.Forms;
using NetJsWire.JsonRpcHandler;
using NetJsWire.JsonRpcHandler.Attributes;
using NetJsWire.JsonRpcHandler.Util;
using NetJsWire.Widgets.Config;
using NetJsWire.Widgets.Core;

namespace NetJsWire.Widgets.Ui.Layout
{
    //[Designer("System.Windows.Forms.Design.ControlDesigner, System.Design, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a"), Designer("System.Windows.Forms.Design.UserControlDocumentDesigner, System.Design, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", typeof(IRootDesigner)), DefaultEvent("Load"), DesignerCategory("UserControl")]
    //[Designer("System.Windows.Forms.Design.UserControlDocumentDesigner, System.Design, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", typeof(IRootDesigner)), Designer("System.Windows.Forms.Design.ControlDesigner, System.Design, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a"), DesignerCategory("UserControl"), DefaultEvent("Load")]
    [ Renderer( typeof (Renderers.Layout.BorderLayout) ) ]
    public partial class BorderLayout : ContainerControl, IWidget, IContentPanel, IWidgetContainer
    {
        private readonly IRenderer _renderer;
        private IConfigOptions _configOptions;
        private string _region = "center";
        private string _title = string.Empty;
        private Unit clientWidth;
        private bool closable;

        public BorderLayout()
        {
            InitializeComponent();
            _renderer = Helper.GetRenderer( this );
        }

        public BorderLayout( IContainer container )
        {
            container.Add( this );

            InitializeComponent();
            _renderer = Helper.GetRenderer( this );
        }

        public Unit ClientWidth
        {
            get { return clientWidth; }
            set
            {
                clientWidth = value;
                if ( !value.IsEmpty )
                {
                    if ( clientWidth.Type == UnitType.Pixel )
                        Size = new Size( ( int ) clientWidth.Value, Size.Height );
                    else if ( clientWidth.Type == UnitType.Percentage )
                    {
                        if ( Parent != null )
                        {
                            double _w = Parent.ClientSize.Width/100;
                            _w = _w*clientWidth.Value;
                            Width = ( int ) _w;
                        }
                    }
                    else
                    {
                        throw new Exception( "You can only use Pixel and Percent values" );
                    }
                }
            }
        }

        public Unit ClientHeight { get; set; }

        public override bool AutoSize
        {
            get { return base.AutoSize; }
            set
            {
                base.AutoSize = value;
                if ( value )
                {
                    if ( Parent is ClientDocument )
                    {
                        Width = Parent.ClientSize.Width;
                        Height = Parent.ClientSize.Height;
                        Top = 0;
                        Left = 0;
                    }
                }
            }
        }

        public string Title
        {
            get { return _title; }
            set
            {
                _title = value;
                if ( !Renderer.Rendered && !string.IsNullOrEmpty( value ) )
                {
                    Renderer.SetConstructionProperty( "titlebar", true );
                    Renderer.SetConstructionProperty( "title", value );
                }
            }
        }

        public bool Closable
        {
            get { return closable; }
            set
            {
                closable = value;
                if ( !Renderer.Rendered && value )
                    Renderer.SetConstructionProperty( "closable", value );
            }
        }

        #region IContentPanel Members

        [ DefaultValue( "center" ) ]
        public new string Region
        {
            get { return _region; }
            set { _region = value; }
        }

        [ DefaultValue( ( int ) LayoutType.Border ) ]
        public LayoutType Layout
        {
            get { return LayoutType.Border; }
            set { ; }
        }

        #endregion

        #region IWidget Members

        [ Browsable( false ), EditorBrowsable( EditorBrowsableState.Advanced ) ]
        public IRenderer Renderer
        {
            get { return _renderer; }
        }

        public void On( string name, string function )
        {
            Renderer.AddClientEvent( name, function );
        }

        public void AddClass( string cls )
        {
            Renderer.AddCommand( "addClass", cls );
        }

        public void RemoveClass( string cls )
        {
            Renderer.AddCommand( "removeClass", cls );
        }

        [ DefaultValue( false ) ]
        [ Description(
            "If the framework should generate a global Javascript variable that points to the Widget, the name of the variable will be the same as ClientID property of the Renderer."
            ) ]
        public bool GenerateClientMember { get; set; }

        [ RefreshProperties( RefreshProperties.All ),
          DesignerSerializationVisibility( DesignerSerializationVisibility.Content ), Category( "Config" ) ]
        public IConfigOptions ConfigOptions
        {
            get
            {
                if ( _configOptions == null )
                {
                    _configOptions = new PanelConfigOptions( this );
                }
                return _configOptions;
            }
        }

        #endregion

        #region IWidgetContainer Members

        [Category( "LayoutConfig" )]
        public string LayoutConfig { get; set; }

        #endregion

        public void Add( string region, Panel panel )
        {
            panel.Region = region;
            CenterPanel.Controls.Add( panel );
            panel.Parent = CenterPanel;
            CenterPanel.Renderer.AddCommand( "add", panel.Renderer.ClientID );
        }

        public void Add( string region, BorderLayout layout )
        {
            var __closable = "";
            if ( layout.Closable )
                __closable = ", closable:true";
            layout.Region = region;
            switch ( region )
            {
                case "center":
                    if ( CenterPanel == null )
                    {
                        throw new Exception( "You cannot add to an empty region" );
                    }
                    layout.Renderer.SetConstructionProperty( "closable", true );
                    CenterPanel.Controls.Add( layout );
                    layout.Parent = CenterPanel;
                    break;
                default:
                    break;
            }
            //layout.Renderer.Render();
            //Renderer.AddCommand("add", layout.Renderer.ClientID);
            //Renderer.AddCommand("doLayout", null);
        }
    }
}