﻿#region Header
// Copyright 2007-2011, Jerónimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.ComponentModel;
using System.Windows.Forms;
using NetJsWire.JsonRpcHandler;
using NetJsWire.Util;

namespace NetJsWire.Widgets.Ui.Layout.Config
{
    [ Category( "LayoutConfig" ) ]
    [ TypeConverter( typeof (ExpandableObjectConverter) ) ]
    [ Serializable ]
    public class BoxLayoutConfig : LayoutConfig
    {
        private Padding _defaulMargins = Padding.Empty;
        private BoxPack _pack = BoxPack.Start;
        private Padding _padding = Padding.Empty;

        public BoxLayoutConfig( IRenderer renderer )
            : base( renderer )
        {
        }

        public Padding DefaultMargins
        {
            get { return _defaulMargins; }
            set
            {
                value = SetNegativePaddingToZero( value );

                if ( !_renderer.Rendered )
                {
                    if ( null != value && value.All != 0 ||
                         ( value.Top != 0 || value.Right != 0 || value.Bottom != 0 || value.Left != 0 ) )
                    {
                        _renderer.SetLayoutConfigProperty( "defaultMargins",
                                                           string.Format(
                                                               "{{top:{0}, right:{1}, bottom:{2}, left:{3}}}", value.Top,
                                                               value.Right, value.Bottom, value.Left ) );
                    }
                    else
                    {
                        _renderer.RemoveLayoutConfigProperty( "defaultMargins" );
                    }
                }
                else
                {
                    _renderer.RemoveLayoutConfigProperty( "defaultMargins" );
                }

                //if ( value != _defaulMargins )
                _defaulMargins = value;
            }
        }

        public Padding Padding
        {
            get { return _padding; }
            set
            {
                value = SetNegativePaddingToZero( value );

                if ( !_renderer.Rendered )
                {
                    if ( null != value && value.All != 0 ||
                         ( value.Top != 0 || value.Right != 0 || value.Bottom != 0 || value.Left != 0 ) )
                    {
                        _renderer.SetLayoutConfigProperty( "padding",
                                                           string.Format( "{0} {1} {2} {3}", value.Top, value.Right,
                                                                          value.Bottom, value.Left ) );
                    }
                    else
                    {
                        _renderer.RemoveLayoutConfigProperty( "padding" );
                    }
                }
                else
                {
                    _renderer.RemoveLayoutConfigProperty( "padding" );
                }

                //if ( value != _padding )
                _padding = value;
            }
        }

        [ DefaultValue( BoxPack.Start ) ]
        public new BoxPack Pack
        {
            get { return _pack; }
            set
            {
                if ( !_renderer.Rendered )
                {
                    if ( BoxPack.Start != value )
                    {
                        _renderer.SetLayoutConfigProperty( "pack", value.ToString().ToLower() );
                    }
                    else
                    {
                        _renderer.RemoveLayoutConfigProperty( "pack" );
                    }
                }
                else
                {
                    _renderer.RemoveLayoutConfigProperty( "pack" );
                }

                _pack = value;
            }
        }
    }
}