﻿#region Header
// Copyright 2007-2011, Jerónimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.ComponentModel;
using NetJsWire.JsonRpcHandler;

namespace NetJsWire.Widgets.Ui.Layout.Config
{
    [ Category( "LayoutConfig" ) ]
    [ TypeConverter( typeof (ExpandableObjectConverter) ) ]
    [ Serializable ]
    public sealed class CardLayoutConfig : LayoutConfig
    {
        private bool _deferredRender;
        private bool _layoutOnCardChange;

        public CardLayoutConfig( IRenderer renderer )
            : base( renderer )
        {
        }

        /// <summary>
        ///   True to render each contained item at the time it becomes active, false to render all contained items as soon as the layout is rendered (defaults to false).
        ///   If there is a significant amount of content or a lot of heavy controls being rendered into panels that are not displayed by default, setting this to true might improve performance.
        /// </summary>
        [ DefaultValue( false ) ]
        public bool DeferredRender
        {
            get { return _deferredRender; }
            set
            {
                if ( !_renderer.Rendered )
                {
                    if ( value )
                    {
                        _renderer.SetLayoutConfigProperty( "deferredRender", value );
                    }
                    else
                    {
                        _renderer.RemoveLayoutConfigProperty( "deferredRender" );
                    }
                }
                else
                {
                    _renderer.RemoveLayoutConfigProperty( "deferredRender" );
                }

                _deferredRender = value;
            }
        }

        /// <summary>
        ///   True to force a layout of the active item when the active card is changed. Defaults to false.
        /// </summary>
        [ DefaultValue( false ) ]
        public bool LayoutOnCardChange
        {
            get { return _layoutOnCardChange; }
            set
            {
                if ( !_renderer.Rendered )
                {
                    if ( value )
                    {
                        _renderer.SetLayoutConfigProperty( "layoutOnCardChange", value );
                    }
                    else
                    {
                        _renderer.RemoveLayoutConfigProperty( "layoutOnCardChange" );
                    }
                }
                else
                {
                    _renderer.RemoveLayoutConfigProperty( "layoutOnCardChange" );
                }

                _layoutOnCardChange = value;
            }
        }
    }
}