﻿#region Header
// Copyright 2007-2011, Jerónimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace NetJsWire.Widgets.Ui.List
{
    public class ListViewTypeDescriptor : CustomTypeDescriptor
    {
        private static readonly List<string> _hiddenProperties = new List<string> { 
                                "AccessibleDescription", 
                                "AccessibleName", 
                                "AccessibleRole", 
                                "AllowDrop", 
                                "ContextMenu", 
                                "AllowUserToAddRows", 
                                "AllowUserToDeleteRows", 
                                "AllowUserToOrderColumns",
                                "AllowUserToResizeColumns",
                                "AllowUserToResizeRows",
                                "AlternatingRowsDefaultCellStyle",
                                "AutoSizeColumnsMode",
                                "AutoSizeRowsMode",
                                "BackgroundColor",
                                "BorderStyle"
            };

        /// <summary>
        ///   Returns the properties for this instance of a component.
        /// </summary>
        /// <returns>
        ///   A <see cref = "T:System.ComponentModel.PropertyDescriptorCollection" /> that represents the properties for this component instance.
        /// </returns>
        public override PropertyDescriptorCollection GetProperties()
        {
            return GetProperties( null );
        }

        /// <summary>
        ///   Returns the properties for this instance of a component using the attribute array as a filter.
        /// </summary>
        /// <returns>
        ///   A <see cref = "T:System.ComponentModel.PropertyDescriptorCollection" /> that represents the filtered properties for this component instance.
        /// </returns>
        /// <param name = "attributes">An array of type <see cref = "T:System.Attribute" /> that is used as a filter. 
        /// </param>
        public override PropertyDescriptorCollection GetProperties( Attribute[] attributes )
        {
            var filtering = ( attributes != null && attributes.Length > 0 );
            var props = new PropertyDescriptorCollection( null );
            foreach ( PropertyDescriptor prop in TypeDescriptor.GetProperties( this, attributes, true ) )
            {
                var cn = GetClassName();

                if ( prop.IsBrowsable && !_hiddenProperties.Contains( prop.DisplayName ) && !_hiddenProperties.Contains( prop.Name ) )
                {
                    props.Add( prop );
                }
            }

            return props;
        }
    }
}
