#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.ComponentModel;
using NetJsWire.JsonRpcHandler;
using NetJsWire.JsonRpcHandler.Attributes;
using NetJsWire.Widgets.Config;

namespace NetJsWire.Widgets.Ui.MessageBox
{
    [ Renderer( typeof (Renderers.MessageBox.MessageBox) ) ]
    public partial class MessageBox : Window.Window
    {
        private readonly string _message = string.Empty;
        private IConfigOptions _configOptions;

        public MessageBox()
        {
            InitializeComponent();
        }

        public MessageBox( string message ) : this()
        {
            _message = message;
            WebApplication.Instance.HasChanges = true;
            //btnAccept.Left = ((Width / 2) - (btnAccept.Width / 2)) - 4;
        }

        public MessageBox( string caption, string message )
            : this( message )
        {
            if ( !string.IsNullOrEmpty( caption ) )
                Text = caption;
            else
                Text = "Pregunta";
        }

        public MessageBox( string caption, string message, bool showCancel )
            : this( caption, message )
        {
            if ( !string.IsNullOrEmpty( caption ) )
                Text = caption;
            else
                Text = "Pregunta";
            _message = message;
            //btnAccept.Left = ((Width / 2) - (btnAccept.Width + 5)) - 4;
            if ( showCancel )
            {
                //btnCancel.Visible = true;
                //btnCancel.Left = btnAccept.Left + btnAccept.Width + 5;
                IsConfirm = true;
            }
            //btnAccept.Top = btnCancel.Top;
            Width += 14;
        }

        public bool IsConfirm { get; set; }

        public string Message
        {
            get { return _message; }
        }

        [ DefaultValue( false ) ]
        [ Description(
            "If the framework should generate a global Javascript variable that points to the Widget, the name of the variable will be the same as ClientID property of the Renderer."
            ) ]
        public bool GenerateClientMember { get; set; }

        [ RefreshProperties( RefreshProperties.All ),
          DesignerSerializationVisibility( DesignerSerializationVisibility.Content ), Category( "Config" ) ]
        public IConfigOptions ConfigOptions
        {
            get
            {
                if ( _configOptions == null )
                {
                    _configOptions = new ComponentConfigOptions( this );
                }
                return _configOptions;
            }
        }

        public event EventHandler Accept;
        public event EventHandler Cancel;

        private void btnAccept_Click( object sender, EventArgs e )
        {
            if ( Accept != null )
                Accept( this, EventArgs.Empty );
            Close();
        }

        private void btnCancel_Click( object sender, EventArgs e )
        {
            if ( Cancel != null )
                Cancel( this, EventArgs.Empty );
            Close();
        }

        public static void Alert( string text )
        {
            var mb = new MessageBox( text );
            //mb.ShowDialog();
        }

        public static void Alert( string caption, string text )
        {
            var mb = new MessageBox( caption, text );
            //mb.ShowDialog();
        }

        public static void Confirm( string text )
        {
            var mb = new MessageBox( null, text, true );
            //mb.ShowDialog();
        }

        public static void Confirm( string caption, string text )
        {
            var mb = new MessageBox( caption, text, true );
            //mb.ShowDialog();
        }

        private void MessageBox_appear( object sender, EventArgs e )
        {
            //btnAccept.Focus();
        }
    }
}