#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.ComponentModel;
using System.Windows.Forms;
using NetJsWire.JsonRpcHandler;
using NetJsWire.JsonRpcHandler.Attributes;
using NetJsWire.JsonRpcHandler.Util;
using NetJsWire.Widgets.Config;
using NetJsWire.Widgets.Core;
using NetJsWire.Widgets.Ui.Layout;

namespace NetJsWire.Widgets.Ui.Tabs
{
    //[System.ComponentModel.Designer(typeof(NetJsWire.Design.Tabs.TabControlDesigner))]
    [ Renderer( typeof (Renderers.Tabs.TabControl) ) ]
    public partial class TabControl : System.Windows.Forms.TabControl, IWidget, IWidgetContainer
    {
        private readonly IRenderer _renderer;
        private PanelConfigOptions _configOptions;
        private LayoutType _layout = LayoutType.Card;

        public TabControl()
        {
            InitializeComponent();
            _renderer = Helper.GetRenderer( this );

            ControlAdded += TabControl_ControlAdded;
        }

        public TabControl( IContainer container )
        {
            container.Add( this );

            InitializeComponent();
            _renderer = Helper.GetRenderer( this );

            ControlAdded += TabControl_ControlAdded;
        }

        #region IWidget Members

        [ Browsable( false ), EditorBrowsable( EditorBrowsableState.Advanced ) ]
        public IRenderer Renderer
        {
            get { return _renderer; }
        }

        public void On( string name, string function )
        {
            Renderer.AddClientEvent( name, function );
        }

        public void AddClass( string cls )
        {
            Renderer.AddCommand( "addClass", cls );
        }

        public void RemoveClass( string cls )
        {
            Renderer.AddCommand( "removeClass", cls );
        }

        [ DefaultValue( false ) ]
        [ Description(
            "If the framework should generate a global Javascript variable that points to the Widget, the name of the variable will be the same as ClientID property of the Renderer."
            ) ]
        public bool GenerateClientMember { get; set; }

        [ RefreshProperties( RefreshProperties.All ),
          DesignerSerializationVisibility( DesignerSerializationVisibility.Content ), Category( "Config" ) ]
        public IConfigOptions ConfigOptions
        {
            get
            {
                if ( _configOptions == null )
                {
                    _configOptions = new TabPanelConfigOptions( this ) { Xtype = "tabpanel" };
                }
                return _configOptions;
            }
        }

        #endregion

        #region Hidden members

        [ Bindable( false ), EditorBrowsable( EditorBrowsableState.Never ), Browsable( false ) ]
        public new bool Multiline
        {
            get { return base.Multiline; }
            set { base.Multiline = value; }
        }

        #endregion

        #region IWidgetContainer Members

        [ DefaultValue( LayoutType.Card ) ]
        public new LayoutType Layout
        {
            get { return _layout; }
            set
            {
                //_layout = value;
            }
        }

        [Category( "LayoutConfig" )]
        public string LayoutConfig { get; set; }

        #endregion

        private void TabControl_ControlAdded( object sender, ControlEventArgs e )
        {
            if ( e.Control is System.Windows.Forms.TabPage && !( e.Control is TabPage ) )
            {
                TabPages.Remove( e.Control as System.Windows.Forms.TabPage );
                Controls.Remove( e.Control );
                Container.Remove( e.Control );

                var newTab = new TabPage();
                newTab.Name = "tabPage" + ( TabPages.Count + 1 );
                newTab.Text = newTab.Name;

                TabPages.Add( newTab );
            }
        }

        public void AddTab( TabPage newTab )
        {
            ( Renderer as Renderers.Tabs.TabControl ).AddTab( newTab, true );
        }
    }
}