#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.ComponentModel;
using System.Windows.Forms;
using NetJsWire.JsonRpcHandler;
using NetJsWire.JsonRpcHandler.Attributes;
using NetJsWire.JsonRpcHandler.Util;
using NetJsWire.Widgets.Config;

namespace NetJsWire.Widgets.Ui.ToolBar
{
    ///<summary>
    ///</summary>
    [ Renderer( typeof (Renderers.ToolBar.ColorMenu) ) ]
    public partial class ColorMenu : ToolStripMenuItem, IWidget
    {
        private readonly IRenderer _renderer;
        private IConfigOptions _configOptions;
        private bool _enabled = true;

        private string _text;

        public ColorMenu()
        {
            InitializeComponent();
            _renderer = Helper.GetRenderer( this );
        }

        public ColorMenu( IContainer container )
        {
            container.Add( this );

            InitializeComponent();
            _renderer = Helper.GetRenderer( this );
        }

        public override bool Enabled
        {
            get { return _enabled; }
            set
            {
                if ( DesignMode )
                    base.Enabled = value;
                _enabled = value;
                if ( !Renderer.Rendered )
                {
                    Renderer.SetConstructionProperty( "disabled", !value );
                }
                else
                {
                    if ( value )
                        Renderer.CallCommand( "enable", null );
                    else
                        Renderer.CallCommand( "disable", null );
                }
            }
        }

        public override string Text
        {
            get { return _text; }
            set
            {
                if ( DesignMode )
                    base.Text = value;
                _text = value;
                if ( Renderer.Rendered )
                {
                    Renderer.CallCommand( "setText", value );
                }
            }
        }

        #region IWidget Members

        [ Browsable( false ), EditorBrowsable( EditorBrowsableState.Advanced ) ]
        public IRenderer Renderer
        {
            get { return _renderer; }
        }

        public void On( string name, string function )
        {
            Renderer.AddClientEvent( name, function );
        }

        public void AddClass( string cls )
        {
            Renderer.AddCommand( "addClass", cls );
        }

        public void RemoveClass( string cls )
        {
            Renderer.AddCommand( "removeClass", cls );
        }

        [ DefaultValue( false ) ]
        [ Description(
            "If the framework should generate a global Javascript variable that points to the Widget, the name of the variable will be the same as ClientID property of the Renderer."
            ) ]
        public bool GenerateClientMember { get; set; }

        [ RefreshProperties( RefreshProperties.All ),
          DesignerSerializationVisibility( DesignerSerializationVisibility.Content ), Category( "Config" ) ]
        public IConfigOptions ConfigOptions
        {
            get
            {
                if ( _configOptions == null )
                {
                    _configOptions = new ComponentConfigOptions( this );
                    _configOptions.Xtype = "colormenu";
                }
                return _configOptions;
            }
        }

        #endregion

        ///<summary>
        ///</summary>
        public new event EventHandler Click;

        ///<summary>
        ///</summary>
        public event EventHandler ColorSelected;
    }
}