﻿//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Author: Jeronimo Milea
//
// Copyright 2007-2011 - Jeronimo Milea

using System;
using System.Collections.Generic;
using System.Text;
using JSONRpcHandler;
using System.Runtime.Serialization;
using System.Collections.Specialized;
using System.Collections;
using System.Globalization;
using TreeView = NetJsWire.Widgets.Ui.Tree.TreePanel;
using System.ComponentModel;
using System.Drawing;
using System.Security.Permissions;
using System.Windows.Forms;
using System.Drawing.Design;
namespace NetJsWire.Widgets.Ui.Tree
{
    [Serializable, DefaultProperty("Text"), TypeConverter(typeof(TreeNodeConverter))]
    public class TreeNode : MarshalByRefObject, ICloneable, ISerializable, IWidget
    {
        // Fields
        private const int ALLOWEDIMAGES = 14;
        private const int CHECKED = 0x2000;
        internal int childCount;
        internal TreeNode[] children;
        private bool collapseOnRealization;
        private System.Windows.Forms.ContextMenu contextMenu;
        private System.Windows.Forms.ContextMenuStrip contextMenuStrip;
        private bool expandOnRealization;
        internal IntPtr handle;
        private TreeNodeImageIndexer imageIndexer;
        internal int index;
        private static readonly int insertMask = 0x23;
        internal const int MAX_TREENODES_OPS = 200;
        internal string name;
        private TreeNodeCollection nodes;
        internal bool nodesCleared;
        internal TreeNode parent;
        internal System.Windows.Forms.OwnerDrawPropertyBag propBag;
        private TreeNodeImageIndexer selectedImageIndexer;
        private const int SHIFTVAL = 12;
        private TreeNodeImageIndexer stateImageIndexer;
        internal string text;
        private string toolTipText;
        private BitVector32 treeNodeState;
        private const int TREENODESTATE_isChecked = 1;
        internal TreeView treeView;
        private const int UNCHECKED = 0x1000;
        private object userData;

        // Methods
        public TreeNode()
        {
            this.toolTipText = "";
            this.treeNodeState = new BitVector32();
        }

        public TreeNode(string text)
            : this()
        {
            this.text = text;
        }

        internal TreeNode(TreeView treeView)
            : this()
        {
            this.treeView = treeView;
        }

        protected TreeNode(SerializationInfo serializationInfo, StreamingContext context)
            : this()
        {
            this.Deserialize(serializationInfo, context);
        }

        public TreeNode(string text, TreeNode[] children)
            : this()
        {
            this.text = text;
            this.Nodes.AddRange(children);
        }

        public TreeNode(string text, int imageIndex, int selectedImageIndex)
            : this()
        {
            this.text = text;
            this.ImageIndexer.Index = imageIndex;
            this.SelectedImageIndexer.Index = selectedImageIndex;
        }

        public TreeNode(string text, int imageIndex, int selectedImageIndex, TreeNode[] children)
            : this()
        {
            this.text = text;
            this.ImageIndexer.Index = imageIndex;
            this.SelectedImageIndexer.Index = selectedImageIndex;
            this.Nodes.AddRange(children);
        }

        internal int AddSorted(TreeNode node)
        {
            int index = 0;
            string text = node.Text;
            TreeView parentTreeView = this.TreeView;
            if (this.childCount > 0)
            {
                int num2;
                int childCount;
                int num4;
                if (parentTreeView.TreeViewNodeSorter == null)
                {
                    CompareInfo compareInfo = Application.CurrentCulture.CompareInfo;
                    if (compareInfo.Compare(this.children[this.childCount - 1].Text, text) <= 0)
                    {
                        index = this.childCount;
                    }
                    else
                    {
                        num2 = 0;
                        childCount = this.childCount;
                        while (num2 < childCount)
                        {
                            num4 = (num2 + childCount) / 2;
                            if (compareInfo.Compare(this.children[num4].Text, text) <= 0)
                            {
                                num2 = num4 + 1;
                            }
                            else
                            {
                                childCount = num4;
                            }
                        }
                        index = num2;
                    }
                }
                else
                {
                    IComparer treeViewNodeSorter = parentTreeView.TreeViewNodeSorter;
                    num2 = 0;
                    childCount = this.childCount;
                    while (num2 < childCount)
                    {
                        num4 = (num2 + childCount) / 2;
                        if (treeViewNodeSorter.Compare(this.children[num4], node) <= 0)
                        {
                            num2 = num4 + 1;
                        }
                        else
                        {
                            childCount = num4;
                        }
                    }
                    index = num2;
                }
            }
            node.SortChildren(parentTreeView);
            this.InsertNodeAt(index, node);
            return index;
        }

        public void BeginEdit()
        {
            if (this.handle != IntPtr.Zero)
            {
                TreeView wrapper = this.TreeView;
                if (!wrapper.LabelEdit)
                {
                    throw new InvalidOperationException("TreeNodeBeginEditFailed");
                }
                if (!wrapper.Focused)
                {
                    //wrapper.FocusInternal();
                }
                //UnsafeNativeMethods.SendMessage(new HandleRef(wrapper, wrapper.Handle), NativeMethods.TVM_EDITLABEL, 0, this.handle);
            }
        }

        internal void Clear()
        {
            bool flag = false;
            TreeView treeView = this.TreeView;
            try
            {
                if (treeView != null)
                {
                    //treeView.nodesCollectionClear = true;
                    if ((treeView != null) && (this.childCount > 200))
                    {
                        flag = true;
                        treeView.BeginUpdate();
                    }
                }
                while (this.childCount > 0)
                {
                    this.children[this.childCount - 1].Remove(true);
                }
                this.children = null;
                if ((treeView != null) && flag)
                {
                    treeView.EndUpdate();
                }
            }
            finally
            {
                if (treeView != null)
                {
                    //treeView.nodesCollectionClear = false;
                }
                this.nodesCleared = true;
            }
        }

        public virtual object Clone()
        {
            Type type = base.GetType();
            TreeNode node = null;
            if (type == typeof(TreeNode))
            {
                node = new TreeNode(this.text, this.ImageIndexer.Index, this.SelectedImageIndexer.Index);
            }
            else
            {
                node = (TreeNode)Activator.CreateInstance(type);
            }
            node.Text = this.text;
            node.Name = this.name;
            node.ImageIndexer.Index = this.ImageIndexer.Index;
            node.SelectedImageIndexer.Index = this.SelectedImageIndexer.Index;
            node.StateImageIndexer.Index = this.StateImageIndexer.Index;
            node.ToolTipText = this.toolTipText;
            node.ContextMenu = this.contextMenu;
            node.ContextMenuStrip = this.contextMenuStrip;
            if (!string.IsNullOrEmpty(this.ImageIndexer.Key))
            {
                node.ImageIndexer.Key = this.ImageIndexer.Key;
            }
            if (!string.IsNullOrEmpty(this.SelectedImageIndexer.Key))
            {
                node.SelectedImageIndexer.Key = this.SelectedImageIndexer.Key;
            }
            if (!string.IsNullOrEmpty(this.StateImageIndexer.Key))
            {
                node.StateImageIndexer.Key = this.StateImageIndexer.Key;
            }
            if (this.childCount > 0)
            {
                node.children = new TreeNode[this.childCount];
                for (int i = 0; i < this.childCount; i++)
                {
                    node.Nodes.Add((TreeNode)this.children[i].Clone());
                }
            }
            if (this.propBag != null)
            {
                node.propBag = OwnerDrawPropertyBag.Copy(this.propBag);
            }
            node.Checked = this.Checked;
            node.Tag = this.Tag;
            return node;
        }

        public void Collapse()
        {
            this.CollapseInternal(false);
        }

        public void Collapse(bool ignoreChildren)
        {
            this.CollapseInternal(ignoreChildren);
        }

        private void CollapseInternal(bool ignoreChildren)
        {
            TreeView tv = this.TreeView;
            bool flag = false;
            this.collapseOnRealization = false;
            this.expandOnRealization = false;
            if ((tv == null) || !tv.IsHandleCreated)
            {
                this.collapseOnRealization = true;
            }
            else
            {
                if (ignoreChildren)
                {
                    this.DoCollapse(tv);
                }
                else
                {
                    if (!ignoreChildren && (this.childCount > 0))
                    {
                        for (int i = 0; i < this.childCount; i++)
                        {
                            if (tv.SelectedNode == this.children[i])
                            {
                                flag = true;
                            }
                            this.children[i].DoCollapse(tv);
                            this.children[i].Collapse();
                        }
                    }
                    this.DoCollapse(tv);
                }
                if (flag)
                {
                    tv.SelectedNode = this;
                }
                tv.Invalidate();
                this.collapseOnRealization = false;
            }
        }

        protected virtual void Deserialize(SerializationInfo serializationInfo, StreamingContext context)
        {
            int num = 0;
            int num2 = -1;
            string text = null;
            int num3 = -1;
            string text2 = null;
            int num4 = -1;
            string text3 = null;
            foreach (SerializationEntry entry in serializationInfo)
            {
                switch (entry.Name)
                {
                    case "PropBag":
                        this.propBag = (OwnerDrawPropertyBag)serializationInfo.GetValue(entry.Name, typeof(OwnerDrawPropertyBag));
                        break;

                    case "Text":
                        this.Text = serializationInfo.GetString(entry.Name);
                        break;

                    case "Name":
                        this.Name = serializationInfo.GetString(entry.Name);
                        break;

                    case "IsChecked":
                        this.CheckedStateInternal = serializationInfo.GetBoolean(entry.Name);
                        break;

                    case "ImageIndex":
                        num2 = serializationInfo.GetInt32(entry.Name);
                        break;

                    case "SelectedImageIndex":
                        num3 = serializationInfo.GetInt32(entry.Name);
                        break;

                    case "ImageKey":
                        text = serializationInfo.GetString(entry.Name);
                        break;

                    case "SelectedImageKey":
                        text2 = serializationInfo.GetString(entry.Name);
                        break;

                    case "StateImageKey":
                        text3 = serializationInfo.GetString(entry.Name);
                        break;

                    case "StateImageIndex":
                        num4 = serializationInfo.GetInt32(entry.Name);
                        break;

                    case "ChildCount":
                        num = serializationInfo.GetInt32(entry.Name);
                        break;

                    case "UserData":
                        this.userData = entry.Value;
                        break;
                }
            }
            if (text != null)
            {
                this.ImageKey = text;
            }
            else if (num2 != -1)
            {
                this.ImageIndex = num2;
            }
            if (text2 != null)
            {
                this.SelectedImageKey = text2;
            }
            else if (num3 != -1)
            {
                this.SelectedImageIndex = num3;
            }
            if (text3 != null)
            {
                this.StateImageKey = text3;
            }
            else if (num4 != -1)
            {
                this.StateImageIndex = num4;
            }
            if (num > 0)
            {
                TreeNode[] nodes = new TreeNode[num];
                for (int i = 0; i < num; i++)
                {
                    nodes[i] = (TreeNode)serializationInfo.GetValue("children" + i, typeof(TreeNode));
                }
                this.Nodes.AddRange(nodes);
            }
        }

        private void DoCollapse(TreeView tv)
        {
            if ((this.State & 0x20) != 0)
            {
                TreeViewCancelEventArgs e = new TreeViewCancelEventArgs(this, false, TreeViewAction.Collapse);
                tv.OnBeforeCollapse(e);
                if (!e.Cancel)
                {
                    UnsafeNativeMethods.SendMessage(new HandleRef(tv, tv.Handle), 0x1102, 1, this.Handle);
                    tv.OnAfterCollapse(new TreeViewEventArgs(this));
                }
            }
        }

        public void EndEdit(bool cancel)
        {
            if (this.TreeView != null)
            {
                UnsafeNativeMethods.SendMessage(new HandleRef(this.TreeView, this.TreeView.Handle), 0x1116, cancel ? 1 : 0, 0);
            }
        }

        internal void EnsureCapacity(int num)
        {
            int num2 = num;
            if (num2 < 4)
            {
                num2 = 4;
            }
            if (this.children == null)
            {
                this.children = new TreeNode[num2];
            }
            else if ((this.childCount + num) > this.children.Length)
            {
                int num3 = this.childCount + num;
                if (num == 1)
                {
                    num3 = this.childCount * 2;
                }
                TreeNode[] destinationArray = new TreeNode[num3];
                Array.Copy(this.children, 0, destinationArray, 0, this.childCount);
                this.children = destinationArray;
            }
        }

        private void EnsureStateImageValue()
        {
            if ((this.treeView != null) && (this.treeView.CheckBoxes && (this.treeView.StateImageList != null)))
            {
                if (!string.IsNullOrEmpty(this.StateImageKey))
                {
                    this.StateImageIndex = this.Checked ? 1 : 0;
                    this.StateImageKey = this.treeView.StateImageList.Images.Keys[this.StateImageIndex];
                }
                else
                {
                    this.StateImageIndex = this.Checked ? 1 : 0;
                }
            }
        }

        public void EnsureVisible()
        {
            if (this.TreeView != null)
            {
                UnsafeNativeMethods.SendMessage(new HandleRef(this.TreeView, this.TreeView.Handle), 0x1114, 0, this.Handle);
            }
        }

        public void Expand()
        {
            TreeView tv = this.TreeView;
            if ((tv == null) || !tv.IsHandleCreated)
            {
                this.expandOnRealization = true;
            }
            else
            {
                this.ResetExpandedState(tv);
                if (!this.IsExpanded)
                {
                    UnsafeNativeMethods.SendMessage(new HandleRef(tv, tv.Handle), 0x1102, 2, this.Handle);
                }
                this.expandOnRealization = false;
            }
        }

        public void ExpandAll()
        {
            this.Expand();
            for (int i = 0; i < this.childCount; i++)
            {
                this.children[i].ExpandAll();
            }
        }

        internal TreeView FindTreeView()
        {
            TreeNode parent = this;
            while (parent.parent != null)
            {
                parent = parent.parent;
            }
            return parent.treeView;
        }

        public static TreeNode FromHandle(TreeView tree, IntPtr handle)
        {
            IntSecurity.ControlFromHandleOrLocation.Demand();
            return tree.NodeFromHandle(handle);
        }

        private void GetFullPath(StringBuilder path, string pathSeparator)
        {
            if (this.parent != null)
            {
                this.parent.GetFullPath(path, pathSeparator);
                if (this.parent.parent != null)
                {
                    path.Append(pathSeparator);
                }
                path.Append(this.text);
            }
        }

        public int GetNodeCount(bool includeSubTrees)
        {
            int childCount = this.childCount;
            if (includeSubTrees)
            {
                for (int i = 0; i < this.childCount; i++)
                {
                    childCount += this.children[i].GetNodeCount(true);
                }
            }
            return childCount;
        }

        internal void InsertNodeAt(int index, TreeNode node)
        {
            this.EnsureCapacity(1);
            node.parent = this;
            node.index = index;
            for (int i = this.childCount; i > index; i--)
            {
                TreeNode node2;
                this.children[i] = node2 = this.children[i - 1];
                node2.index = i;
            }
            this.children[index] = node;
            this.childCount++;
            node.Realize(false);
            if ((this.TreeView != null) && (node == this.TreeView.selectedNode))
            {
                this.TreeView.SelectedNode = node;
            }
        }

        private void InvalidateHostTree()
        {
            if ((this.treeView != null) && this.treeView.IsHandleCreated)
            {
                this.treeView.Invalidate();
            }
        }

        internal void Realize(bool insertFirst)
        {
            /*
            TreeView wrapper = this.TreeView;
            if ((wrapper != null) && wrapper.IsHandleCreated)
            {
                if (((this.parent == null) && !wrapper.IsUpdating()) && Application.RenderWithVisualStyles)
                {
                    wrapper.SendMessage(11, 0, 0);
                }
                try
                {
                    if (this.parent != null)
                    {
                        if (wrapper.InvokeRequired)
                        {
                            throw new InvalidOperationException(SR.GetString("InvalidCrossThreadControlCall"));
                        }
                        NativeMethods.TV_INSERTSTRUCT lParam = new NativeMethods.TV_INSERTSTRUCT();
                        lParam.item_mask = insertMask;
                        lParam.hParent = this.parent.handle;
                        TreeNode prevNode = this.PrevNode;
                        if (insertFirst || (prevNode == null))
                        {
                            lParam.hInsertAfter = (IntPtr)(-65535);
                        }
                        else
                        {
                            lParam.hInsertAfter = prevNode.handle;
                        }
                        lParam.item_pszText = Marshal.StringToHGlobalAuto(this.text);
                        lParam.item_iImage = (this.ImageIndexer.ActualIndex == -1) ? wrapper.ImageIndexer.ActualIndex : this.ImageIndexer.ActualIndex;
                        lParam.item_iSelectedImage = (this.SelectedImageIndexer.ActualIndex == -1) ? wrapper.SelectedImageIndexer.ActualIndex : this.SelectedImageIndexer.ActualIndex;
                        lParam.item_mask = 1;
                        lParam.item_stateMask = 0;
                        lParam.item_state = 0;
                        if (wrapper.CheckBoxes)
                        {
                            lParam.item_mask |= 8;
                            lParam.item_stateMask |= 0xf000;
                            lParam.item_state |= this.CheckedInternal ? 0x2000 : 0x1000;
                        }
                        else if ((wrapper.StateImageList != null) && (this.StateImageIndexer.ActualIndex >= 0))
                        {
                            lParam.item_mask |= 8;
                            lParam.item_stateMask = 0xf000;
                            lParam.item_state = (this.StateImageIndexer.ActualIndex + 1) << 12;
                        }
                        if (lParam.item_iImage >= 0)
                        {
                            lParam.item_mask |= 2;
                        }
                        if (lParam.item_iSelectedImage >= 0)
                        {
                            lParam.item_mask |= 0x20;
                        }
                        bool flag = false;
                        if (UnsafeNativeMethods.SendMessage(new HandleRef(this.TreeView, this.TreeView.Handle), 0x110f, 0, 0) != IntPtr.Zero)
                        {
                            flag = true;
                            UnsafeNativeMethods.SendMessage(new HandleRef(this.TreeView, this.TreeView.Handle), 0x1116, 0, 0);
                        }
                        this.handle = UnsafeNativeMethods.SendMessage(new HandleRef(this.TreeView, this.TreeView.Handle), NativeMethods.TVM_INSERTITEM, 0, ref lParam);
                        wrapper.nodeTable[this.handle] = this;
                        this.UpdateNode(4);
                        Marshal.FreeHGlobal(lParam.item_pszText);
                        if (flag)
                        {
                            UnsafeNativeMethods.PostMessage(new HandleRef(this.TreeView, this.TreeView.Handle), NativeMethods.TVM_EDITLABEL, IntPtr.Zero, this.handle);
                        }
                        SafeNativeMethods.InvalidateRect(new HandleRef(wrapper, wrapper.Handle), (NativeMethods.COMRECT)null, false);
                        if ((this.parent.nodesCleared && (insertFirst || (prevNode == null))) && !wrapper.Scrollable)
                        {
                            UnsafeNativeMethods.SendMessage(new HandleRef(this.TreeView, this.TreeView.Handle), 11, 1, 0);
                            this.nodesCleared = false;
                        }
                    }
                    for (int i = this.childCount - 1; i >= 0; i--)
                    {
                        this.children[i].Realize(true);
                    }
                    if (this.expandOnRealization)
                    {
                        this.Expand();
                    }
                    if (this.collapseOnRealization)
                    {
                        this.Collapse();
                    }
                }
                finally
                {
                    if ((!wrapper.IsUpdating() && (this.parent == null)) && Application.RenderWithVisualStyles)
                    {
                        wrapper.SendMessage(11, -1, 0);
                    }
                }
            }
             * */
        }

        public void Remove()
        {
            this.Remove(true);
        }

        internal void Remove(bool notify)
        {
            bool isExpanded = this.IsExpanded;
            for (int i = 0; i < this.childCount; i++)
            {
                this.children[i].Remove(false);
            }
            if (notify && (this.parent != null))
            {
                if (this.index == (this.parent.childCount - 1))
                {
                    this.parent.children[this.index] = null;
                }
                for (int j = this.index; j < (this.parent.childCount - 1); j++)
                {
                    TreeNode node;
                    this.parent.children[j] = node = this.parent.children[j + 1];
                    node.index = j;
                }
                this.parent.childCount--;
                this.parent = null;
            }
            this.expandOnRealization = isExpanded;
            if (this.TreeView != null)
            {
                if (this.handle != IntPtr.Zero)
                {
                    if (notify && this.TreeView.IsHandleCreated)
                    {
                        UnsafeNativeMethods.SendMessage(new HandleRef(this.TreeView, this.TreeView.Handle), 0x1101, 0, this.handle);
                    }
                    this.treeView.nodeTable.Remove(this.handle);
                    this.handle = IntPtr.Zero;
                }
                this.treeView = null;
            }
        }

        private void RemovePropBagIfEmpty()
        {
            if ((this.propBag != null) && this.propBag.IsEmpty())
            {
                this.propBag = null;
            }
        }

        private void ResetExpandedState(TreeView tv)
        {
            NativeMethods.TV_ITEM lParam = new NativeMethods.TV_ITEM();
            lParam.mask = 0x18;
            lParam.hItem = this.handle;
            lParam.stateMask = 0x40;
            lParam.state = 0;
            UnsafeNativeMethods.SendMessage(new HandleRef(tv, tv.Handle), NativeMethods.TVM_SETITEM, 0, ref lParam);
        }

        [SecurityPermission(SecurityAction.Demand, Flags = SecurityPermissionFlag.SerializationFormatter), SecurityPermission(SecurityAction.InheritanceDemand, Flags = SecurityPermissionFlag.SerializationFormatter)]
        protected virtual void Serialize(SerializationInfo si, StreamingContext context)
        {
            if (this.propBag != null)
            {
                si.AddValue("PropBag", this.propBag, typeof(OwnerDrawPropertyBag));
            }
            si.AddValue("Text", this.text);
            si.AddValue("Name", this.Name);
            si.AddValue("IsChecked", this.treeNodeState[1]);
            si.AddValue("ImageIndex", this.ImageIndexer.Index);
            si.AddValue("ImageKey", this.ImageIndexer.Key);
            si.AddValue("SelectedImageIndex", this.SelectedImageIndexer.Index);
            si.AddValue("SelectedImageKey", this.SelectedImageIndexer.Key);
            if ((this.treeView != null) && (this.treeView.StateImageList != null))
            {
                si.AddValue("StateImageIndex", this.StateImageIndexer.Index);
            }
            if ((this.treeView != null) && (this.treeView.StateImageList != null))
            {
                si.AddValue("StateImageKey", this.StateImageIndexer.Key);
            }
            si.AddValue("ChildCount", this.childCount);
            if (this.childCount > 0)
            {
                for (int i = 0; i < this.childCount; i++)
                {
                    si.AddValue("children" + i, this.children[i], typeof(TreeNode));
                }
            }
            if ((this.userData != null) && this.userData.GetType().IsSerializable)
            {
                si.AddValue("UserData", this.userData, this.userData.GetType());
            }
        }

        private bool ShouldSerializeBackColor()
        {
            return (this.BackColor != Color.Empty);
        }

        private bool ShouldSerializeForeColor()
        {
            return (this.ForeColor != Color.Empty);
        }

        private void SortChildren(TreeView parentTreeView)
        {
            if (this.childCount > 0)
            {
                TreeNode[] nodeArray = new TreeNode[this.childCount];
                if ((parentTreeView == null) || (parentTreeView.TreeViewNodeSorter == null))
                {
                    CompareInfo compareInfo = Application.CurrentCulture.CompareInfo;
                    for (int i = 0; i < this.childCount; i++)
                    {
                        int index = -1;
                        for (int j = 0; j < this.childCount; j++)
                        {
                            if (this.children[j] != null)
                            {
                                if (index == -1)
                                {
                                    index = j;
                                }
                                else if (compareInfo.Compare(this.children[j].Text, this.children[index].Text) <= 0)
                                {
                                    index = j;
                                }
                            }
                        }
                        nodeArray[i] = this.children[index];
                        this.children[index] = null;
                        nodeArray[i].index = i;
                        nodeArray[i].SortChildren(parentTreeView);
                    }
                    this.children = nodeArray;
                }
                else
                {
                    IComparer treeViewNodeSorter = parentTreeView.TreeViewNodeSorter;
                    for (int k = 0; k < this.childCount; k++)
                    {
                        int num5 = -1;
                        for (int m = 0; m < this.childCount; m++)
                        {
                            if (this.children[m] != null)
                            {
                                if (num5 == -1)
                                {
                                    num5 = m;
                                }
                                else if (treeViewNodeSorter.Compare(this.children[m], this.children[num5]) <= 0)
                                {
                                    num5 = m;
                                }
                            }
                        }
                        nodeArray[k] = this.children[num5];
                        this.children[num5] = null;
                        nodeArray[k].index = k;
                        nodeArray[k].SortChildren(parentTreeView);
                    }
                    this.children = nodeArray;
                }
            }
        }

        [SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter)]
        void ISerializable.GetObjectData(SerializationInfo si, StreamingContext context)
        {
            this.Serialize(si, context);
        }

        public void Toggle()
        {
            if (this.IsExpanded)
            {
                this.Collapse();
            }
            else
            {
                this.Expand();
            }
        }

        public override string ToString()
        {
            return ("TreeNode: " + ((this.text == null) ? "" : this.text));
        }

        internal void UpdateImage()
        {
            /*
            NativeMethods.TV_ITEM lParam = new NativeMethods.TV_ITEM();
            lParam.mask = 0x12;
            lParam.hItem = this.Handle;
            lParam.iImage = Math.Max(0, (this.ImageIndexer.ActualIndex >= this.TreeView.ImageList.Images.Count) ? (this.TreeView.ImageList.Images.Count - 1) : this.ImageIndexer.ActualIndex);
            UnsafeNativeMethods.SendMessage(new HandleRef(this.TreeView, this.TreeView.Handle), NativeMethods.TVM_SETITEM, 0, ref lParam);
             * */
        }

        private void UpdateNode(int mask)
        {
            if (this.handle != IntPtr.Zero)
            {
                /*
                TreeView wrapper = this.TreeView;
                NativeMethods.TV_ITEM lParam = new NativeMethods.TV_ITEM();
                lParam.mask = 0x10 | mask;
                lParam.hItem = this.handle;
                if ((mask & 1) != 0)
                {
                    lParam.pszText = Marshal.StringToHGlobalAuto(this.text);
                }
                if ((mask & 2) != 0)
                {
                    lParam.iImage = (this.ImageIndexer.ActualIndex == -1) ? wrapper.ImageIndexer.ActualIndex : this.ImageIndexer.ActualIndex;
                }
                if ((mask & 0x20) != 0)
                {
                    lParam.iSelectedImage = (this.SelectedImageIndexer.ActualIndex == -1) ? wrapper.SelectedImageIndexer.ActualIndex : this.SelectedImageIndexer.ActualIndex;
                }
                if ((mask & 8) != 0)
                {
                    lParam.stateMask = 0xf000;
                    if (this.StateImageIndexer.ActualIndex != -1)
                    {
                        lParam.state = (this.StateImageIndexer.ActualIndex + 1) << 12;
                    }
                }
                if ((mask & 4) != 0)
                {
                    lParam.lParam = this.handle;
                }
                //UnsafeNativeMethods.SendMessage(new HandleRef(wrapper, wrapper.Handle), NativeMethods.TVM_SETITEM, 0, ref lParam);
                if ((mask & 1) != 0)
                {
                    //Marshal.FreeHGlobal(lParam.pszText);
                    if (wrapper.Scrollable)
                    {
                        wrapper.ForceScrollbarUpdate(false);
                    }
                }
                 * */
            }
        }

        // Properties
        [Category("Appearance")]
        public Color BackColor
        {
            get
            {
                if (this.propBag == null)
                {
                    return Color.Empty;
                }
                return this.propBag.BackColor;
            }
            set
            {
                Color backColor = this.BackColor;
                if (value.IsEmpty)
                {
                    if (this.propBag != null)
                    {
                        this.propBag.BackColor = Color.Empty;
                        this.RemovePropBagIfEmpty();
                    }
                    if (!backColor.IsEmpty)
                    {
                        this.InvalidateHostTree();
                    }
                }
                else
                {
                    if (this.propBag == null)
                    {
                        this.propBag = new OwnerDrawPropertyBag();
                    }
                    this.propBag.BackColor = value;
                    if (!value.Equals(backColor))
                    {
                        this.InvalidateHostTree();
                    }
                }
            }
        }

        [Browsable(false)]
        public Rectangle Bounds
        {
            get
            {
                if (this.TreeView == null)
                {
                    return Rectangle.Empty;
                }
                NativeMethods.RECT lParam = new NativeMethods.RECT();
                (IntPtr) & lParam.left = this.Handle;
                if (((int)UnsafeNativeMethods.SendMessage(new HandleRef(this.TreeView, this.TreeView.Handle), 0x1104, 1, ref lParam)) == 0)
                {
                    return Rectangle.Empty;
                }
                return Rectangle.FromLTRB(lParam.left, lParam.top, lParam.right, lParam.bottom);
            }
        }

        [DefaultValue(false)]
        public bool Checked
        {
            get
            {
                return this.CheckedInternal;
            }
            set
            {
                TreeView treeView = this.TreeView;
                if (treeView != null)
                {
                    if (!treeView.TreeViewBeforeCheck(this, TreeViewAction.Unknown))
                    {
                        this.CheckedInternal = value;
                        treeView.TreeViewAfterCheck(this, TreeViewAction.Unknown);
                    }
                }
                else
                {
                    this.CheckedInternal = value;
                }
            }
        }

        internal bool CheckedInternal
        {
            get
            {
                return this.CheckedStateInternal;
            }
            set
            {
                this.CheckedStateInternal = value;
                if (this.handle != IntPtr.Zero)
                {
                    TreeView wrapper = this.TreeView;
                    if ((wrapper != null) && wrapper.IsHandleCreated)
                    {
                        NativeMethods.TV_ITEM lParam = new NativeMethods.TV_ITEM();
                        lParam.mask = 0x18;
                        lParam.hItem = this.handle;
                        lParam.stateMask = 0xf000;
                        lParam.state |= value ? 0x2000 : 0x1000;
                        UnsafeNativeMethods.SendMessage(new HandleRef(wrapper, wrapper.Handle), NativeMethods.TVM_SETITEM, 0, ref lParam);
                    }
                }
            }
        }

        internal bool CheckedStateInternal
        {
            get
            {
                return this.treeNodeState[1];
            }
            set
            {
                this.treeNodeState[1] = value;
            }
        }

        [DefaultValue((string)null)]
        public virtual ContextMenu ContextMenu
        {
            get
            {
                return this.contextMenu;
            }
            set
            {
                this.contextMenu = value;
            }
        }

        [DefaultValue((string)null)]
        public virtual ContextMenuStrip ContextMenuStrip
        {
            get
            {
                return this.contextMenuStrip;
            }
            set
            {
                this.contextMenuStrip = value;
            }
        }

        [Browsable(false)]
        public TreeNode FirstNode
        {
            get
            {
                if (this.childCount == 0)
                {
                    return null;
                }
                return this.children[0];
            }
        }

        private TreeNode FirstVisibleParent
        {
            get
            {
                TreeNode parent = this;
                while ((parent != null) && parent.Bounds.IsEmpty)
                {
                    parent = parent.Parent;
                }
                return parent;
            }
        }

        public Color ForeColor
        {
            get
            {
                if (this.propBag == null)
                {
                    return Color.Empty;
                }
                return this.propBag.ForeColor;
            }
            set
            {
                Color foreColor = this.ForeColor;
                if (value.IsEmpty)
                {
                    if (this.propBag != null)
                    {
                        this.propBag.ForeColor = Color.Empty;
                        this.RemovePropBagIfEmpty();
                    }
                    if (!foreColor.IsEmpty)
                    {
                        this.InvalidateHostTree();
                    }
                }
                else
                {
                    if (this.propBag == null)
                    {
                        this.propBag = new OwnerDrawPropertyBag();
                    }
                    this.propBag.ForeColor = value;
                    if (!value.Equals(foreColor))
                    {
                        this.InvalidateHostTree();
                    }
                }
            }
        }

        [Browsable(false)]
        public string FullPath
        {
            get
            {
                TreeView treeView = this.TreeView;
                if (treeView == null)
                {
                    throw new InvalidOperationException("TreeNodeNoParent");
                }
                StringBuilder path = new StringBuilder();
                this.GetFullPath(path, treeView.PathSeparator);
                return path.ToString();
            }
        }

        [Browsable(false)]
        public IntPtr Handle
        {
            get
            {
                if (this.handle == IntPtr.Zero)
                {
                    this.TreeView.CreateControl();
                }
                return this.handle;
            }
        }

        [TypeConverter(typeof(TreeViewImageIndexConverter)), Editor("System.Windows.Forms.Design.ImageIndexEditor, System.Design, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", typeof(UITypeEditor)), RefreshProperties(RefreshProperties.Repaint), DefaultValue(-1), RelatedImageList("TreeView.ImageList"), Localizable(true)]
        public int ImageIndex
        {
            get
            {
                return this.ImageIndexer.Index;
            }
            set
            {
                this.ImageIndexer.Index = value;
                this.UpdateNode(2);
            }
        }

        internal TreeNodeImageIndexer ImageIndexer
        {
            get
            {
                if (this.imageIndexer == null)
                {
                    this.imageIndexer = new TreeNodeImageIndexer(this, TreeNodeImageIndexer.ImageListType.Default);
                }
                return this.imageIndexer;
            }
        }

        [Editor("System.Windows.Forms.Design.ImageIndexEditor, System.Design, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", typeof(UITypeEditor)), TypeConverter(typeof(TreeViewImageKeyConverter)), DefaultValue(""), Localizable(true), RelatedImageList("TreeView.ImageList"), RefreshProperties(RefreshProperties.Repaint)]
        public string ImageKey
        {
            get
            {
                return this.ImageIndexer.Key;
            }
            set
            {
                this.ImageIndexer.Key = value;
                this.UpdateNode(2);
            }
        }

        public int Index
        {
            get
            {
                return this.index;
            }
        }

        [Browsable(false)]
        public bool IsEditing
        {
            get
            {
                TreeView treeView = this.TreeView;
                if (treeView != null)
                {
                    return (treeView.editNode == this);
                }
                return false;
            }
        }

        [Browsable(false)]
        public bool IsExpanded
        {
            get
            {
                if (this.handle == IntPtr.Zero)
                {
                    return this.expandOnRealization;
                }
                return ((this.State & 0x20) != 0);
            }
        }

        [Browsable(false)]
        public bool IsSelected
        {
            get
            {
                if (this.handle == IntPtr.Zero)
                {
                    return false;
                }
                return ((this.State & 2) != 0);
            }
        }

        [Browsable(false)]
        public bool IsVisible
        {
            get
            {
                if (this.handle == IntPtr.Zero)
                {
                    return false;
                }
                TreeView wrapper = this.TreeView;
                NativeMethods.RECT lParam = new NativeMethods.RECT();
                (IntPtr) & lParam.left = this.Handle;
                bool flag = ((int)UnsafeNativeMethods.SendMessage(new HandleRef(wrapper, wrapper.Handle), 0x1104, 1, ref lParam)) != 0;
                if (flag)
                {
                    Size clientSize = wrapper.ClientSize;
                    flag = (((lParam.bottom > 0) && (lParam.right > 0)) && (lParam.top < clientSize.Height)) && (lParam.left < clientSize.Width);
                }
                return flag;
            }
        }

        [Browsable(false)]
        public TreeNode LastNode
        {
            get
            {
                if (this.childCount == 0)
                {
                    return null;
                }
                return this.children[this.childCount - 1];
            }
        }

        [Browsable(false)]
        public int Level
        {
            get
            {
                if (this.Parent == null)
                {
                    return 0;
                }
                return (this.Parent.Level + 1);
            }
        }

        public string Name
        {
            get
            {
                if (this.name != null)
                {
                    return this.name;
                }
                return "";
            }
            set
            {
                this.name = value;
            }
        }

        [Browsable(false)]
        public TreeNode NextNode
        {
            get
            {
                if ((this.index + 1) < this.parent.Nodes.Count)
                {
                    return this.parent.Nodes[this.index + 1];
                }
                return null;
            }
        }

        [Browsable(false)]
        public TreeNode NextVisibleNode
        {
            get
            {
                if (this.TreeView != null)
                {
                    TreeNode firstVisibleParent = this.FirstVisibleParent;
                    if (firstVisibleParent != null)
                    {
                        IntPtr handle = UnsafeNativeMethods.SendMessage(new HandleRef(this.TreeView, this.TreeView.Handle), 0x110a, 6, firstVisibleParent.Handle);
                        if (handle != IntPtr.Zero)
                        {
                            return this.TreeView.NodeFromHandle(handle);
                        }
                    }
                }
                return null;
            }
        }

        [Localizable(true), DefaultValue((string)null)]
        public Font NodeFont
        {
            get
            {
                if (this.propBag == null)
                {
                    return null;
                }
                return this.propBag.Font;
            }
            set
            {
                Font nodeFont = this.NodeFont;
                if (value == null)
                {
                    if (this.propBag != null)
                    {
                        this.propBag.Font = null;
                        this.RemovePropBagIfEmpty();
                    }
                    if (nodeFont != null)
                    {
                        this.InvalidateHostTree();
                    }
                }
                else
                {
                    if (this.propBag == null)
                    {
                        this.propBag = new OwnerDrawPropertyBag();
                    }
                    this.propBag.Font = value;
                    if (!value.Equals(nodeFont))
                    {
                        this.InvalidateHostTree();
                    }
                }
            }
        }

        [Browsable(false), ListBindable(false)]
        public TreeNodeCollection Nodes
        {
            get
            {
                if (this.nodes == null)
                {
                    this.nodes = new TreeNodeCollection(this);
                }
                return this.nodes;
            }
        }

        [Browsable(false)]
        public TreeNode Parent
        {
            get
            {
                TreeView treeView = this.TreeView;
                if ((treeView != null) && (this.parent == treeView.root))
                {
                    return null;
                }
                return this.parent;
            }
        }

        [Browsable(false)]
        public TreeNode PrevNode
        {
            get
            {
                int index = this.index;
                int fixedIndex = this.parent.Nodes.FixedIndex;
                if (fixedIndex > 0)
                {
                    index = fixedIndex;
                }
                if ((index > 0) && (index <= this.parent.Nodes.Count))
                {
                    return this.parent.Nodes[index - 1];
                }
                return null;
            }
        }

        [Browsable(false)]
        public TreeNode PrevVisibleNode
        {
            get
            {
                TreeNode firstVisibleParent = this.FirstVisibleParent;
                if (firstVisibleParent != null)
                {
                    if (this.TreeView == null)
                    {
                        return null;
                    }
                    IntPtr handle = UnsafeNativeMethods.SendMessage(new HandleRef(this.TreeView, this.TreeView.Handle), 0x110a, 7, firstVisibleParent.Handle);
                    if (handle != IntPtr.Zero)
                    {
                        return this.TreeView.NodeFromHandle(handle);
                    }
                }
                return null;
            }
        }

        internal Rectangle RowBounds
        {
            get
            {
                NativeMethods.RECT lParam = new NativeMethods.RECT();
                (IntPtr) & lParam.left = this.Handle;
                if (this.TreeView == null)
                {
                    return Rectangle.Empty;
                }
                if (((int)UnsafeNativeMethods.SendMessage(new HandleRef(this.TreeView, this.TreeView.Handle), 0x1104, 0, ref lParam)) == 0)
                {
                    return Rectangle.Empty;
                }
                return Rectangle.FromLTRB(lParam.left, lParam.top, lParam.right, lParam.bottom);
            }
        }

        [DefaultValue(-1), TypeConverter(typeof(TreeViewImageIndexConverter)), Localizable(true), RefreshProperties(RefreshProperties.Repaint), Editor("System.Windows.Forms.Design.ImageIndexEditor, System.Design, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", typeof(UITypeEditor)), RelatedImageList("TreeView.ImageList")]
        public int SelectedImageIndex
        {
            get
            {
                return this.SelectedImageIndexer.Index;
            }
            set
            {
                this.SelectedImageIndexer.Index = value;
                this.UpdateNode(0x20);
            }
        }

        internal TreeNodeImageIndexer SelectedImageIndexer
        {
            get
            {
                if (this.selectedImageIndexer == null)
                {
                    this.selectedImageIndexer = new TreeNodeImageIndexer(this, TreeNodeImageIndexer.ImageListType.Default);
                }
                return this.selectedImageIndexer;
            }
        }

        [TypeConverter(typeof(TreeViewImageKeyConverter)), DefaultValue(""), RefreshProperties(RefreshProperties.Repaint), Editor("System.Windows.Forms.Design.ImageIndexEditor, System.Design, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", typeof(UITypeEditor)), RelatedImageList("TreeView.ImageList"), Localizable(true)]
        public string SelectedImageKey
        {
            get
            {
                return this.SelectedImageIndexer.Key;
            }
            set
            {
                this.SelectedImageIndexer.Key = value;
                this.UpdateNode(0x20);
            }
        }

        internal int State
        {
            get
            {
                if (this.handle == IntPtr.Zero)
                {
                    return 0;
                }
                if (this.TreeView == null)
                {
                    return 0;
                }
                NativeMethods.TV_ITEM lParam = new NativeMethods.TV_ITEM();
                lParam.hItem = this.Handle;
                lParam.mask = 0x18;
                lParam.stateMask = 0x22;
                UnsafeNativeMethods.SendMessage(new HandleRef(this.TreeView, this.TreeView.Handle), NativeMethods.TVM_GETITEM, 0, ref lParam);
                return lParam.state;
            }
        }

        [Editor("System.Windows.Forms.Design.ImageIndexEditor, System.Design, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", typeof(UITypeEditor)), TypeConverter(typeof(NoneExcludedImageIndexConverter)), Localizable(true), RefreshProperties(RefreshProperties.Repaint), RelatedImageList("TreeView.StateImageList"), DefaultValue(-1)]
        public int StateImageIndex
        {
            get
            {
                if ((this.treeView != null) && (this.treeView.StateImageList != null))
                {
                    return this.StateImageIndexer.Index;
                }
                return -1;
            }
            set
            {
                if ((value < -1) || (value > 14))
                {
                    throw new ArgumentOutOfRangeException("StateImageIndex", SR.GetString("InvalidArgument", new object[] { "StateImageIndex", value.ToString(CultureInfo.CurrentCulture) }));
                }
                this.StateImageIndexer.Index = value;
                if ((this.treeView != null) && !this.treeView.CheckBoxes)
                {
                    this.UpdateNode(8);
                }
            }
        }

        internal TreeNodeImageIndexer StateImageIndexer
        {
            get
            {
                if (this.stateImageIndexer == null)
                {
                    this.stateImageIndexer = new TreeNodeImageIndexer(this, TreeNodeImageIndexer.ImageListType.State);
                }
                return this.stateImageIndexer;
            }
        }

        [RefreshProperties(RefreshProperties.Repaint), Category("Behavior"), TypeConverter(typeof(System.Windows.Forms.ImageKeyConverter)), DefaultValue(""), Editor("System.Windows.Forms.Design.ImageIndexEditor, System.Design, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", typeof(UITypeEditor)), Localizable(true), System.Windows.Forms.RelatedImageList("TreeView.StateImageList")]
        public string StateImageKey
        {
            get
            {
                return this.StateImageIndexer.Key;
            }
            set
            {
                if (this.StateImageIndexer.Key != value)
                {
                    this.StateImageIndexer.Key = value;
                    if ((this.treeView != null) && !this.treeView.CheckBoxes)
                    {
                        this.UpdateNode(8);
                    }
                }
            }
        }

        [Localizable(false), Bindable(true), Category("Data"), DefaultValue((string)null), TypeConverter(typeof(StringConverter))]
        public object Tag
        {
            get
            {
                return this.userData;
            }
            set
            {
                this.userData = value;
            }
        }

        [Localizable(true), Category("Appearance")]
        public string Text
        {
            get
            {
                if (this.text != null)
                {
                    return this.text;
                }
                return "";
            }
            set
            {
                this.text = value;
                this.UpdateNode(1);
            }
        }

        [DefaultValue(""), Category("Appearance"), Localizable(false)]
        public string ToolTipText
        {
            get
            {
                return this.toolTipText;
            }
            set
            {
                this.toolTipText = value;
            }
        }

        [Browsable(false)]
        public TreeView TreeView
        {
            get
            {
                if (this.treeView == null)
                {
                    this.treeView = this.FindTreeView();
                }
                return this.treeView;
            }
        }
        // Nested Types
        internal class TreeNodeImageIndexer
        {
            // Fields
            private ImageListType imageListType;
            private TreeNode owner;
            // Fields
            private System.Windows.Forms.ImageList imageList;
            private int index = -1;
            private string key = string.Empty;
            private bool useIntegerIndex = true;

            // Properties
            public virtual int ActualIndex
            {
                get
                {
                    if (this.useIntegerIndex)
                    {
                        return this.Index;
                    }
                    if (this.ImageList != null)
                    {
                        return this.ImageList.Images.IndexOfKey(this.Key);
                    }
                    return -1;
                }
            }

            public virtual int Index
            {
                get
                {
                    return this.index;
                }
                set
                {
                    this.key = string.Empty;
                    this.index = value;
                    this.useIntegerIndex = true;
                }
            }

            public virtual string Key
            {
                get
                {
                    return this.key;
                }
                set
                {
                    this.index = -1;
                    this.key = (value == null) ? string.Empty : value;
                    this.useIntegerIndex = false;
                }
            }


            // Methods
            public TreeNodeImageIndexer(TreeNode node, ImageListType imageListType)
            {
                this.owner = node;
                this.imageListType = imageListType;
            }

            // Properties
            public System.Windows.Forms.ImageList ImageList
            {
                get
                {
                    if (this.owner.TreeView == null)
                    {
                        return null;
                    }
                    if (this.imageListType == ImageListType.State)
                    {
                        return this.owner.TreeView.StateImageList;
                    }
                    return this.owner.TreeView.ImageList;
                }
                set
                {
                }
            }
        }

        // Nested Types
        public enum ImageListType
        {
            Default,
            State
        }

        #region IWidget Members

        [Browsable(false), EditorBrowsable(EditorBrowsableState.Advanced)]\n\rpublic IRenderer Renderer
        {
            get { throw new Exception("The method or operation is not implemented."); }
        }

        #endregion
    }
}

