#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.ComponentModel;
using System.Windows.Forms;
using NetJsWire.JsonRpcHandler;
using NetJsWire.JsonRpcHandler.Attributes;
using NetJsWire.JsonRpcHandler.Util;
using NetJsWire.Widgets.Config;
using Newtonsoft.Json.Linq;

namespace NetJsWire.Widgets.Ui.Tree
{
    [ Renderer( typeof (Renderers.Tree.TreePanel) ) ]
    public partial class TreePanel : TreeView, IWidget
    {
        #region Delegates

        public delegate void NodeDropEventHandler( TreePanel tree, TreeNode node );

        #endregion

        private readonly IRenderer _renderer;
        private IConfigOptions _configOptions;

        private bool showRoot = true;

        //private TreeNode root;

        public TreePanel()
        {
            InitializeComponent();
            _renderer = Helper.GetRenderer( this );
            //Click += new EventHandler(TreePanel_Click);
            NodeDropped += TreePanel_NodeDropped;
        }


        public bool ShowRoot
        {
            get { return showRoot; }
            set { showRoot = value; }
        }

        public new bool Enabled
        {
            get { return base.Enabled; }
            set
            {
                base.Enabled = value;
                if ( value )
                    Renderer.AddCommand( "enable", null );
                else
                    Renderer.AddCommand( "disable", null );
            }
        }

        #region IWidget Members

        [ Browsable( false ), EditorBrowsable( EditorBrowsableState.Advanced ) ]
        public IRenderer Renderer
        {
            get { return _renderer; }
        }

        public void On( string name, string function )
        {
            Renderer.AddClientEvent( name, function );
        }

        public void AddClass( string cls )
        {
            Renderer.AddCommand( "addClass", cls );
        }

        public void RemoveClass( string cls )
        {
            Renderer.AddCommand( "removeClass", cls );
        }

        [ DefaultValue( false ) ]
        [ Description(
            "If the framework should generate a global Javascript variable that points to the Widget, the name of the variable will be the same as ClientID property of the Renderer."
            ) ]
        public bool GenerateClientMember { get; set; }

        [ RefreshProperties( RefreshProperties.All ),
          DesignerSerializationVisibility( DesignerSerializationVisibility.Content ), Category( "Config" ) ]
        public IConfigOptions ConfigOptions
        {
            get
            {
                if ( _configOptions == null )
                {
                    _configOptions = new PanelConfigOptions( this );
                }
                return _configOptions;
            }
        }

        #endregion

        public new event EventHandler Click;
        public event NodeDropEventHandler NodeDrop;
        private event EventHandler NodeDropped;

        private void TreePanel_NodeDropped( object sender, EventArgs e )
        {
            var data = ( JArray ) sender;
            var tpSender = WebApplication.Instance.Objects[ data[ 0 ].ToString() ] as TreePanel;
            var tnDropped = tpSender.Nodes[ 0 ].Nodes[ data[ 1 ].ToString() ];
            object o = NodeDrop;
            if ( o != null )
            {
                NodeDrop( tpSender, tnDropped );
            }
        }
    }
}